/* 
   ===================================================================
   Contents from file: before_setup.sql
   ===================================================================
*/
  
  
SET NAMES utf8;
SET @sql_log_bin = @@sql_log_bin;
SET sql_log_bin = 0;

CREATE DATABASE IF NOT EXISTS wbsys DEFAULT CHARACTER SET utf8;

USE wbsys;

CREATE OR REPLACE VIEW version AS SELECT '2.0.1' AS version;
/* 
   ===================================================================
   Contents from file: functions/extract_schema_from_file_name.sql
   ===================================================================
*/
  
  
/*
 * Function: extract_schema_from_file_name()
 * 
 * Takes a raw file path, and extracts the schema name from it
 *
 * Parameters
 *   path: The raw file name value to extract the schema name from
 */

DROP FUNCTION IF EXISTS extract_schema_from_file_name;

DELIMITER $$

CREATE FUNCTION extract_schema_from_file_name(path VARCHAR(512))
  RETURNS VARCHAR(512) DETERMINISTIC
  RETURN SUBSTRING_INDEX(SUBSTRING_INDEX(REPLACE(path, '\\', '/'), '/', -2), '/', 1)
$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: functions/extract_table_from_file_name.sql
   ===================================================================
*/
  
  
/*
 * Function: extract_table_from_file_name()
 * 
 * Takes a raw file path, and extracts the table name from it
 *
 * Parameters
 *   path: The raw file name value to extract the table name from
 */

DROP FUNCTION IF EXISTS extract_table_from_file_name;

DELIMITER $$

CREATE FUNCTION extract_table_from_file_name(path VARCHAR(512))
  RETURNS VARCHAR(512) DETERMINISTIC
  RETURN SUBSTRING_INDEX(REPLACE(SUBSTRING_INDEX(REPLACE(path, '\\', '/'), '/', -1), '@0024', '$'), '.', 1);
$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: functions/format_bytes.sql
   ===================================================================
*/
  
  
/*
 * Function: format_bytes()
 * 
 * Takes a raw bytes value, and converts it to a human readable form
 *
 * Parameters
 *   bytes: The raw bytes value to convert
 *
 * mysql> select format_bytes(2348723492723746);
 * +--------------------------------+
 * | format_bytes(2348723492723746) |
 * +--------------------------------+
 * | 2.09 PiB                       |
 * +--------------------------------+
 * 1 row in set (0.00 sec)
 * 
 * mysql> select format_bytes(2348723492723);
 * +-----------------------------+
 * | format_bytes(2348723492723) |
 * +-----------------------------+
 * | 2.14 TiB                    |
 * +-----------------------------+
 * 1 row in set (0.00 sec)
 * 
 * mysql> select format_bytes(23487234);
 * +------------------------+
 * | format_bytes(23487234) |
 * +------------------------+
 * | 22.40 MiB              |
 * +------------------------+
 * 1 row in set (0.00 sec)
 */

DROP FUNCTION IF EXISTS format_bytes;

DELIMITER $$

CREATE FUNCTION format_bytes(bytes BIGINT)
  RETURNS VARCHAR(16) DETERMINISTIC
BEGIN
  IF bytes IS NULL THEN RETURN NULL;
  ELSEIF bytes >= 1125899906842624 THEN RETURN CONCAT(ROUND(bytes / 1125899906842624, 2), ' PiB');
  ELSEIF bytes >= 1099511627776 THEN RETURN CONCAT(ROUND(bytes / 1099511627776, 2), ' TiB');
  ELSEIF bytes >= 1073741824 THEN RETURN CONCAT(ROUND(bytes / 1073741824, 2), ' GiB');
  ELSEIF bytes >= 1048576 THEN RETURN CONCAT(ROUND(bytes / 1048576, 2), ' MiB');
  ELSEIF bytes >= 1024 THEN RETURN CONCAT(ROUND(bytes / 1024, 2), ' KiB');
  ELSE RETURN CONCAT(bytes, ' bytes');
  END IF;
END $$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: functions/format_path.sql
   ===================================================================
*/
  
  
/*
 * Function: format_path()
 * 
 * Takes a raw path value, and strips out the datadir or tmpdir
 * replacing with @@datadir and @@tmpdir respectively. 
 *
 * Also normalizes the paths across operating systems, so backslashes
 * on Windows are converted to forward slashes
 *
 * Parameters
 *   path: The raw file path value to format
 *
 * mysql> select @@datadir;
 * +-----------------------------------------------+
 * | @@datadir                                     |
 * +-----------------------------------------------+
 * | /Users/mark/sandboxes/SmallTree/AMaster/data/ |
 * +-----------------------------------------------+
 * 1 row in set (0.06 sec)
 * 
 * mysql> select format_path('/Users/mark/sandboxes/SmallTree/AMaster/data/mysql/proc.MYD');
 * +----------------------------------------------------------------------------+
 * | format_path('/Users/mark/sandboxes/SmallTree/AMaster/data/mysql/proc.MYD') |
 * +----------------------------------------------------------------------------+
 * | @@datadir/mysql/proc.MYD                                                   |
 * +----------------------------------------------------------------------------+
 * 1 row in set (0.03 sec)
 */

DROP FUNCTION IF EXISTS format_path;

DELIMITER $$

CREATE FUNCTION format_path(path VARCHAR(260))
  RETURNS VARCHAR(260) CHARSET UTF8 DETERMINISTIC
BEGIN
  DECLARE v_path VARCHAR(260);

  /* OSX hides /private/ in variables, but Performance Schema does not */
  IF path LIKE '/private/%' 
    THEN SET v_path = REPLACE(path, '/private', '');
    ELSE SET v_path = path;
  END IF;

  IF v_path IS NULL THEN RETURN NULL;
  ELSEIF v_path LIKE CONCAT(@@global.datadir, '%') ESCAPE '|' THEN 
    RETURN REPLACE(REPLACE(REPLACE(v_path, @@global.datadir, '@@datadir/'), '\\\\', ''), '\\', '/');
  ELSEIF v_path LIKE CONCAT(@@global.tmpdir, '%') ESCAPE '|' THEN 
    RETURN REPLACE(REPLACE(REPLACE(v_path, @@global.tmpdir, '@@tmpdir/'), '\\\\', ''), '\\', '/');
  ELSE RETURN v_path;
  END IF;
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: functions/format_statement.sql
   ===================================================================
*/
  
  
/*
 * Function: format_statement()
 * 
 * Formats a normalized statement with a truncated string if > 64 characters long
 *
 * Parameters
 *   filename: The raw file name value to extract the table name from
 */

DROP FUNCTION IF EXISTS format_statement;

DELIMITER $$

CREATE FUNCTION format_statement(statement LONGTEXT)
  RETURNS VARCHAR(65) DETERMINISTIC
BEGIN
  IF LENGTH(statement) > 64 THEN RETURN REPLACE(CONCAT(LEFT(statement, 30), ' ... ', RIGHT(statement, 30)), '\n', ' ');
  ELSE RETURN REPLACE(statement, '\n', ' ');
  END IF;
END $$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: functions/format_time.sql
   ===================================================================
*/
  
  
/*
 * Function: format_time()
 * 
 * Takes a raw picoseconds value, and converts it to a human readable form.
 * Picoseconds are the precision that all latency values are printed in 
 * within MySQL's Performance Schema.
 *
 * Parameters
 *   picoseconds: The raw picoseconds value to convert
 *
 * mysql> select format_time(342342342342345);
 * +------------------------------+
 * | format_time(342342342342345) |
 * +------------------------------+
 * | 00:05:42                     |
 * +------------------------------+
 * 1 row in set (0.00 sec)
 * 
 * mysql> select format_time(342342342);
 * +------------------------+
 * | format_time(342342342) |
 * +------------------------+
 * | 342.34 µs              |
 * +------------------------+
 * 1 row in set (0.00 sec)
 * 
 * mysql> select format_time(34234);
 * +--------------------+
 * | format_time(34234) |
 * +--------------------+
 * | 34.23 ns           |
 * +--------------------+
 * 1 row in set (0.00 sec)
 * 
 * mysql> select format_time(342);
 * +------------------+
 * | format_time(342) |
 * +------------------+
 * | 342 ps           |
 * +------------------+
 * 1 row in set (0.00 sec)
 */

DROP FUNCTION IF EXISTS format_time;

DELIMITER $$

CREATE FUNCTION format_time(picoseconds BIGINT UNSIGNED)
  RETURNS VARCHAR(16) CHARSET UTF8 DETERMINISTIC
BEGIN
  IF picoseconds IS NULL THEN RETURN NULL;
  ELSEIF picoseconds >= 3600000000000000 THEN RETURN CONCAT(ROUND(picoseconds / 3600000000000000, 2), 'h');
  ELSEIF picoseconds >= 60000000000000 THEN RETURN SEC_TO_TIME(ROUND(picoseconds / 1000000000000, 2));
  ELSEIF picoseconds >= 1000000000000 THEN RETURN CONCAT(ROUND(picoseconds / 1000000000000, 2), ' s');
  ELSEIF picoseconds >= 1000000000 THEN RETURN CONCAT(ROUND(picoseconds / 1000000000, 2), ' ms');
  ELSEIF picoseconds >= 1000000 THEN RETURN CONCAT(ROUND(picoseconds / 1000000, 2), ' us');
  ELSEIF picoseconds >= 1000 THEN RETURN CONCAT(ROUND(picoseconds / 1000, 2), ' ns');
  ELSE RETURN CONCAT(picoseconds, ' ps');
  END IF;
END $$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: functions/is_account_enabled.sql
   ===================================================================
*/
  
  
/*
 * Function: is_account_enabled()
 * 
 * Determines whether instrumentation of an account is enabled.
 *
 * Parameters
 *   in_host .....: The hostname of the account to check.
 *   in_user .....: The username of the account to check.
 *   
 * Returns
 *   An enum whether the account is enabled or not.
 *
 * mysql> SELECT is_account_enabled('localhost', 'root');
 * +-----------------------------------------+
 * | is_account_enabled('localhost', 'root') |
 * +-----------------------------------------+
 * | YES                                     |
 * +-----------------------------------------+
 * 1 row in set (0.00 sec)
 *
 * Contributed by Jesper Krogh of MySQL Support @ Oracle
 */

DROP FUNCTION IF EXISTS is_account_enabled;

DELIMITER $$

CREATE DEFINER='root'@'localhost' FUNCTION is_account_enabled(in_host VARCHAR(60), in_user VARCHAR(16)) RETURNS enum('YES','NO', 'PARTIAL')
  COMMENT 'Returns whether a user account is enabled.'
  LANGUAGE SQL
  DETERMINISTIC 
  READS SQL DATA 
  SQL SECURITY INVOKER
BEGIN
    RETURN IF(EXISTS(SELECT 1
                       FROM performance_schema.setup_actors
                      WHERE     (`HOST` = '%' OR `HOST` = in_host)
                            AND (`USER` = '%' OR `USER` = in_user)
                    ),
              'YES', 'NO'
           );
END $$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: functions/reverse_format_time.sql
   ===================================================================
*/
  
  
/*
 * Function: reverse_format_time()
 * 
 * Takes a value from the format_time() function, and reverses it 
 * to raw values. 
 *
 * This is useful for cases where you want to use a view that exposes
 * formatted values, but would like to ORDER BY some other column within
 * the result set. 
 * 
 * NOTE: However, this does mean that some precision may be lost from 
 *       the original values, as when we convert using format_time we
 *       ROUND to two decimal places
 *
 * Parameters
 *   time_string: The value that is formatted by the format_time() function
 *
 * Example to show precision lost:
 * 
 * mysql> SELECT event_name AS event,
 *     ->        format_time(sum_timer_wait) AS total_latency,
 *     ->        reverse_format_time(format_time(sum_timer_wait)) AS reverse_format,
 *     ->        sum_timer_wait
 *     ->   FROM performance_schema.events_waits_summary_global_by_event_name
 *     ->  WHERE event_name != 'idle'
 *     ->  ORDER BY sum_timer_wait DESC LIMIT 5;
 * +--------------------------------------+---------------+----------------+----------------+
 * | event                                | total_latency | reverse_format | sum_timer_wait |
 * +--------------------------------------+---------------+----------------+----------------+
 * | wait/io/file/sql/file_parser         | 7.97 s        |  8000000000000 |  7974907810740 |
 * | wait/io/file/innodb/innodb_data_file | 848.17 ms     |   848000000000 |   848170566100 |
 * | wait/io/file/sql/FRM                 | 501.60 ms     |   502000000000 |   501604256790 |
 * | wait/io/file/innodb/innodb_log_file  | 54.30 ms      |    54000000000 |    54298899070 |
 * | wait/io/file/myisam/kfile            | 32.51 ms      |    33000000000 |    32512834380 |
 * +--------------------------------------+---------------+----------------+----------------+
 * 5 rows in set (0.01 sec)
 *
 * Example to show use against a wbsys view:
 *
 * mysql> SELECT event_name, total_latency, avg_latency FROM top_global_io_consumers_by_latency;
 * +-------------------------+---------------+-------------+
 * | event_name              | total_latency | avg_latency |
 * +-------------------------+---------------+-------------+
 * | sql/file_parser         | 7.97 s        | 5.70 ms     |
 * | innodb/innodb_data_file | 848.17 ms     | 4.35 ms     |
 * | sql/FRM                 | 502.21 ms     | 275.49 us   |
 * | innodb/innodb_log_file  | 54.30 ms      | 2.71 ms     |
 * ...
 * | archive/data            | 9.73 us       | 9.73 us     |
 * +-------------------------+---------------+-------------+
 * 15 rows in set (0.01 sec)
 * 
 * mysql> SELECT event_name, total_latency, avg_latency FROM top_global_io_consumers_by_latency
 *     ->  ORDER BY reverse_format_time(avg_latency) DESC;
 * +-------------------------+---------------+-------------+
 * | event_name              | total_latency | avg_latency |
 * +-------------------------+---------------+-------------+
 * | mysys/charset           | 24.24 ms      | 8.08 ms     |
 * | sql/file_parser         | 7.97 s        | 5.70 ms     |
 * | innodb/innodb_data_file | 848.17 ms     | 4.35 ms     |
 * | sql/ERRMSG              | 20.43 ms      | 4.09 ms     |
 * ...
 * | sql/global_ddl_log      | 14.60 us      | 7.30 us     |
 * +-------------------------+---------------+-------------+
 * 15 rows in set (0.01 sec)
 */

DROP FUNCTION IF EXISTS reverse_format_time;

DELIMITER $$

CREATE FUNCTION reverse_format_time(time_string VARCHAR(32))
  RETURNS BIGINT
  DETERMINISTIC
BEGIN
  IF time_string IS NULL THEN RETURN NULL;
  ELSEIF time_string LIKE '% ps' THEN RETURN CAST(LEFT(time_string, LENGTH(time_string) - 3) AS DECIMAL);
  ELSEIF time_string LIKE '% ns' THEN RETURN CAST(LEFT(time_string, LENGTH(time_string) - 3) AS DECIMAL) * 1000;
  ELSEIF time_string LIKE '% us' THEN RETURN CAST(LEFT(time_string, LENGTH(time_string) - 3) AS DECIMAL) * 1000000;
  ELSEIF time_string LIKE '% ms' THEN RETURN CAST(LEFT(time_string, LENGTH(time_string) - 3) AS DECIMAL) * 1000000000;
  ELSEIF time_string LIKE '% s' THEN RETURN CAST(LEFT(time_string, LENGTH(time_string) - 2) AS DECIMAL) * 1000000000000;
  ELSEIF time_string LIKE '% h' THEN RETURN CAST(LEFT(time_string, LENGTH(time_string) - 2) AS DECIMAL) * 3600000000000000;
  ELSE RETURN TIME_TO_SEC(time_string) * 1000000000000;
  END IF;
END $$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/analyze_statement_digest.sql
   ===================================================================
*/
  
  
/* 
 * Procedure: analyze_statement_digest()
 *
 * Parameters
 *   in_digest:      The statement digest identifier you would like to analyze
 *   in_runtime:     The number of seconds to run analysis for (defaults to a minute)
 *   in_interval:    The interval (in seconds, may be fractional) at which to try
 *                   and take snapshots (defaults to a second)
 *   in_start_fresh: Whether to TRUNCATE the events_statements_history_long and
 *                   events_stages_history_long tables before starting (default false)
 *   in_auto_enable: Whether to automatically turn on required consumers (default false)
 *
 * mysql> call analyze_statement_digest('891ec6860f98ba46d89dd20b0c03652c', 10, 0.1, true, true);
 * +--------------------+
 * | SUMMARY STATISTICS |
 * +--------------------+
 * | SUMMARY STATISTICS |
 * +--------------------+
 * 1 row in set (9.11 sec)
 * 
 * +------------+-----------+-----------+-----------+---------------+------------+------------+
 * | executions | exec_time | lock_time | rows_sent | rows_examined | tmp_tables | full_scans |
 * +------------+-----------+-----------+-----------+---------------+------------+------------+
 * |         21 | 4.11 ms   | 2.00 ms   |         0 |            21 |          0 |          0 |
 * +------------+-----------+-----------+-----------+---------------+------------+------------+
 * 1 row in set (9.11 sec)
 * 
 * +------------------------------------------+-------+-----------+
 * | event_name                               | count | latency   |
 * +------------------------------------------+-------+-----------+
 * | stage/sql/checking query cache for query |    16 | 724.37 µs |
 * | stage/sql/statistics                     |    16 | 546.92 µs |
 * | stage/sql/freeing items                  |    18 | 520.11 µs |
 * | stage/sql/init                           |    51 | 466.80 µs |
 * | stage/sql/Waiting for query cache lock   |    17 | 460.18 µs |
 * | stage/sql/Sending data                   |    16 | 164.54 µs |
 * | stage/sql/Opening tables                 |    18 | 162.22 µs |
 * | stage/sql/optimizing                     |    16 | 101.64 µs |
 * | stage/sql/updating                       |     1 | 75.48 µs  |
 * | stage/sql/System lock                    |    17 | 68.86 µs  |
 * | stage/sql/preparing                      |    16 | 62.90 µs  |
 * | stage/sql/closing tables                 |    18 | 37.08 µs  |
 * | stage/sql/query end                      |    18 | 22.51 µs  |
 * | stage/sql/checking permissions           |    17 | 20.86 µs  |
 * | stage/sql/end                            |    18 | 15.56 µs  |
 * | stage/sql/cleaning up                    |    18 | 11.92 µs  |
 * | stage/sql/executing                      |    16 | 6.95 µs   |
 * ------------------------------------------+-------+-----------+
 * 17 rows in set (9.12 sec)
 * 
 * +---------------------------+
 * | LONGEST RUNNING STATEMENT |
 * +---------------------------+
 * | LONGEST RUNNING STATEMENT |
 * +---------------------------+
 * 1 row in set (9.16 sec)
 * 
 * +-----------+-----------+-----------+-----------+---------------+------------+-----------+
 * | thread_id | exec_time | lock_time | rows_sent | rows_examined | tmp_tables | full_scan |
 * +-----------+-----------+-----------+-----------+---------------+------------+-----------+
 * |    166646 | 618.43 µs | 1.00 ms   |         0 |             1 |          0 |         0 |
 * +-----------+-----------+-----------+-----------+---------------+------------+-----------+
 * 1 row in set (9.16 sec)
 * 
 * // Truncated for clarity...
 * +-----------------------------------------------------------------+
 * | sql_text                                                        |
 * +-----------------------------------------------------------------+
 * | select hibeventhe0_.id as id1382_, hibeventhe0_.createdTime ... |
 * +-----------------------------------------------------------------+
 * 1 row in set (9.17 sec)
 * 
 * +------------------------------------------+-----------+
 * | event_name                               | latency   |
 * +------------------------------------------+-----------+
 * | stage/sql/init                           | 8.61 µs   |
 * | stage/sql/Waiting for query cache lock   | 453.23 µs |
 * | stage/sql/init                           | 331.07 ns |
 * | stage/sql/checking query cache for query | 43.04 µs  |
 * | stage/sql/checking permissions           | 1.32 µs   |
 * | stage/sql/Opening tables                 | 8.61 µs   |
 * | stage/sql/init                           | 16.22 µs  |
 * | stage/sql/System lock                    | 2.98 µs   |
 * | stage/sql/optimizing                     | 5.63 µs   |
 * | stage/sql/statistics                     | 30.13 µs  |
 * | stage/sql/preparing                      | 3.31 µs   |
 * | stage/sql/executing                      | 331.07 ns |
 * | stage/sql/Sending data                   | 9.60 µs   |
 * | stage/sql/end                            | 662.13 ns |
 * | stage/sql/query end                      | 993.20 ns |
 * | stage/sql/closing tables                 | 1.66 µs   |
 * | stage/sql/freeing items                  | 30.46 µs  |
 * | stage/sql/cleaning up                    | 662.13 ns |
 * +------------------------------------------+-----------+
 * 18 rows in set (9.23 sec)
 * 
 * +----+-------------+--------------+-------+---------------+-----------+---------+-------------+------+-------+
 * | id | select_type | table        | type  | possible_keys | key       | key_len | ref         | rows | Extra |
 * +----+-------------+--------------+-------+---------------+-----------+---------+-------------+------+-------+
 * |  1 | SIMPLE      | hibeventhe0_ | const | fixedTime     | fixedTime | 775     | const,const |    1 | NULL  |
 * +----+-------------+--------------+-------+---------------+-----------+---------+-------------+------+-------+
 * 1 row in set (9.27 sec)
 * 
 * Query OK, 0 rows affected (9.28 sec)
 * 
 * Versions: 5.6.3+
 */

DROP PROCEDURE IF EXISTS analyze_statement_digest;

DELIMITER $$

CREATE PROCEDURE analyze_statement_digest(IN in_digest VARCHAR(32), IN in_runtime INT, 
    IN in_interval DECIMAL(2,2), IN in_start_fresh BOOLEAN, IN in_auto_enable BOOLEAN)
    COMMENT 'Parameters: in_digest (varchar(32)), in_runtime (int), in_interval (decimal(2,2)), in_start_fresh (boolean), in_auto_enable (boolean)'
    SQL SECURITY INVOKER 
BEGIN

    DECLARE v_start_fresh BOOLEAN DEFAULT false;
    DECLARE v_auto_enable BOOLEAN DEFAULT false;
    DECLARE v_runtime INT DEFAULT 0;
    DECLARE v_start INT DEFAULT 0;
    DECLARE v_found_stmts INT;

    /* Do not track the current thread, it will kill the stack */
    CALL disable_current_thread();

    DROP TEMPORARY TABLE IF EXISTS stmt_trace;
    CREATE TEMPORARY TABLE stmt_trace (
        thread_id BIGINT,
        timer_start BIGINT,
        event_id BIGINT,
        sql_text longtext,
        timer_wait BIGINT,
        lock_time BIGINT,
        errors BIGINT,
        mysql_errno BIGINT,
        rows_sent BIGINT,
        rows_affected BIGINT,
        rows_examined BIGINT,
        created_tmp_tables BIGINT,
        created_tmp_disk_tables BIGINT,
        no_index_used BIGINT,
        PRIMARY KEY (thread_id, timer_start)
    );

    DROP TEMPORARY TABLE IF EXISTS stmt_stages;
    CREATE TEMPORARY TABLE stmt_stages (
       event_id BIGINT,
       stmt_id BIGINT,
       event_name VARCHAR(128),
       timer_wait BIGINT,
       PRIMARY KEY (event_id)
    );

    SET v_start_fresh = in_start_fresh;
    IF v_start_fresh THEN
        TRUNCATE TABLE performance_schema.events_statements_history_long;
        TRUNCATE TABLE performance_schema.events_stages_history_long;
    END IF;

    SET v_auto_enable = in_auto_enable;
    IF v_auto_enable THEN
        CALL wbsys.save_current_config();
    END IF;

    WHILE v_runtime < in_runtime DO
        SELECT UNIX_TIMESTAMP() INTO v_start;

        INSERT IGNORE INTO stmt_trace
        SELECT thread_id, timer_start, event_id, sql_text, timer_wait, lock_time, errors, mysql_errno, 
               rows_sent, rows_affected, rows_examined, created_tmp_tables, created_tmp_disk_tables, no_index_used
          FROM performance_schema.events_statements_history_long
        WHERE digest = in_digest;

        INSERT IGNORE INTO stmt_stages
        SELECT stages.event_id, stmt_trace.event_id,
               stages.event_name, stages.timer_wait
          FROM performance_schema.events_stages_history_long AS stages
          JOIN stmt_trace ON stages.nesting_event_id = stmt_trace.event_id;

        SELECT SLEEP(in_interval) INTO @sleep;
        SET v_runtime = v_runtime + (UNIX_TIMESTAMP() - v_start);
    END WHILE;

    IF v_auto_enable THEN
        CALL wbsys.reload_saved_config();
    END IF;

    SELECT "SUMMARY STATISTICS";

    SELECT COUNT(*) executions,
           format_time(SUM(timer_wait)) AS exec_time,
           format_time(SUM(lock_time)) AS lock_time,
           SUM(rows_sent) AS rows_sent,
           SUM(rows_affected) AS rows_affected,
           SUM(rows_examined) AS rows_examined,
           SUM(created_tmp_tables) AS tmp_tables,
           SUM(no_index_used) AS full_scans
      FROM stmt_trace;

    SELECT event_name,
           COUNT(*) as count,
           format_time(SUM(timer_wait)) as latency
      FROM stmt_stages
     GROUP BY event_name
     ORDER BY SUM(timer_wait) DESC;

    SELECT "LONGEST RUNNING STATEMENT";

    SELECT thread_id,
           format_time(timer_wait) AS exec_time,
           format_time(lock_time) AS lock_time,
           rows_sent,
           rows_affected,
           rows_examined,
           created_tmp_tables AS tmp_tables,
           no_index_used AS full_scan
      FROM stmt_trace
     ORDER BY timer_wait DESC LIMIT 1;

    SELECT sql_text
      FROM stmt_trace
     ORDER BY timer_wait DESC LIMIT 1;

    SELECT sql_text, event_id INTO @sql, @sql_id
      FROM stmt_trace
    ORDER BY timer_wait DESC LIMIT 1;

    SELECT event_name,
           format_time(timer_wait) as latency
      FROM stmt_stages
     WHERE stmt_id = @sql_id
     ORDER BY event_id;

    DROP TEMPORARY TABLE stmt_trace;
    DROP TEMPORARY TABLE stmt_stages;

    SET @stmt := CONCAT("EXPLAIN ", @sql);
    PREPARE explain_stmt FROM @stmt;
    EXECUTE explain_stmt;
    DEALLOCATE PREPARE explain_stmt;

END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/currently_enabled.sql
   ===================================================================
*/
  
  
/*
 * Procedure: currently_enabled()
 *
 * Show all enabled events / consumers
 *
 * Parameters
 *   show_instruments: Whether to show instrument configuration as well
 *   show_threads: Whether to show threads that are currently enabled
 *
 * Versions: 5.5+
 *
 * mysql> call currently_enabled(TRUE, TRUE);
 * +----------------------------+
 * | performance_schema_enabled |
 * +----------------------------+
 * |                          1 |
 * +----------------------------+
 * 1 row in set (0.00 sec)
 * 
 * +---------------+
 * | enabled_users |
 * +---------------+
 * | '%'@'%'       |
 * +---------------+
 * 1 row in set (0.01 sec)
 * 
 * +----------------------+---------+-------+
 * | objects              | enabled | timed |
 * +----------------------+---------+-------+
 * | mysql.%              | NO      | NO    |
 * | performance_schema.% | NO      | NO    |
 * | information_schema.% | NO      | NO    |
 * | %.%                  | YES     | YES   |
 * +----------------------+---------+-------+
 * 4 rows in set (0.01 sec)
 * 
 * +---------------------------+
 * | enabled_consumers         |
 * +---------------------------+
 * | events_statements_current |
 * | global_instrumentation    |
 * | thread_instrumentation    |
 * | statements_digest         |
 * +---------------------------+
 * 4 rows in set (0.05 sec)
 * 
 * +--------------------------+-------------+
 * | enabled_threads          | thread_type |
 * +--------------------------+-------------+
 * | innodb/srv_master_thread | BACKGROUND  |
 * | root@localhost           | FOREGROUND  |
 * | root@localhost           | FOREGROUND  |
 * | root@localhost           | FOREGROUND  |
 * | root@localhost           | FOREGROUND  |
 * +--------------------------+-------------+
 * 5 rows in set (0.03 sec)
 * 
 * +-------------------------------------+-------+
 * | enabled_instruments                 | timed |
 * +-------------------------------------+-------+
 * | wait/io/file/sql/map                | YES   |
 * | wait/io/file/sql/binlog             | YES   |
 * ...
 * | statement/com/Error                 | YES   |
 * | statement/com/                      | YES   |
 * | idle                                | YES   |
 * +-------------------------------------+-------+
 * 210 rows in set (0.08 sec)
 * 
 * Query OK, 0 rows affected (0.89 sec)
 * 
 */

DROP PROCEDURE IF EXISTS currently_enabled;

DELIMITER $$

CREATE PROCEDURE currently_enabled(IN show_instruments BOOLEAN, IN show_threads BOOLEAN)
    COMMENT 'Parameters: show_instruments (boolean), show_threads (boolean)'
    SQL SECURITY INVOKER 
BEGIN
    SELECT @@performance_schema AS performance_schema_enabled;

    SELECT CONCAT('\'', host, '\'@\'', user, '\'') AS enabled_users
      FROM performance_schema.setup_actors;

    SELECT CONCAT(object_schema, '.', object_name) AS objects,
           enabled,
           timed
      FROM performance_schema.setup_objects;

    SELECT name AS enabled_consumers
      FROM performance_schema.setup_consumers
     WHERE enabled = 'YES';

    IF (show_threads) THEN
        SELECT IF(name = 'thread/sql/one_connection', 
                  CONCAT(processlist_user, '@', processlist_host), 
                  REPLACE(name, 'thread/', '')) AS enabled_threads,
        TYPE AS thread_type
          FROM performance_schema.threads
         WHERE INSTRUMENTED = 'YES';
    END IF;

    IF (show_instruments) THEN
        SELECT name AS enabled_instruments,
               timed
          FROM performance_schema.setup_instruments
         WHERE enabled = 'YES';
    END IF;
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/disable_background_threads.sql
   ===================================================================
*/
  
  
/*
 * Procedure: disable_background_threads()
 *
 * Disable performance_schema instrumentation for all background threads
 *
 * Example: CALL disable_background_threads();
 *
 * Versions: 5.6.2+
 */

DROP PROCEDURE IF EXISTS disable_background_threads;

DELIMITER $$

CREATE PROCEDURE disable_background_threads()
    SQL SECURITY INVOKER 
BEGIN
    UPDATE performance_schema.threads
       SET instrumented = 'NO'
     WHERE type = 'BACKGROUND';
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/disable_current_thread.sql
   ===================================================================
*/
  
  
/*
 * Procedure: disable_current_thread()
 *
 * Disable performance_schema instrumentation for the current thread
 *
 * Example: CALL disable_current_thread();
 *
 * Versions: 5.6.2+
 */

DROP PROCEDURE IF EXISTS disable_current_thread;

DELIMITER $$

CREATE PROCEDURE disable_current_thread()
    SQL SECURITY INVOKER 
BEGIN
    UPDATE performance_schema.threads
       SET instrumented = 'NO'
     WHERE processlist_id = CONNECTION_ID();
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/dump_thread_stack.sql
   ===================================================================
*/
  
  
/* 
 * Procedure: dump_thread_stack()
 *
 * Dumps all data within performance_schema for an instrumented thread
 * to create a DOT formatted graph file. 
 * Each resultset returned from the procedure should be used for a complete graph
 *
 * Example command line:
 *    CALL wbsys.dump_thread_stack(218, '/tmp/stack.dot', 10.00, 1.0, TRUE, TRUE, FALSE)
 * 
 * After this dot file is generated, load in to a graphing package such as Graphviz
 *
 * Parameters
 *   in_thread_id  : The thread that you would like a stack trace for
 *   in_outfile    : The filename the dot file will be written to
 *   in_max_runtime: The maximum time to keep collecting data. Use NULL to get the default which is 60 seconds.
 *   in_interval   : How long to sleep between data collections. Use NULL to get the default which is 1 second.
 *   in_start_fresh: Whether to reset all P_S data.
 *   in_auto_setup : Whether to disable all other threads and enable all consumers/instruments. This will also reset
 *                   the settings at the end of the run.
 *   in_debug      : Whether you would like to include file:lineno in the graph
 *
 * Versions: 5.6.3+
 * 
 * Parts contributed by Jesper Krogh of MySQL Support @ Oracle
 */


DROP PROCEDURE IF EXISTS dump_thread_stack;

DELIMITER $$
CREATE PROCEDURE dump_thread_stack(IN in_thread_id INT, IN in_outfile varchar(255), IN in_max_runtime DECIMAL(20,2),
    IN in_interval DECIMAL(20,2), IN in_start_fresh BOOLEAN, IN in_auto_setup BOOLEAN, IN in_debug BOOLEAN)
    COMMENT 'Parameters: in_thread_id (int), in_outfile (varchar(255)), in_max_runtime (decumal(20,2) - default 60 seconds), in_interval (DECIMAL(20,2) - default 1 second), in_start_fresh (boolean), in_auto_setup(boolean), in_debug (boolean)'
BEGIN
    DECLARE v_done bool DEFAULT FALSE;
    DECLARE v_start, v_runtime DECIMAL(20,2) DEFAULT 0.0;
    DECLARE v_min_event_id bigint unsigned DEFAULT 0;
    DECLARE v_event longtext;
    DECLARE c_stack CURSOR FOR
        SELECT CONCAT(IF(nesting_event_id IS NOT NULL, CONCAT(nesting_event_id, ' -> '), ''), 
                    event_id, '; ', event_id, ' [label="',
                    /* Convert from picoseconds to microseconds */
                    '(', wbsys.format_time(timer_wait), ') ',
                    IF (event_name NOT LIKE 'wait/io%', 
                        SUBSTRING_INDEX(event_name, '/', -2), 
                        IF (event_name NOT LIKE 'wait/io/file%' OR event_name NOT LIKE 'wait/io/socket%',
                            SUBSTRING_INDEX(event_name, '/', -4),
                            event_name)
                        ),
                    /* Always dump the extra wait information gathered for statements */
                    IF (event_name LIKE 'statement/%', IFNULL(CONCAT('\\n', wait_info), ''), ''),
                    /* If debug is enabled, add the file:lineno information for waits */
                    IF (in_debug AND event_name LIKE 'wait%', wait_info, ''),
                    '", ', 
                    /* Depending on the type of event, style appropriately */
                    CASE WHEN event_name LIKE 'wait/io/file%' THEN 
                           'shape=box, style=filled, color=red'
                         WHEN event_name LIKE 'wait/io/table%' THEN 
                           'shape=box, style=filled, color=green'
                         WHEN event_name LIKE 'wait/io/socket%' THEN
                           'shape=box, style=filled, color=yellow'
                         WHEN event_name LIKE 'wait/synch/mutex%' THEN
                           'style=filled, color=lightskyblue'
                         WHEN event_name LIKE 'wait/synch/cond%' THEN
                           'style=filled, color=darkseagreen3'
                         WHEN event_name LIKE 'wait/synch/rwlock%' THEN
                           'style=filled, color=orchid'
                         WHEN event_name LIKE 'wait/lock%' THEN
                           'shape=box, style=filled, color=tan'
                         WHEN event_name LIKE 'statement/%' THEN
                           CONCAT('shape=box, style=bold',
                                  /* Style statements depending on COM vs SQL */
                                  CASE WHEN event_name LIKE 'statement/com/%' THEN
                                         ' style=filled, color=darkseagreen'
                                       ELSE
                                         /* Use long query time from the server to
                                            flag long running statements in red */
                                         IF((timer_wait/1000000000000) > @@long_query_time, 
                                            ' style=filled, color=red', 
                                            ' style=filled, color=lightblue')
                                  END
                           )
                         WHEN event_name LIKE 'stage/%' THEN
                           'style=filled, color=slategray3'
                         /* IDLE events are on their own, call attention to them */
                         WHEN event_name LIKE '%idle%' THEN
                           'shape=box, style=filled, color=firebrick3'
                         ELSE '' END,
                     '];\n'
                   ) event, event_id
        FROM (
             /* Select all statements, with the extra tracing information available */
             (SELECT thread_id, event_id, event_name, timer_wait, timer_start, nesting_event_id, 
                     CONCAT(sql_text, '\\n',
                            'errors: ', errors, '\\n',
                            'warnings: ', warnings, '\\n',
                            'lock time: ', wbsys.format_time(lock_time),'\\n',
                            'rows affected: ', rows_affected, '\\n',
                            'rows sent: ', rows_sent, '\\n',
                            'rows examined: ', rows_examined, '\\n',
                            'tmp tables: ', created_tmp_tables, '\\n',
                            'tmp disk tables: ', created_tmp_disk_tables, '\\n'
                            'select scan: ', select_scan, '\\n',
                            'select full join: ', select_full_join, '\\n',
                            'select full range join: ', select_full_range_join, '\\n',
                            'select range: ', select_range, '\\n',
                            'select range check: ', select_range_check, '\\n', 
                            'sort merge passes: ', sort_merge_passes, '\\n',
                            'sort rows: ', sort_rows, '\\n',
                            'sort range: ', sort_range, '\\n',
                            'sort scan: ', sort_scan, '\\n',
                            'no index used: ', IF(no_index_used, 'TRUE', 'FALSE'), '\\n',
                            'no good index used: ', IF(no_good_index_used, 'TRUE', 'FALSE'), '\\n'
                     ) AS wait_info
                FROM performance_schema.events_statements_history_long
               WHERE thread_id = in_thread_id AND event_id > v_min_event_id)
             UNION
             /* Select all stages */
             (SELECT thread_id, event_id, event_name, timer_wait, timer_start, nesting_event_id, null AS wait_info
                FROM performance_schema.events_stages_history_long 
               WHERE thread_id = in_thread_id AND event_id > v_min_event_id)
             UNION 
             /* Select all events, adding information appropriate to the event */
             (SELECT thread_id, event_id, 
                     CONCAT(event_name, 
                            IF(event_name NOT LIKE 'wait/synch/mutex%', IFNULL(CONCAT(' - ', operation), ''), ''), 
                            IF(number_of_bytes IS NOT NULL, CONCAT(' ', number_of_bytes, ' bytes'), ''),
                            IF(event_name LIKE 'wait/io/file%', '\\n', ''),
                            IF(object_schema IS NOT NULL, CONCAT('\\nObject: ', object_schema, '.'), ''), 
                            IF(object_name IS NOT NULL, 
                               IF (event_name LIKE 'wait/io/socket%',
                                   /* Print the socket if used, else the IP:port as reported */
                                   CONCAT('\\n', IF (object_name LIKE ':0%', @@socket, object_name)),
                                   object_name),
                               ''
                            ),
                            IF(index_name IS NOT NULL, CONCAT(' Index: ', index_name), ''), '\\n'
                     ) AS event_name,
                     timer_wait, timer_start, nesting_event_id, source AS wait_info
                FROM performance_schema.events_waits_history_long
               WHERE thread_id = in_thread_id AND event_id > v_min_event_id)
           ) events 
       ORDER BY event_id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET v_done = TRUE;

    /* Do not track the current thread, it will kill the stack */
    CALL wbsys.disable_current_thread();

    IF (in_auto_setup) THEN
        CALL wbsys.save_current_config();
        
        /* Ensure only the thread to create the stack trace for is instrumented and that we instrument
           everything. */
        DELETE FROM performance_schema.setup_actors;

        UPDATE performance_schema.threads
           SET INSTRUMENTED = IF(THREAD_ID = in_thread_id, 'YES', 'NO');

        UPDATE performance_schema.setup_consumers
           SET ENABLED = 'YES'
         WHERE NAME NOT LIKE '%\_history'; -- only the %_history_long tables and it ancestors are needed

        UPDATE performance_schema.setup_instruments
           SET ENABLED = 'YES',
               TIMED   = 'YES';
    END IF;

    IF (in_start_fresh) THEN
        TRUNCATE performance_schema.events_statements_history_long;
        TRUNCATE performance_schema.events_stages_history_long;
        TRUNCATE performance_schema.events_waits_history_long;
    END IF;

    DROP TEMPORARY TABLE IF EXISTS tmp_events;
    CREATE TEMPORARY TABLE tmp_events (
      event_id bigint unsigned NOT NULL,
      event longblob,
      PRIMARY KEY (event_id)
    );

    /* Print headers for a .dot file */
    INSERT INTO tmp_events VALUES (0, CONCAT('digraph events { rankdir=LR; nodesep=0.10;\n',
                                             '// Stack created .....: ', NOW(), '\n',
                                             '// MySQL version .....: ', VERSION(), '\n',
                                             '// MySQL hostname ....: ', @@hostname, '\n',
                                             '// MySQL port ........: ', @@port, '\n',
                                             '// MySQL socket ......: ', @@socket, '\n',
                                             '// MySQL user ........: ', CURRENT_USER(), '\n'));

    SELECT CONCAT('Data collection starting for THREAD_ID = ', in_thread_id) AS 'Info';

    SET v_min_event_id = 0,
        v_start        = UNIX_TIMESTAMP(),
        in_interval    = IFNULL(in_interval, 1.00),
        in_max_runtime = IFNULL(in_max_runtime, 60.00);

    WHILE (v_runtime < in_max_runtime
           AND (SELECT INSTRUMENTED FROM performance_schema.threads WHERE THREAD_ID = in_thread_id) = 'YES') DO
        SET v_done = FALSE;
        OPEN c_stack;
        c_stack_loop: LOOP
            FETCH c_stack INTO v_event, v_min_event_id;
            IF v_done THEN
                LEAVE c_stack_loop;
            END IF;

            IF (LENGTH(v_event) > 0) THEN
                INSERT INTO tmp_events VALUES (v_min_event_id, v_event);
            END IF;
        END LOOP;
        CLOSE c_stack;

        SELECT SLEEP(in_interval) INTO @sleep;
        SET v_runtime = v_runtime + (UNIX_TIMESTAMP() - v_start);
    END WHILE;

    /* Reset the settings for the performance schema */
    IF (in_auto_setup) THEN
        CALL wbsys.reload_saved_config();
    END IF;

    INSERT INTO tmp_events VALUES (v_min_event_id+1, '}');
   
    SET @query = CONCAT('SELECT event FROM tmp_events ORDER BY event_id INTO OUTFILE ''', in_outfile, ''' FIELDS ESCAPED BY '''' LINES TERMINATED BY ''''');
    PREPARE stmt_output FROM @query;
    EXECUTE stmt_output;
    DEALLOCATE PREPARE stmt_output;
   
    SELECT CONCAT('Stack trace written to ', in_outfile) AS 'Info';
    SELECT CONCAT('dot -Tpdf -o /tmp/stack_', in_thread_id, '.pdf ', in_outfile) AS 'Convert to PDF';
    SELECT CONCAT('dot -Tpng -o /tmp/stack_', in_thread_id, '.png ', in_outfile) AS 'Convert to PNG';
    DROP TEMPORARY TABLE tmp_events;
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/enable_background_threads.sql
   ===================================================================
*/
  
  
/*
 * Procedure: enable_background_threads()
 *
 * Enable performance_schema instrumentation for all background threads
 *
 * Example: CALL enable_background_threads();
 *
 * Versions: 5.6.2+
 */

DROP PROCEDURE IF EXISTS enable_background_threads;

DELIMITER $$

CREATE PROCEDURE enable_background_threads()
BEGIN
    UPDATE performance_schema.threads
       SET instrumented = 'YES'
     WHERE type = 'BACKGROUND';
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/enable_current_thread.sql
   ===================================================================
*/
  
  
/*
 * Procedure: enable_current_thread()
 *
 * Enable performance_schema instrumentation for the current thread
 *
 * Example: CALL enable_current_thread();
 *
 * Versions: 5.6.2+
 */

DROP PROCEDURE IF EXISTS enable_current_thread;

DELIMITER $$

CREATE PROCEDURE enable_current_thread()
    SQL SECURITY INVOKER 
BEGIN
     UPDATE performance_schema.threads
        SET instrumented = 'YES'
      WHERE processlist_id = CONNECTION_ID();
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/only_enable.sql
   ===================================================================
*/
  
  
/*
 * Procedure: only_enable()
 *
 * Only enable a certain form of wait event
 *
 * Parameters
 *   pattern: A LIKE pattern match of events to leave enabled
 *
 * Versions: 5.5+
 */

DROP PROCEDURE IF EXISTS only_enable;

DELIMITER $$

CREATE PROCEDURE only_enable(IN pattern VARCHAR(128))
    COMMENT 'Parameters: pattern (VARCHAR(128))'
    SQL SECURITY INVOKER 
BEGIN
    UPDATE performance_schema.setup_instruments
       SET enabled = IF(name LIKE pattern, 'YES', 'NO'),
           timed = IF(name LIKE pattern, 'YES', 'NO');
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/reload_saved_config.sql
   ===================================================================
*/
  
  
/*
 * Procedure: reload_saved_config()
 * 
 * Reloads configuration of performance_schema, as saved by
 * the save_current_config() procedure.
 *
 * Versions: 5.6+
 *
 * Contributed by Jesper Krogh of MySQL Support @ Oracle, extended by Mark Leith
 */

DROP PROCEDURE IF EXISTS reload_saved_config;

DELIMITER $$

CREATE PROCEDURE reload_saved_config()
    SQL SECURITY INVOKER
BEGIN
    DECLARE v_done bool DEFAULT FALSE;
    DECLARE EXIT HANDLER FOR SQLEXCEPTION
    BEGIN
        SIGNAL SQLSTATE VALUE '90000'
           SET MESSAGE_TEXT = 'An error occurred, was save_current_config() run before this procedure?';
    END;

    DELETE FROM performance_schema.setup_actors;
    INSERT INTO performance_schema.setup_actors SELECT * FROM tmp_setup_actors;


    BEGIN
        /* Workaround for http://bugs.mysql.com/bug.php?id=70025 */
        DECLARE v_name varchar(64);
        DECLARE v_enabled enum('YES', 'NO');
        DECLARE c_consumers CURSOR FOR SELECT NAME, ENABLED FROM tmp_setup_consumers;
        DECLARE CONTINUE HANDLER FOR NOT FOUND SET v_done = TRUE;

        SET v_done = FALSE;
        OPEN c_consumers;
        c_consumers_loop: LOOP
            FETCH c_consumers INTO v_name, v_enabled;
            IF v_done THEN
               LEAVE c_consumers_loop;
            END IF;

            UPDATE performance_schema.setup_consumers
               SET ENABLED = v_enabled
             WHERE NAME = v_name;
         END LOOP;
         CLOSE c_consumers;
    END;

    UPDATE performance_schema.setup_instruments
     INNER JOIN tmp_setup_instruments USING (NAME)
       SET performance_schema.setup_instruments.ENABLED = tmp_setup_instruments.ENABLED,
           performance_schema.setup_instruments.TIMED   = tmp_setup_instruments.TIMED;
    BEGIN
        /* Workaround for http://bugs.mysql.com/bug.php?id=70025 */
        DECLARE v_thread_id bigint unsigned;
        DECLARE v_instrumented enum('YES', 'NO');
        DECLARE c_threads CURSOR FOR SELECT THREAD_ID, INSTRUMENTED FROM tmp_threads;
        DECLARE CONTINUE HANDLER FOR NOT FOUND SET v_done = TRUE;

        SET v_done = FALSE;
        OPEN c_threads;
        c_threads_loop: LOOP
            FETCH c_threads INTO v_thread_id, v_instrumented;
            IF v_done THEN
               LEAVE c_threads_loop;
            END IF;

            UPDATE performance_schema.threads
               SET INSTRUMENTED = v_instrumented
             WHERE THREAD_ID = v_thread_id;
        END LOOP;
        CLOSE c_threads;
    END;

    UPDATE performance_schema.threads
       SET INSTRUMENTED = IF(PROCESSLIST_USER IS NOT NULL,
                             wbsys.is_account_enabled(PROCESSLIST_HOST, PROCESSLIST_USER),
                             'YES')
     WHERE THREAD_ID NOT IN (SELECT THREAD_ID FROM tmp_threads);

    DROP TEMPORARY TABLE tmp_setup_actors;
    DROP TEMPORARY TABLE tmp_setup_consumers;
    DROP TEMPORARY TABLE tmp_setup_instruments;
    DROP TEMPORARY TABLE tmp_threads;
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/reset_to_default_57.sql
   ===================================================================
*/
  
  
/**
 * Procedure: reset_to_default
 *
 * Reset the settings to the default 5.7 settings.
 *
 * Parameters
 *   in_verbose: Whether to print each statement before executing
 *
 * Versions: 5.7+
 *
 * Contributed by Jesper Krogh of MySQL Support @ Oracle
 */

-- Because of bug 11750980/bug 41686, set the sql_mode to ''
SET @old_sql_mode = @@session.sql_mode;
SET sql_mode = '';

DROP PROCEDURE IF EXISTS reset_to_default;

DELIMITER $$

CREATE PROCEDURE reset_to_default(IN in_verbose BOOLEAN)
   COMMENT 'Parameters: in_verbose (boolean)'
BEGIN
   SET @query = 'DELETE
  				   FROM performance_schema.setup_actors
                  WHERE NOT (HOST = ''%'' AND USER = ''%'' AND ROLE = ''%'')';

   IF (in_verbose) THEN
      SELECT CONCAT('Resetting: setup_actors\n', @query) AS status;
   END IF;

   PREPARE reset_stmt FROM @query;
   EXECUTE reset_stmt;
   DEALLOCATE PREPARE reset_stmt;

   SET @query = 'INSERT IGNORE INTO performance_schema.setup_actors
                 VALUES (''%'', ''%'', ''%'')';

   IF (in_verbose) THEN
      SELECT CONCAT('Resetting: setup_actors\n', @query) AS status;
   END IF;

   PREPARE reset_stmt FROM @query;
   EXECUTE reset_stmt;
   DEALLOCATE PREPARE reset_stmt;

   SET @query = 'UPDATE performance_schema.setup_instruments
                    SET ENABLED = ''NO'', TIMED = ''NO''
                  WHERE NAME NOT LIKE ''wait/io/file/%''
                    AND NAME NOT LIKE ''wait/io/table/%''
                    AND NAME NOT LIKE ''statement/%''
                    AND NAME NOT IN (''wait/lock/table/sql/handler'', ''idle'')';

   IF (in_verbose) THEN
      SELECT CONCAT('Resetting: setup_instruments\n', @query) AS status;
   END IF;

   PREPARE reset_stmt FROM @query;
   EXECUTE reset_stmt;
   DEALLOCATE PREPARE reset_stmt;
         
   SET @query = 'UPDATE performance_schema.setup_consumers
                    SET ENABLED = IF(NAME IN (''events_statements_current'', ''global_instrumentation'', ''thread_instrumentation'', ''statements_digest''), ''YES'', ''NO'')';

   IF (in_verbose) THEN
      SELECT CONCAT('Resetting: setup_consumers\n', @query) AS status;
   END IF;

   PREPARE reset_stmt FROM @query;
   EXECUTE reset_stmt;
   DEALLOCATE PREPARE reset_stmt;

   SET @query = 'DELETE
                   FROM performance_schema.setup_objects
                  WHERE NOT (OBJECT_TYPE IN (''EVENT'', ''FUNCTION'', ''PROCEDURE'', ''TABLE'', ''TRIGGER'') AND OBJECT_NAME = ''%'' 
                  	AND (OBJECT_SCHEMA = ''mysql''              AND ENABLED = ''NO''  AND TIMED = ''NO'' )
                     OR (OBJECT_SCHEMA = ''performance_schema'' AND ENABLED = ''NO''  AND TIMED = ''NO'' )
                     OR (OBJECT_SCHEMA = ''information_schema'' AND ENABLED = ''NO''  AND TIMED = ''NO'' )
                     OR (OBJECT_SCHEMA = ''%''                  AND ENABLED = ''YES'' AND TIMED = ''YES''))';

   IF (in_verbose) THEN
      SELECT CONCAT('Resetting: setup_objects\n', @query) AS status;
   END IF;

   PREPARE reset_stmt FROM @query;
   EXECUTE reset_stmt;
   DEALLOCATE PREPARE reset_stmt;

   SET @query = 'INSERT IGNORE INTO performance_schema.setup_objects
                 VALUES (''EVENT''    , ''mysql''             , ''%'', ''NO'' , ''NO'' ),
                        (''EVENT''    , ''performance_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''EVENT''    , ''information_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''EVENT''    , ''%''                 , ''%'', ''YES'', ''YES''),
                        (''FUNCTION'' , ''mysql''             , ''%'', ''NO'' , ''NO'' ),
                        (''FUNCTION'' , ''performance_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''FUNCTION'' , ''information_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''FUNCTION'' , ''%''                 , ''%'', ''YES'', ''YES''),
                        (''PROCEDURE'', ''mysql''             , ''%'', ''NO'' , ''NO'' ),
                        (''PROCEDURE'', ''performance_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''PROCEDURE'', ''information_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''PROCEDURE'', ''%''                 , ''%'', ''YES'', ''YES''),
                        (''TABLE''    , ''mysql''             , ''%'', ''NO'' , ''NO'' ),
                        (''TABLE''    , ''performance_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''TABLE''    , ''information_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''TABLE''    , ''%''                 , ''%'', ''YES'', ''YES''),
                        (''TRIGGER''  , ''mysql''             , ''%'', ''NO'' , ''NO'' ),
                        (''TRIGGER''  , ''performance_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''TRIGGER''  , ''information_schema'', ''%'', ''NO'' , ''NO'' ),
                        (''TRIGGER''  , ''%''                 , ''%'', ''YES'', ''YES'')';

   IF (in_verbose) THEN
      SELECT CONCAT('Resetting: setup_objects\n', @query) AS status;
   END IF;

   PREPARE reset_stmt FROM @query;
   EXECUTE reset_stmt;
   DEALLOCATE PREPARE reset_stmt;

   SET @query = 'UPDATE performance_schema.threads
                    SET INSTRUMENTED = ''YES''';

   IF (in_verbose) THEN
      SELECT CONCAT('Resetting: threads\n', @query) AS status;
   END IF;

   PREPARE reset_stmt FROM @query;
   EXECUTE reset_stmt;
   DEALLOCATE PREPARE reset_stmt;
END$$

DELIMITER ;

SET sql_mode = @old_sql_mode;
/* 
   ===================================================================
   Contents from file: procedures/save_current_config.sql
   ===================================================================
*/
  
  
/*
 * Procedure: save_current_config()
 * 
 * Saves the current configuration of performance_schema, so that you 
 * can alter the setup, but restore it to what was set before.
 * 
 * Use the companion procedure - reload_saved_config(), to 
 * restore the saved config.
 *
 * Versions: 5.6+
 *
 * Contributed by Jesper Krogh of MySQL Support @ Oracle
 */

DROP PROCEDURE IF EXISTS save_current_config;

DELIMITER $$

CREATE PROCEDURE save_current_config()
    SQL SECURITY INVOKER
BEGIN
    DROP TEMPORARY TABLE IF EXISTS tmp_setup_actors;
    DROP TEMPORARY TABLE IF EXISTS tmp_setup_consumers;
    DROP TEMPORARY TABLE IF EXISTS tmp_setup_instruments;
    DROP TEMPORARY TABLE IF EXISTS tmp_threads;

    CREATE TEMPORARY TABLE tmp_setup_actors LIKE performance_schema.setup_actors;
    CREATE TEMPORARY TABLE tmp_setup_consumers LIKE performance_schema.setup_consumers;
    CREATE TEMPORARY TABLE tmp_setup_instruments LIKE performance_schema.setup_instruments;
    CREATE TEMPORARY TABLE tmp_threads (THREAD_ID bigint unsigned NOT NULL PRIMARY KEY, INSTRUMENTED enum('YES','NO') NOT NULL);

    INSERT INTO tmp_setup_actors SELECT * FROM performance_schema.setup_actors;
    INSERT INTO tmp_setup_consumers SELECT * FROM performance_schema.setup_consumers;
    INSERT INTO tmp_setup_instruments SELECT * FROM performance_schema.setup_instruments;
    INSERT INTO tmp_threads SELECT THREAD_ID, INSTRUMENTED FROM performance_schema.threads;
END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: procedures/truncate_all.sql
   ===================================================================
*/
  
  
/*
 * Procedure: truncate_all()
 *
 * Truncates all summary tables, to reset all performance schema statistics
 *
 * Parameters
 *   verbose: Whether to print each TRUNCATE statement before running
 *
 * Versions: 5.5+
 */

DROP PROCEDURE IF EXISTS truncate_all;

DELIMITER $$

CREATE PROCEDURE truncate_all(IN verbose BOOLEAN)
    COMMENT 'Parameters: verbose (boolean)'
    SQL SECURITY INVOKER 
BEGIN
    DECLARE v_done INT DEFAULT FALSE;
    DECLARE v_ps_table VARCHAR(64);
    DECLARE ps_tables CURSOR FOR
        SELECT table_name 
          FROM INFORMATION_SCHEMA.TABLES 
         WHERE table_schema = 'performance_schema' 
           AND (table_name LIKE '%summary%' 
           OR table_name LIKE '%history%');
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET v_done = TRUE;

    SET @log_bin := @@sql_log_bin;
    SET sql_log_bin = 0;

    OPEN ps_tables;

    ps_tables_loop: LOOP
        FETCH ps_tables INTO v_ps_table;
        IF v_done THEN
          LEAVE ps_tables_loop;
        END IF;

        SET @truncate_stmt := CONCAT('TRUNCATE TABLE performance_schema.', v_ps_table);
        IF verbose THEN
            SELECT CONCAT('Running: ', @truncate_stmt) AS status;
        END IF;

        PREPARE truncate_stmt FROM @truncate_stmt;
        EXECUTE truncate_stmt;
        DEALLOCATE PREPARE truncate_stmt;

    END LOOP;

    CLOSE ps_tables;

    SET sql_log_bin = @log_bin;

END$$

DELIMITER ;
/* 
   ===================================================================
   Contents from file: views/i_s/innodb_buffer_stats_by_schema.sql
   ===================================================================
*/
  
  
/* 
 * View: innodb_buffer_stats_by_schema
 * 
 * Summarizes the output of the INFORMATION_SCHEMA.INNODB_BUFFER_PAGE 
 * table, aggregating by schema
 *
 * mysql> SELECT * FROM innodb_buffer_stats_by_schema;
 * +---------------+------------+------------+-------+--------------+-----------+-------------+
 * | object_schema | allocated  | data       | pages | pages_hashed | pages_old | rows_cached |
 * +---------------+------------+------------+-------+--------------+-----------+-------------+
 * | common_schema | 1.06 MiB   | 529.29 KiB |    68 |           68 |        68 |         697 |
 * | InnoDB System | 144.00 KiB | 15.67 KiB  |     9 |            9 |         9 |          43 |
 * | mysql         | 80.00 KiB  | 9.01 KiB   |     5 |            5 |         5 |          83 |
 * +---------------+------------+------------+-------+--------------+-----------+-------------+
 * 3 rows in set (0.08 sec)
 *
 * Versions: 5.5.28+
 */

/*!50528 DROP VIEW IF EXISTS innodb_buffer_stats_by_schema */;

/*!50528 
CREATE SQL SECURITY INVOKER VIEW innodb_buffer_stats_by_schema AS
SELECT IF(LOCATE('.', ibp.table_name) = 0, 'InnoDB System', REPLACE(SUBSTRING_INDEX(ibp.table_name, '.', 1), '`', '')) AS object_schema,
       format_bytes(SUM(IF(ibp.compressed_size = 0, 16384, compressed_size))) AS allocated,
       format_bytes(SUM(ibp.data_size)) AS data,
       COUNT(ibp.page_number) AS pages,
       COUNT(IF(ibp.is_hashed = 'YES', 1, 0)) AS pages_hashed,
       COUNT(IF(ibp.is_old = 'YES', 1, 0)) AS pages_old,
       ROUND(SUM(ibp.number_records)/COUNT(DISTINCT ibp.index_name)) AS rows_cached 
  FROM information_schema.innodb_buffer_page ibp 
 WHERE table_name IS NOT NULL
 GROUP BY object_schema
 ORDER BY SUM(IF(ibp.compressed_size = 0, 16384, compressed_size)) DESC */;

/* 
 * View: innodb_buffer_stats_by_schema_raw
 * 
 * Summarizes the output of the INFORMATION_SCHEMA.INNODB_BUFFER_PAGE 
 * table, aggregating by schema
 *
 * mysql> SELECT * FROM innodb_buffer_stats_by_schema_raw;
 * +---------------+-----------+--------+-------+--------------+-----------+-------------+
 * | object_schema | allocated | data   | pages | pages_hashed | pages_old | rows_cached |
 * +---------------+-----------+--------+-------+--------------+-----------+-------------+
 * | common_schema |   1114112 | 541996 |    68 |           68 |        68 |         697 |
 * | InnoDB System |    147456 |  16047 |     9 |            9 |         9 |          43 |
 * | mysql         |     81920 |   9224 |     5 |            5 |         5 |          83 |
 * +---------------+-----------+--------+-------+--------------+-----------+-------------+
 * 3 rows in set (0.10 sec)
 *
 * Versions: 5.5.28+
 */

/*!50528 DROP VIEW IF EXISTS innodb_buffer_stats_by_schema_raw */;

/*!50528 
CREATE SQL SECURITY INVOKER VIEW innodb_buffer_stats_by_schema_raw AS
SELECT IF(LOCATE('.', ibp.table_name) = 0, 'InnoDB System', REPLACE(SUBSTRING_INDEX(ibp.table_name, '.', 1), '`', '')) AS object_schema,
       SUM(IF(ibp.compressed_size = 0, 16384, compressed_size)) AS allocated,
       SUM(ibp.data_size) AS data,
       COUNT(ibp.page_number) AS pages,
       COUNT(IF(ibp.is_hashed = 'YES', 1, 0)) AS pages_hashed,
       COUNT(IF(ibp.is_old = 'YES', 1, 0)) AS pages_old,
       ROUND(SUM(ibp.number_records)/COUNT(DISTINCT ibp.index_name)) AS rows_cached 
  FROM information_schema.innodb_buffer_page ibp 
 WHERE table_name IS NOT NULL
 GROUP BY object_schema
 ORDER BY SUM(IF(ibp.compressed_size = 0, 16384, compressed_size)) DESC */;
/* 
   ===================================================================
   Contents from file: views/i_s/innodb_buffer_stats_by_table.sql
   ===================================================================
*/
  
  
/* View: innodb_buffer_stats_by_table
 * 
 * Summarizes the output of the INFORMATION_SCHEMA.INNODB_BUFFER_PAGE 
 * table, aggregating by schema and table name
 *
 * mysql> select * from innodb_buffer_stats_by_table;
 * +---------------+-------------+-----------+-----------+-------+--------------+-----------+-------------+
 * | object_schema | object_name | allocated | data      | pages | pages_hashed | pages_old | rows_cached |
 * +---------------+-------------+-----------+-----------+-------+--------------+-----------+-------------+
 * | InnoDB System | SYS_FOREIGN | 32.00 KiB | 0 bytes   |     2 |            2 |         2 |           0 |
 * | InnoDB System | SYS_COLUMNS | 16.00 KiB | 501 bytes |     1 |            1 |         1 |           8 |
 * | InnoDB System | SYS_FIELDS  | 16.00 KiB | 203 bytes |     1 |            1 |         1 |           5 |
 * | InnoDB System | SYS_INDEXES | 16.00 KiB | 266 bytes |     1 |            1 |         1 |           4 |
 * | InnoDB System | SYS_TABLES  | 16.00 KiB | 149 bytes |     1 |            1 |         1 |           2 |
 * +---------------+-------------+-----------+-----------+-------+--------------+-----------+-------------+
 * 5 rows in set (2.16 sec)
 *
 * Versions: 5.5.28+
 */

/*!50528 DROP VIEW IF EXISTS innodb_buffer_stats_by_table */;

/*!50528 
CREATE SQL SECURITY INVOKER VIEW innodb_buffer_stats_by_table AS
SELECT IF(LOCATE('.', ibp.table_name) = 0, 'InnoDB System', REPLACE(SUBSTRING_INDEX(ibp.table_name, '.', 1), '`', '')) AS object_schema,
       REPLACE(SUBSTRING_INDEX(ibp.table_name, '.', -1), '`', '') AS object_name,
       format_bytes(SUM(IF(ibp.compressed_size = 0, 16384, compressed_size))) AS allocated,
       format_bytes(SUM(ibp.data_size)) AS data,
       COUNT(ibp.page_number) AS pages,
       COUNT(IF(ibp.is_hashed = 'YES', 1, 0)) AS pages_hashed,
       COUNT(IF(ibp.is_old = 'YES', 1, 0)) AS pages_old,
       ROUND(SUM(ibp.number_records)/COUNT(DISTINCT ibp.index_name)) AS rows_cached 
  FROM information_schema.innodb_buffer_page ibp 
 WHERE table_name IS NOT NULL
 GROUP BY object_schema, object_name
 ORDER BY SUM(IF(ibp.compressed_size = 0, 16384, compressed_size)) DESC */;

/* View: innodb_buffer_stats_by_table_raw
 * 
 * Summarizes the output of the INFORMATION_SCHEMA.INNODB_BUFFER_PAGE 
 * table, aggregating by schema and table name
 *
 * mysql> select * from innodb_buffer_stats_by_table_raw;
 * +---------------+-------------+-----------+------+-------+--------------+-----------+-------------+
 * | object_schema | object_name | allocated | data | pages | pages_hashed | pages_old | rows_cached |
 * +---------------+-------------+-----------+------+-------+--------------+-----------+-------------+
 * | InnoDB System | SYS_FOREIGN |     32768 |    0 |     2 |            2 |         2 |           0 |
 * | InnoDB System | SYS_COLUMNS |     16384 |  501 |     1 |            1 |         1 |           8 |
 * | InnoDB System | SYS_FIELDS  |     16384 |  203 |     1 |            1 |         1 |           5 |
 * | InnoDB System | SYS_INDEXES |     16384 |  266 |     1 |            1 |         1 |           4 |
 * | InnoDB System | SYS_TABLES  |     16384 |  149 |     1 |            1 |         1 |           2 |
 * +---------------+-------------+-----------+------+-------+--------------+-----------+-------------+
 * 5 rows in set (1.80 sec)
 *
 * Versions: 5.5.28+
 */

/*!50528 DROP VIEW IF EXISTS innodb_buffer_stats_by_table_raw */;

/*!50528
CREATE SQL SECURITY INVOKER VIEW innodb_buffer_stats_by_table_raw AS
SELECT IF(LOCATE('.', ibp.table_name) = 0, 'InnoDB System', REPLACE(SUBSTRING_INDEX(ibp.table_name, '.', 1), '`', '')) AS object_schema,
       REPLACE(SUBSTRING_INDEX(ibp.table_name, '.', -1), '`', '') AS object_name,
       SUM(IF(ibp.compressed_size = 0, 16384, compressed_size)) AS allocated,
       SUM(ibp.data_size) AS data,
       COUNT(ibp.page_number) AS pages,
       COUNT(IF(ibp.is_hashed = 'YES', 1, 0)) AS pages_hashed,
       COUNT(IF(ibp.is_old = 'YES', 1, 0)) AS pages_old,
       ROUND(SUM(ibp.number_records)/COUNT(DISTINCT ibp.index_name)) AS rows_cached 
  FROM information_schema.innodb_buffer_page ibp 
 WHERE table_name IS NOT NULL
 GROUP BY object_schema, object_name
 ORDER BY SUM(IF(ibp.compressed_size = 0, 16384, compressed_size)) DESC */;
/* 
   ===================================================================
   Contents from file: views/i_s/schema_object_overview.sql
   ===================================================================
*/
  
  
/* 
 * View: schema_object_overview
 * 
 * Shows an overview of the types of objects within each schema
 *
 * Note: On instances with a large number of objects, this could take
 *       some time to execute, and is not recommended.
 *
 * mysql> select * from schema_object_overview;
 * +--------------------+---------------+-------+
 * | db                 | object_type   | count |
 * +--------------------+---------------+-------+
 * | information_schema | SYSTEM VIEW   |    59 |
 * | mysql              | BASE TABLE    |    28 |
 * | mysql              | INDEX (BTREE) |    63 |
 * | performance_schema | BASE TABLE    |    52 |
 * | wbsys          | FUNCTION      |     8 |
 * | wbsys          | PROCEDURE     |    12 |
 * | wbsys          | VIEW          |    49 |
 * | test               | BASE TABLE    |     2 |
 * | test               | INDEX (BTREE) |     2 |
 * +--------------------+---------------+-------+
 * 9 rows in set (0.08 sec)
 *
 * Versions: 5.1+
 */

DROP VIEW IF EXISTS schema_object_overview;

CREATE SQL SECURITY INVOKER VIEW schema_object_overview AS
SELECT ROUTINE_SCHEMA AS db, ROUTINE_TYPE AS object_type, COUNT(*) AS count FROM INFORMATION_SCHEMA.ROUTINES GROUP BY ROUTINE_SCHEMA, ROUTINE_TYPE
 UNION 
SELECT TABLE_SCHEMA, TABLE_TYPE, COUNT(*) FROM INFORMATION_SCHEMA.TABLES GROUP BY TABLE_SCHEMA, TABLE_TYPE
 UNION
SELECT TABLE_SCHEMA, CONCAT('INDEX (', INDEX_TYPE, ')'), COUNT(*) FROM INFORMATION_SCHEMA.STATISTICS GROUP BY TABLE_SCHEMA, INDEX_TYPE
 UNION
SELECT TRIGGER_SCHEMA, 'TRIGGER', COUNT(*) FROM INFORMATION_SCHEMA.TRIGGERS GROUP BY TRIGGER_SCHEMA
 UNION
SELECT EVENT_SCHEMA, 'EVENT', COUNT(*) FROM INFORMATION_SCHEMA.EVENTS GROUP BY EVENT_SCHEMA
ORDER BY DB, OBJECT_TYPE;
/* 
   ===================================================================
   Contents from file: views/p_s/check_lost_instrumentation.sql
   ===================================================================
*/
  
  
/*
 * View: check_lost_instrumentation
 * 
 * Used to check whether Performance Schema is not able to monitor
 * all runtime data - only returns variables that have lost instruments
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS check_lost_instrumentation;

CREATE SQL SECURITY INVOKER VIEW check_lost_instrumentation AS
SELECT variable_name, variable_value
  FROM information_schema.global_status
 WHERE variable_name LIKE 'perf%lost'
   AND variable_value > 0;
/* 
   ===================================================================
   Contents from file: views/p_s/processlist_57.sql
   ===================================================================
*/
  
  
/*
 * View: processlist
 *
 * A detailed non-blocking processlist view to replace 
 * [INFORMATION_SCHEMA. | SHOW FULL] PROCESSLIST
 *
 * mysql> select * from processlist_full where conn_id is not null\G
 * ...
 * *************************** 8. row ***************************
 *                 thd_id: 31
 *                conn_id: 12
 *                   user: root@localhost
 *                     db: wbsys
 *                command: Query
 *                  state: Sending data
 *                   time: 0
 *      current_statement: select * from processlist limit 5
 *           lock_latency: 684.00 us
 *          rows_examined: 0
 *              rows_sent: 0
 *          rows_affected: 0
 *             tmp_tables: 2
 *        tmp_disk_tables: 0
 *              full_scan: YES
 *         current_memory: 1.29 MiB
 *         last_statement: NULL
 * last_statement_latency: NULL
 *              last_wait: wait/synch/mutex/sql/THD::LOCK_query_plan
 *      last_wait_latency: 260.13 ns
 *                 source: sql_optimizer.cc:1075
 *
 * Versions: 5.7.2+
 *
 */
 
DROP VIEW IF EXISTS processlist;

CREATE SQL SECURITY INVOKER VIEW processlist AS
SELECT pps.thread_id AS thd_id,
       pps.processlist_id AS conn_id,
       IF(pps.name = 'thread/sql/one_connection', 
          CONCAT(pps.processlist_user, '@', pps.processlist_host), 
          REPLACE(pps.name, 'thread/', '')) user,
       pps.processlist_db AS db,
       pps.processlist_command AS command,
       pps.processlist_state AS state,
       pps.processlist_time AS time,
       wbsys.format_statement(pps.processlist_info) AS current_statement,
       wbsys.format_time(esc.lock_time) AS lock_latency,
       esc.rows_examined,
       esc.rows_sent,
       esc.rows_affected,
       esc.created_tmp_tables AS tmp_tables,
       esc.created_tmp_disk_tables AS tmp_disk_tables,
       IF(esc.no_good_index_used > 0 OR esc.no_index_used > 0, 
          'YES', 'NO') AS full_scan,
       wbsys.format_bytes(SUM(mem.current_number_of_bytes_used)) AS current_memory,
       IF(esc.timer_wait IS NOT NULL,
          wbsys.format_statement(esc.sql_text),
          NULL) AS last_statement,
       IF(esc.timer_wait IS NOT NULL,
          wbsys.format_time(esc.timer_wait),
          NULL) as last_statement_latency,
       ewc.event_name AS last_wait,
       IF(ewc.timer_wait IS NULL AND ewc.event_name IS NOT NULL, 
          'Still Waiting', 
          wbsys.format_time(ewc.timer_wait)) last_wait_latency,
       ewc.source
  FROM performance_schema.threads AS pps
  LEFT JOIN performance_schema.events_waits_current AS ewc USING (thread_id)
  LEFT JOIN performance_schema.events_statements_current as esc USING (thread_id)
  LEFT JOIN performance_schema.memory_summary_by_thread_by_event_name as mem USING (thread_id)
GROUP BY thread_id
ORDER BY pps.processlist_time DESC, last_wait_latency DESC;

/*
 * View: processlist_raw
 *
 * A detailed non-blocking processlist view to replace 
 * [INFORMATION_SCHEMA. | SHOW FULL] PROCESSLIST
 * 
 * mysql> select * from processlist_full where conn_id is not null\G
 * *************************** 1. row ***************************
 *                 thd_id: 31
 *                conn_id: 12
 *                   user: root@localhost
 *                     db: wbsys
 *                command: Query
 *                  state: Sending data
 *                   time: 0
 *      current_statement: select * from processlist_raw limit 5
 *           lock_latency: 1066000000
 *          rows_examined: 0
 *              rows_sent: 0
 *          rows_affected: 0
 *             tmp_tables: 2
 *        tmp_disk_tables: 1
 *              full_scan: YES
 *         current_memory: 1464694
 *         last_statement: NULL
 * last_statement_latency: NULL
 *              last_wait: wait/io/file/myisam/dfile
 *      last_wait_latency: 1602250
 *                 source: mf_iocache.c:163
 *
 * Versions: 5.7.2+
 *
 */
 
DROP VIEW IF EXISTS processlist_raw;

CREATE SQL SECURITY INVOKER VIEW processlist_raw AS
SELECT pps.thread_id AS thd_id,
       pps.processlist_id AS conn_id,
       IF(pps.name = 'thread/sql/one_connection', 
          CONCAT(pps.processlist_user, '@', pps.processlist_host), 
          REPLACE(pps.name, 'thread/', '')) user,
       pps.processlist_db AS db,
       pps.processlist_command AS command,
       pps.processlist_state AS state,
       pps.processlist_time AS time,
       pps.processlist_info AS current_statement,
       esc.lock_time AS lock_latency,
       esc.rows_examined,
       esc.rows_sent,
       esc.rows_affected,
       esc.created_tmp_tables AS tmp_tables,
       esc.created_tmp_disk_tables AS tmp_disk_tables,
       IF(esc.no_good_index_used > 0 OR esc.no_index_used > 0, 
          'YES', 'NO') AS full_scan,
       SUM(mem.current_number_of_bytes_used) AS current_memory,
       IF(esc.timer_wait IS NOT NULL,
          esc.sql_text,
          NULL) AS last_statement,
       IF(esc.timer_wait IS NOT NULL,
          esc.timer_wait,
          NULL) as last_statement_latency,
       ewc.event_name AS last_wait,
       IF(ewc.timer_wait IS NULL AND ewc.event_name IS NOT NULL, 
          'Still Waiting', 
          ewc.timer_wait) last_wait_latency,
       ewc.source
  FROM performance_schema.threads AS pps
  LEFT JOIN performance_schema.events_waits_current AS ewc USING (thread_id)
  LEFT JOIN performance_schema.events_statements_current as esc USING (thread_id)
  LEFT JOIN performance_schema.memory_summary_by_thread_by_event_name as mem USING (thread_id)
GROUP BY thread_id
ORDER BY pps.processlist_time DESC, last_wait_latency DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/latest_file_io.sql
   ===================================================================
*/
  
  
/*
 * View: latest_file_io
 *
 * Latest file IO, by file / thread
 *
 * mysql> select * from latest_file_io limit 5;
 * +----------------------+----------------------------------------+------------+-----------+-----------+
 * | thread               | file                                   | latency    | operation | requested |
 * +----------------------+----------------------------------------+------------+-----------+-----------+
 * | msandbox@localhost:1 | @@tmpdir/#sqlcf28_1_4e.MYI             | 9.26 µs    | write     | 124 bytes |
 * | msandbox@localhost:1 | @@tmpdir/#sqlcf28_1_4e.MYI             | 4.00 µs    | write     | 2 bytes   |
 * | msandbox@localhost:1 | @@tmpdir/#sqlcf28_1_4e.MYI             | 56.34 µs   | close     | NULL      |
 * | msandbox@localhost:1 | @@tmpdir/#sqlcf28_1_4e.MYD             | 53.93 µs   | close     | NULL      |
 * | msandbox@localhost:1 | @@tmpdir/#sqlcf28_1_4e.MYI             | 104.05 ms  | delete    | NULL      |
 * +----------------------+----------------------------------------+------------+-----------+-----------+
 * 5 rows in set (0.05 sec)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS latest_file_io;

CREATE SQL SECURITY INVOKER VIEW latest_file_io AS
SELECT IF(id IS NULL, 
             CONCAT(SUBSTRING_INDEX(name, '/', -1), ':', thread_id), 
             CONCAT(user, '@', host, ':', id)
          ) thread, 
       format_path(object_name) file, 
       format_time(timer_wait) AS latency, 
       operation, 
       format_bytes(number_of_bytes) AS requested
  FROM performance_schema.events_waits_history_long 
  JOIN performance_schema.threads USING (thread_id)
  LEFT JOIN information_schema.processlist ON processlist_id = id
 WHERE object_name IS NOT NULL
   AND event_name LIKE 'wait/io/file/%'
 ORDER BY timer_start;

/*
 * View: latest_file_io_raw
 *
 * Latest file IO, by file / thread without formatting
 *
 * mysql> SELECT * FROM latest_file_io_raw LIMIT 5;
 * +------------------+------------------------------------------------------------------------------------+-------------+-----------+-----------+
 * | thread           | file                                                                               | latency     | operation | requested |
 * +------------------+------------------------------------------------------------------------------------+-------------+-----------+-----------+
 * | root@localhost:6 | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/user_summary_by_statement_type.frm~ |    26152490 | write     |      4210 |
 * | root@localhost:6 | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/user_summary_by_statement_type.frm~ | 30062722690 | sync      |      NULL |
 * | root@localhost:6 | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/user_summary_by_statement_type.frm~ |    34144890 | close     |      NULL |
 * | root@localhost:6 | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/check_lost_instrumentation.frm      |   113001980 | open      |      NULL |
 * | root@localhost:6 | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/check_lost_instrumentation.frm      |     9553180 | read      |        10 |
 * +------------------+------------------------------------------------------------------------------------+-------------+-----------+-----------+
 * 5 rows in set (0.10 sec)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS latest_file_io_raw;

CREATE SQL SECURITY INVOKER VIEW latest_file_io_raw AS
SELECT IF(id IS NULL, 
             CONCAT(SUBSTRING_INDEX(name, '/', -1), ':', thread_id), 
             CONCAT(user, '@', host, ':', id)
          ) thread, 
       object_name file, 
       timer_wait AS latency, 
       operation, 
       number_of_bytes AS requested
  FROM performance_schema.events_waits_history_long 
  JOIN performance_schema.threads USING (thread_id)
  LEFT JOIN information_schema.processlist ON processlist_id = id
 WHERE object_name IS NOT NULL
   AND event_name LIKE 'wait/io/file/%'
 ORDER BY timer_start;
/* 
   ===================================================================
   Contents from file: views/p_s/io_by_thread_by_latency.sql
   ===================================================================
*/
  
  
/*
 * View: io_by_thread_by_latency
 *
 * Show the top IO consumers by thread, ordered by total latency
 *
 * mysql> select * from io_by_thread_by_latency;
 * +---------------------+------------+---------------+-------------+-------------+-------------+-----------+----------------+
 * | user                | count_star | total_latency | min_latency | avg_latency | max_latency | thread_id | processlist_id |
 * +---------------------+------------+---------------+-------------+-------------+-------------+-----------+----------------+
 * | root@localhost      |      11580 | 18.01 s       | 429.78 ns   | 1.12 ms     | 181.07 ms   |        25 |              6 |
 * | main                |       1358 | 1.31 s        | 475.02 ns   | 2.27 ms     | 350.70 ms   |         1 |           NULL |
 * | page_cleaner_thread |        654 | 147.44 ms     | 588.12 ns   | 225.44 us   | 46.41 ms    |        18 |           NULL |
 * | io_write_thread     |        131 | 107.75 ms     | 8.60 us     | 822.55 us   | 27.69 ms    |         8 |           NULL |
 * | io_write_thread     |         46 | 47.07 ms      | 10.64 us    | 1.02 ms     | 16.90 ms    |         9 |           NULL |
 * | io_write_thread     |         71 | 46.99 ms      | 9.11 us     | 661.81 us   | 17.04 ms    |        11 |           NULL |
 * | io_log_thread       |         20 | 21.01 ms      | 14.25 us    | 1.05 ms     | 7.08 ms     |         3 |           NULL |
 * | srv_master_thread   |         13 | 17.60 ms      | 8.49 us     | 1.35 ms     | 9.99 ms     |        16 |           NULL |
 * | srv_purge_thread    |          4 | 1.81 ms       | 34.31 us    | 452.45 us   | 1.02 ms     |        17 |           NULL |
 * | io_write_thread     |         19 | 951.39 us     | 9.75 us     | 50.07 us    | 297.47 us   |        10 |           NULL |
 * | signal_handler      |          3 | 218.03 us     | 21.64 us    | 72.68 us    | 154.84 us   |        19 |           NULL |
 * +---------------------+------------+---------------+-------------+-------------+-------------+-----------+----------------+
 *
 * (Example taken from 5.6.13)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS io_by_thread_by_latency;

CREATE VIEW io_by_thread_by_latency AS
SELECT IF(processlist_id IS NULL, 
             SUBSTRING_INDEX(name, '/', -1), 
             CONCAT(processlist_user, '@', processlist_host)
          ) user, 
       SUM(count_star) count_star,
       format_time(SUM(sum_timer_wait)) total_latency,
       format_time(MIN(min_timer_wait)) min_latency,
       format_time(AVG(avg_timer_wait)) avg_latency,
       format_time(MAX(max_timer_wait)) max_latency,
       thread_id,
       processlist_id
  FROM performance_schema.events_waits_summary_by_thread_by_event_name 
  LEFT JOIN performance_schema.threads USING (thread_id)
 WHERE event_name LIKE 'wait/io/file/%'
   AND sum_timer_wait > 0
 GROUP BY thread_id
 ORDER BY SUM(sum_timer_wait) DESC;

/*
 * View: io_by_thread_by_latency_raw
 *
 * Show the top IO consumers by thread, ordered by total latency
 *
 * Versions: 5.5+
 *
 * mysql> select * from io_by_thread_by_latency_raw;
 * +---------------------+------------+----------------+-------------+-----------------+--------------+-----------+----------------+
 * | user                | count_star | total_latency  | min_latency | avg_latency     | max_latency  | thread_id | processlist_id |
 * +---------------------+------------+----------------+-------------+-----------------+--------------+-----------+----------------+
 * | root@localhost      |      11587 | 18007539905680 |      429780 | 1120831681.6667 | 181065665560 |        25 |              6 |
 * | main                |       1358 |  1309001741320 |      475020 | 2269581997.8000 | 350700491310 |         1 |           NULL |
 * | page_cleaner_thread |        654 |   147435455960 |      588120 |  225436198.0000 |  46412043990 |        18 |           NULL |
 * | io_write_thread     |        131 |   107754483070 |     8603140 |  822553303.0000 |  27691592500 |         8 |           NULL |
 * | io_write_thread     |         46 |    47074926860 |    10642710 | 1023367631.0000 |  16899745070 |         9 |           NULL |
 * | io_write_thread     |         71 |    46988801210 |     9108320 |  661814075.0000 |  17042760020 |        11 |           NULL |
 * | io_log_thread       |         20 |    21007710490 |    14250600 | 1050385336.0000 |   7081255090 |         3 |           NULL |
 * | srv_master_thread   |         13 |    17601511720 |     8486270 | 1353962324.0000 |   9990100380 |        16 |           NULL |
 * | srv_purge_thread    |          4 |     1809792270 |    34307000 |  452447879.0000 |   1018887740 |        17 |           NULL |
 * | io_write_thread     |         19 |      951385890 |     9745450 |   50072763.0000 |    297468080 |        10 |           NULL |
 * | signal_handler      |          3 |      218026640 |    21639800 |   72675421.0000 |    154841440 |        19 |           NULL |
 * +---------------------+------------+----------------+-------------+-----------------+--------------+-----------+----------------+
11 rows in set (0.01 sec)
 *
 * (Example taken from 5.6.13)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS io_by_thread_by_latency_raw;

CREATE VIEW io_by_thread_by_latency_raw AS
SELECT IF(processlist_id IS NULL, 
             SUBSTRING_INDEX(name, '/', -1), 
             CONCAT(processlist_user, '@', processlist_host)
          ) user, 
       SUM(count_star) count_star,
       SUM(sum_timer_wait) total_latency,
       MIN(min_timer_wait) min_latency,
       AVG(avg_timer_wait) avg_latency,
       MAX(max_timer_wait) max_latency,
       thread_id,
       processlist_id
  FROM performance_schema.events_waits_summary_by_thread_by_event_name 
  LEFT JOIN performance_schema.threads USING (thread_id)
 WHERE event_name LIKE 'wait/io/file/%'
   AND sum_timer_wait > 0
 GROUP BY thread_id
 ORDER BY SUM(sum_timer_wait) DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/io_global_by_file_by_bytes.sql
   ===================================================================
*/
  
  
/*
 * View: io_global_by_file_by_bytes
 *
 * Show the top global IO consumers by bytes usage by file
 *
 * mysql> SELECT * FROM io_global_by_file_by_bytes LIMIT 5;
 * +--------------------------------------------+------------+------------+-----------+-------------+---------------+-----------+------------+-----------+
 * | file                                       | count_read | total_read | avg_read  | count_write | total_written | avg_write | total      | write_pct |
 * +--------------------------------------------+------------+------------+-----------+-------------+---------------+-----------+------------+-----------+
 * | @@datadir/ibdata1                          |        147 | 4.27 MiB   | 29.71 KiB |           3 | 48.00 KiB     | 16.00 KiB | 4.31 MiB   |      1.09 |
 * | @@datadir/mysql/proc.MYD                   |        347 | 85.35 KiB  | 252 bytes |         111 | 19.08 KiB     | 176 bytes | 104.43 KiB |     18.27 |
 * | @@datadir/ib_logfile0                      |          6 | 68.00 KiB  | 11.33 KiB |           8 | 4.00 KiB      | 512 bytes | 72.00 KiB  |      5.56 |
 * | /opt/mysql/5.5.33/share/english/errmsg.sys |          3 | 43.68 KiB  | 14.56 KiB |           0 | 0 bytes       | 0 bytes   | 43.68 KiB  |      0.00 |
 * | /opt/mysql/5.5.33/share/charsets/Index.xml |          1 | 17.89 KiB  | 17.89 KiB |           0 | 0 bytes       | 0 bytes   | 17.89 KiB  |      0.00 |
 * +--------------------------------------------+------------+------------+-----------+-------------+---------------+-----------+------------+-----------+
 * 5 rows in set (0.01 sec) *
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS io_global_by_file_by_bytes;

CREATE SQL SECURITY INVOKER VIEW io_global_by_file_by_bytes AS
SELECT format_path(file_name) AS file, 
       count_read, 
       format_bytes(sum_number_of_bytes_read) AS total_read,
       format_bytes(IFNULL(sum_number_of_bytes_read / count_read, 0)) AS avg_read,
       count_write, 
       format_bytes(sum_number_of_bytes_write) AS total_written,
       format_bytes(IFNULL(sum_number_of_bytes_write / count_write, 0.00)) AS avg_write,
       format_bytes(sum_number_of_bytes_read + sum_number_of_bytes_write) AS total, 
       IFNULL(ROUND(100-((sum_number_of_bytes_read/(sum_number_of_bytes_read+sum_number_of_bytes_write))*100), 2), 0.00) AS write_pct 
  FROM performance_schema.file_summary_by_instance
 ORDER BY sum_number_of_bytes_read + sum_number_of_bytes_write DESC;

/*
 * View: io_global_by_file_by_bytes_raw
 *
 * Show the top global IO consumers by bytes usage by file
 *
 * mysql> SELECT * FROM io_global_by_file_by_bytes_raw LIMIT 5;
 * +------------------------------------------------------+------------+------------+------------+-------------+---------------+------------+---------+-----------+
 * | file                                                 | count_read | total_read | avg_read   | count_write | total_written | avg_write  | total   | write_pct |
 * +------------------------------------------------------+------------+------------+------------+-------------+---------------+------------+---------+-----------+
 * | /Users/mark/sandboxes/msb_5_5_33/data/ibdata1        |        147 |    4472832 | 30427.4286 |           3 |         49152 | 16384.0000 | 4521984 |      1.09 |
 * | /Users/mark/sandboxes/msb_5_5_33/data/mysql/proc.MYD |        347 |      87397 |   251.8646 |         111 |         19536 |   176.0000 |  106933 |     18.27 |
 * | /Users/mark/sandboxes/msb_5_5_33/data/ib_logfile0    |          6 |      69632 | 11605.3333 |           8 |          4096 |   512.0000 |   73728 |      5.56 |
 * | /opt/mysql/5.5.33/share/english/errmsg.sys           |          3 |      44724 | 14908.0000 |           0 |             0 |     0.0000 |   44724 |      0.00 |
 * | /opt/mysql/5.5.33/share/charsets/Index.xml           |          1 |      18317 | 18317.0000 |           0 |             0 |     0.0000 |   18317 |      0.00 |
 * +------------------------------------------------------+------------+------------+------------+-------------+---------------+------------+---------+-----------+
 * 5 rows in set (0.00 sec)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS io_global_by_file_by_bytes_raw;

CREATE SQL SECURITY INVOKER VIEW io_global_by_file_by_bytes_raw AS
SELECT file_name AS file, 
       count_read, 
       sum_number_of_bytes_read AS total_read,
       IFNULL(sum_number_of_bytes_read / count_read, 0) AS avg_read,
       count_write, 
       sum_number_of_bytes_write AS total_written,
       IFNULL(sum_number_of_bytes_write / count_write, 0.00) AS avg_write,
       sum_number_of_bytes_read + sum_number_of_bytes_write AS total, 
       IFNULL(ROUND(100-((sum_number_of_bytes_read/(sum_number_of_bytes_read+sum_number_of_bytes_write))*100), 2), 0.00) AS write_pct 
  FROM performance_schema.file_summary_by_instance
 ORDER BY sum_number_of_bytes_read + sum_number_of_bytes_write DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/io_global_by_file_by_latency.sql
   ===================================================================
*/
  
  
/*
 * View: io_global_by_file_by_latency
 *
 * Show the top global IO consumers by latency by file
 *
 * mysql> select * from io_global_by_file_by_latency limit 5;
 * +-----------------------------------------------------------------+------------+---------------+------------+--------------+-------------+---------------+------------+--------------+
 * | file                                                            | count_star | total_latency | count_read | read_latency | count_write | write_latency | count_misc | misc_latency |
 * +-----------------------------------------------------------------+------------+---------------+------------+--------------+-------------+---------------+------------+--------------+
 * | @@datadir/wbsys/wait_classes_global_by_avg_latency_raw.frm~ |         24 | 451.99 ms     |          0 | 0 ps         |           4 | 108.07 us     |         20 | 451.88 ms    |
 * | @@datadir/wbsys/innodb_buffer_stats_by_schema_raw.frm~      |         24 | 379.84 ms     |          0 | 0 ps         |           4 | 108.88 us     |         20 | 379.73 ms    |
 * | @@datadir/wbsys/io_by_thread_by_latency_raw.frm~            |         24 | 379.46 ms     |          0 | 0 ps         |           4 | 101.37 us     |         20 | 379.36 ms    |
 * | @@datadir/ibtmp1                                                |         53 | 373.45 ms     |          0 | 0 ps         |          48 | 246.08 ms     |          5 | 127.37 ms    |
 * | @@datadir/wbsys/statement_analysis_raw.frm~                 |         24 | 353.14 ms     |          0 | 0 ps         |           4 | 94.96 us      |         20 | 353.04 ms    |
 * +-----------------------------------------------------------------+------------+---------------+------------+--------------+-------------+---------------+------------+--------------+
 * 5 rows in set (0.00 sec)
 *
 * Versions: 5.6+
 */

DROP VIEW IF EXISTS io_global_by_file_by_latency;

CREATE SQL SECURITY INVOKER VIEW io_global_by_file_by_latency AS
SELECT wbsys.format_path(file_name) AS file, 
       count_star, 
       wbsys.format_time(sum_timer_wait) AS total_latency,
       count_read,
       wbsys.format_time(sum_timer_read) AS read_latency,
       count_write,
       wbsys.format_time(sum_timer_write) AS write_latency,
       count_misc,
       wbsys.format_time(sum_timer_misc) AS misc_latency
  FROM performance_schema.file_summary_by_instance
 ORDER BY sum_timer_wait DESC;

/*
 * View: io_global_by_file_by_latency_raw
 *
 * Show the top global IO consumers by latency by file
 *
 * mysql> select * from io_global_by_file_by_latency_raw limit 5;
 * +--------------------------------------------------------------------------------------------+------------+---------------+------------+--------------+-------------+---------------+------------+--------------+
 * | file                                                                                       | count_star | total_latency | count_read | read_latency | count_write | write_latency | count_misc | misc_latency |
 * +--------------------------------------------------------------------------------------------+------------+---------------+------------+--------------+-------------+---------------+------------+--------------+
 * | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/wait_classes_global_by_avg_latency_raw.frm~ |         30 |  513959738110 |          0 |            0 |           5 |     132130960 |         25 | 513827607150 |
 * | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/innodb_buffer_stats_by_schema_raw.frm~      |         30 |  490149888410 |          0 |            0 |           5 |     483887040 |         25 | 489666001370 |
 * | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/io_by_thread_by_latency_raw.frm~            |         30 |  427724241620 |          0 |            0 |           5 |     131399580 |         25 | 427592842040 |
 * | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/innodb_buffer_stats_by_schema.frm~          |         30 |  406392559950 |          0 |            0 |           5 |     104082160 |         25 | 406288477790 |
 * | /Users/mark/sandboxes/msb_5_7_2/data/wbsys/statement_analysis_raw.frm~                 |         30 |  395527510430 |          0 |            0 |           5 |     118724840 |         25 | 395408785590 |
 * +--------------------------------------------------------------------------------------------+------------+---------------+------------+--------------+-------------+---------------+------------+--------------+
 * 5 rows in set (0.00 sec)
 *
 * Versions: 5.6+
 */

DROP VIEW IF EXISTS io_global_by_file_by_latency_raw;

CREATE SQL SECURITY INVOKER VIEW io_global_by_file_by_latency_raw AS
SELECT file_name AS file, 
       count_star, 
       sum_timer_wait AS total_latency,
       count_read,
       sum_timer_read AS read_latency,
       count_write,
       sum_timer_write AS write_latency,
       count_misc,
       sum_timer_misc AS misc_latency
  FROM performance_schema.file_summary_by_instance
 ORDER BY sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/io_global_by_wait_by_bytes.sql
   ===================================================================
*/
  
  
/*
 * View: io_global_by_wait_by_bytes
 *
 * Show the top global IO consumer classes by bytes usage
 *
 * mysql> select * from io_global_by_wait_by_bytes;
 * +--------------------+------------+---------------+-------------+-------------+-------------+------------+------------+-----------+-------------+---------------+-------------+-----------------+
 * | event_name         | count_star | total_latency | min_latency | avg_latency | max_latency | count_read | total_read | avg_read  | count_write | total_written | avg_written | total_requested |
 * +--------------------+------------+---------------+-------------+-------------+-------------+------------+------------+-----------+-------------+---------------+-------------+-----------------+
 * | myisam/dfile       |     163681 | 983.13 ms     | 379.08 ns   | 6.01 µs     | 22.06 ms    |      68737 | 127.31 MiB | 1.90 KiB  |     1012221 | 121.52 MiB    | 126 bytes   | 248.83 MiB      |
 * | myisam/kfile       |       1775 | 375.13 ms     | 1.02 µs     | 211.34 µs   | 35.15 ms    |      54066 | 9.97 MiB   | 193 bytes |      428257 | 12.40 MiB     | 30 bytes    | 22.37 MiB       |
 * | sql/FRM            |      57889 | 8.40 s        | 19.44 ns    | 145.05 µs   | 336.71 ms   |       8009 | 2.60 MiB   | 341 bytes |       14675 | 2.91 MiB      | 208 bytes   | 5.51 MiB        |
 * | sql/global_ddl_log |        164 | 75.96 ms      | 5.72 µs     | 463.19 µs   | 7.43 ms     |         20 | 80.00 KiB  | 4.00 KiB  |          76 | 304.00 KiB    | 4.00 KiB    | 384.00 KiB      |
 * | sql/file_parser    |        419 | 601.37 ms     | 1.96 µs     | 1.44 ms     | 37.14 ms    |         66 | 42.01 KiB  | 652 bytes |          64 | 226.98 KiB    | 3.55 KiB    | 268.99 KiB      |
 * | sql/binlog         |        190 | 6.79 s        | 1.56 µs     | 35.76 ms    | 4.21 s      |         52 | 60.54 KiB  | 1.16 KiB  |           0 | 0 bytes       | 0 bytes     | 60.54 KiB       |
 * | sql/ERRMSG         |          5 | 2.03 s        | 8.61 µs     | 405.40 ms   | 2.03 s      |          3 | 51.82 KiB  | 17.27 KiB |           0 | 0 bytes       | 0 bytes     | 51.82 KiB       |
 * | mysys/charset      |          3 | 196.52 µs     | 17.61 µs    | 65.51 µs    | 137.33 µs   |          1 | 17.83 KiB  | 17.83 KiB |           0 | 0 bytes       | 0 bytes     | 17.83 KiB       |
 * | sql/partition      |         81 | 18.87 ms      | 888.08 ns   | 232.92 µs   | 4.67 ms     |         66 | 2.75 KiB   | 43 bytes  |           8 | 288 bytes     | 36 bytes    | 3.04 KiB        |
 * | sql/dbopt          |     329166 | 26.95 s       | 2.06 µs     | 81.89 µs    | 178.71 ms   |          0 | 0 bytes    | 0 bytes   |           9 | 585 bytes     | 65 bytes    | 585 bytes       |
 * | sql/relaylog       |          7 | 1.18 ms       | 838.84 ns   | 168.30 µs   | 892.70 µs   |          0 | 0 bytes    | 0 bytes   |           1 | 120 bytes     | 120 bytes   | 120 bytes       |
 * | mysys/cnf          |          5 | 171.61 µs     | 303.26 ns   | 34.32 µs    | 115.21 µs   |          3 | 56 bytes   | 19 bytes  |           0 | 0 bytes       | 0 bytes     | 56 bytes        |
 * | sql/pid            |          3 | 220.55 µs     | 29.29 µs    | 73.52 µs    | 143.11 µs   |          0 | 0 bytes    | 0 bytes   |           1 | 5 bytes       | 5 bytes     | 5 bytes         |
 * | sql/casetest       |          1 | 121.19 µs     | 121.19 µs   | 121.19 µs   | 121.19 µs   |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     | 0 bytes         |
 * | sql/binlog_index   |          5 | 593.47 µs     | 1.07 µs     | 118.69 µs   | 535.90 µs   |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     | 0 bytes         |
 * | sql/misc           |         23 | 2.73 ms       | 65.14 µs    | 118.50 µs   | 255.31 µs   |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     | 0 bytes         |
 * +--------------------+------------+---------------+-------------+-------------+-------------+------------+------------+-----------+-------------+---------------+-------------+-----------------+
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS io_global_by_wait_by_bytes;

CREATE SQL SECURITY INVOKER VIEW io_global_by_wait_by_bytes AS
SELECT SUBSTRING_INDEX(event_name, '/', -2) event_name,
       count_star,
       format_time(sum_timer_wait) total_latency,
       format_time(min_timer_wait) min_latency,
       format_time(avg_timer_wait) avg_latency,
       format_time(max_timer_wait) max_latency,
       count_read,
       format_bytes(sum_number_of_bytes_read) total_read,
       format_bytes(IFNULL(sum_number_of_bytes_read / count_read, 0)) avg_read,
       count_write,
       format_bytes(sum_number_of_bytes_write) total_written,
       format_bytes(IFNULL(sum_number_of_bytes_write / count_write, 0)) avg_written,
       format_bytes(sum_number_of_bytes_write + sum_number_of_bytes_read) total_requested
  FROM performance_schema.file_summary_by_event_name
 WHERE event_name LIKE 'wait/io/file/%' 
   AND count_star > 0
 ORDER BY sum_number_of_bytes_write + sum_number_of_bytes_read DESC;

/*
 * View: io_global_by_wait_by_bytes_raw
 *
 * Show the top global IO consumer classes by bytes usage
 *
 * mysql> select * from io_global_by_wait_by_bytes_raw;
 * +-------------------------+------------+---------------+-------------+-------------+--------------+------------+------------+------------+-------------+---------------+-------------+-----------------+
 * | event_name              | count_star | total_latency | min_latency | avg_latency | max_latency  | count_read | total_read | avg_read   | count_write | total_written | avg_written | total_requested |
 * +-------------------------+------------+---------------+-------------+-------------+--------------+------------+------------+------------+-------------+---------------+-------------+-----------------+
 * | innodb/innodb_data_file |        151 |  334405721910 |     8399560 |  2214607429 | 107444600380 |        147 |    4472832 | 30427.4286 |           0 |             0 |      0.0000 |         4472832 |
 * | sql/FRM                 |        555 |  147752034170 |      674830 |   266219881 |  57705900850 |        270 |     112174 |   415.4593 |           0 |             0 |      0.0000 |          112174 |
 * | innodb/innodb_log_file  |         22 |   56776429970 |     2476890 |  2580746816 |  18883021430 |          6 |      69632 | 11605.3333 |           5 |          2560 |    512.0000 |           72192 |
 * | sql/ERRMSG              |          5 |   11862056180 |    14883960 |  2372411236 |  11109473700 |          3 |      44724 | 14908.0000 |           0 |             0 |      0.0000 |           44724 |
 * | mysys/charset           |          3 |    7256869230 |    19796270 |  2418956410 |   7198498320 |          1 |      18317 | 18317.0000 |           0 |             0 |      0.0000 |           18317 |
 * | myisam/kfile            |        135 |   10194698280 |      784160 |    75516283 |   2593514950 |         40 |       9216 |   230.4000 |          33 |          1017 |     30.8182 |           10233 |
 * | myisam/dfile            |         68 |   10527909730 |      772850 |   154822201 |   7600014630 |          9 |       6667 |   740.7778 |           0 |             0 |      0.0000 |            6667 |
 * | sql/pid                 |          3 |     216507330 |    41296580 |    72169110 |    100617530 |          0 |          0 |     0.0000 |           1 |             6 |      6.0000 |               6 |
 * | sql/casetest            |          5 |     185261570 |     4105530 |    37052314 |    113488310 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |               0 |
 * | sql/global_ddl_log      |          2 |      21538010 |     3121560 |    10769005 |     18416450 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |               0 |
 * | sql/dbopt               |         10 |    1004267680 |     1164930 |   100426768 |    939894930 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |               0 |
 * +-------------------------+------------+---------------+-------------+-------------+--------------+------------+------------+------------+-------------+---------------+-------------+-----------------+
 * 11 rows in set (0.03 sec) *
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS io_global_by_wait_by_bytes_raw;

CREATE SQL SECURITY INVOKER VIEW io_global_by_wait_by_bytes_raw AS
SELECT SUBSTRING_INDEX(event_name, '/', -2) event_name,
       count_star,
       sum_timer_wait total_latency,
       min_timer_wait min_latency,
       avg_timer_wait avg_latency,
       max_timer_wait max_latency,
       count_read,
       sum_number_of_bytes_read total_read,
       IFNULL(sum_number_of_bytes_read / count_read, 0) avg_read,
       count_write,
       sum_number_of_bytes_write total_written,
       IFNULL(sum_number_of_bytes_write / count_write, 0) avg_written,
       sum_number_of_bytes_write + sum_number_of_bytes_read total_requested
  FROM performance_schema.file_summary_by_event_name
 WHERE event_name LIKE 'wait/io/file/%' 
   AND count_star > 0
 ORDER BY sum_number_of_bytes_write + sum_number_of_bytes_read DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/io_global_by_wait_by_latency.sql
   ===================================================================
*/
  
  
/*
 * View: io_global_by_wait_by_latency
 *
 * Show the top global IO consumers by latency
 *
 * mysql> SELECT * FROM io_global_by_wait_by_latency;
 * +-------------------------+------------+---------------+-------------+-------------+--------------+---------------+--------------+------------+------------+-----------+-------------+---------------+-------------+
 * | event_name              | count_star | total_latency | avg_latency | max_latency | read_latency | write_latency | misc_latency | count_read | total_read | avg_read  | count_write | total_written | avg_written |
 * +-------------------------+------------+---------------+-------------+-------------+--------------+---------------+--------------+------------+------------+-----------+-------------+---------------+-------------+
 * | sql/file_parser         |       5433 | 30.20 s       | 5.56 ms     | 203.65 ms   | 22.08 ms     | 24.89 ms      | 30.16 s      |         24 | 6.18 KiB   | 264 bytes |         737 | 2.15 MiB      | 2.99 KiB    |
 * | innodb/innodb_data_file |       1344 | 1.52 s        | 1.13 ms     | 350.70 ms   | 203.82 ms    | 450.96 ms     | 868.21 ms    |        147 | 2.30 MiB   | 16.00 KiB |        1001 | 53.61 MiB     | 54.84 KiB   |
 * | innodb/innodb_log_file  |        828 | 893.48 ms     | 1.08 ms     | 30.11 ms    | 16.32 ms     | 705.89 ms     | 171.27 ms    |          6 | 68.00 KiB  | 11.33 KiB |         413 | 2.19 MiB      | 5.42 KiB    |
 * | myisam/kfile            |       7642 | 242.34 ms     | 31.71 us    | 19.27 ms    | 73.60 ms     | 23.48 ms      | 145.26 ms    |        758 | 135.63 KiB | 183 bytes |        4386 | 232.52 KiB    | 54 bytes    |
 * | myisam/dfile            |      12540 | 223.47 ms     | 17.82 us    | 32.50 ms    | 87.76 ms     | 16.97 ms      | 118.74 ms    |       5390 | 4.49 MiB   | 873 bytes |        1448 | 2.65 MiB      | 1.88 KiB    |
 * | csv/metadata            |          8 | 28.98 ms      | 3.62 ms     | 20.15 ms    | 399.27 us    | 0 ps          | 28.58 ms     |          2 | 70 bytes   | 35 bytes  |           0 | 0 bytes       | 0 bytes     |
 * | mysys/charset           |          3 | 24.24 ms      | 8.08 ms     | 24.15 ms    | 24.15 ms     | 0 ps          | 93.18 us     |          1 | 17.31 KiB  | 17.31 KiB |           0 | 0 bytes       | 0 bytes     |
 * | sql/ERRMSG              |          5 | 20.43 ms      | 4.09 ms     | 19.31 ms    | 20.32 ms     | 0 ps          | 103.20 us    |          3 | 58.97 KiB  | 19.66 KiB |           0 | 0 bytes       | 0 bytes     |
 * | mysys/cnf               |          5 | 11.37 ms      | 2.27 ms     | 11.28 ms    | 11.29 ms     | 0 ps          | 78.22 us     |          3 | 56 bytes   | 19 bytes  |           0 | 0 bytes       | 0 bytes     |
 * | sql/dbopt               |         57 | 4.04 ms       | 70.92 us    | 843.70 us   | 0 ps         | 186.43 us     | 3.86 ms      |          0 | 0 bytes    | 0 bytes   |           7 | 431 bytes     | 62 bytes    |
 * | csv/data                |          4 | 411.55 us     | 102.89 us   | 234.89 us   | 0 ps         | 0 ps          | 411.55 us    |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     |
 * | sql/misc                |         22 | 340.38 us     | 15.47 us    | 33.77 us    | 0 ps         | 0 ps          | 340.38 us    |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     |
 * | archive/data            |         39 | 277.86 us     | 7.12 us     | 16.18 us    | 0 ps         | 0 ps          | 277.86 us    |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     |
 * | sql/pid                 |          3 | 218.03 us     | 72.68 us    | 154.84 us   | 0 ps         | 21.64 us      | 196.39 us    |          0 | 0 bytes    | 0 bytes   |           1 | 6 bytes       | 6 bytes     |
 * | sql/casetest            |          5 | 197.15 us     | 39.43 us    | 126.31 us   | 0 ps         | 0 ps          | 197.15 us    |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     |
 * | sql/global_ddl_log      |          2 | 14.60 us      | 7.30 us     | 12.12 us    | 0 ps         | 0 ps          | 14.60 us     |          0 | 0 bytes    | 0 bytes   |           0 | 0 bytes       | 0 bytes     |
 * +-------------------------+------------+---------------+-------------+-------------+--------------+---------------+--------------+------------+------------+-----------+-------------+---------------+-------------+
 * 17 rows in set (0.00 sec)
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.6+
 */

DROP VIEW IF EXISTS io_global_by_wait_by_latency;

CREATE SQL SECURITY INVOKER VIEW io_global_by_wait_by_latency AS
SELECT SUBSTRING_INDEX(event_name, '/', -2) event_name,
       count_star,
       format_time(sum_timer_wait) total_latency,
       format_time(avg_timer_wait) avg_latency,
       format_time(max_timer_wait) max_latency,
       format_time(sum_timer_read) read_latency,
       format_time(sum_timer_write) write_latency,
       format_time(sum_timer_misc) misc_latency,
       count_read,
       format_bytes(sum_number_of_bytes_read) total_read,
       format_bytes(IFNULL(sum_number_of_bytes_read / count_read, 0)) avg_read,
       count_write,
       format_bytes(sum_number_of_bytes_write) total_written,
       format_bytes(IFNULL(sum_number_of_bytes_write / count_write, 0)) avg_written
  FROM performance_schema.file_summary_by_event_name 
 WHERE event_name LIKE 'wait/io/file/%'
   AND count_star > 0
 ORDER BY sum_timer_wait DESC;

/*
 * View: io_global_by_wait_by_latency_raw
 *
 * Show the top global IO consumers by latency, without any formatting
 *
 * mysql> select * from io_global_by_wait_by_latency_raw;
 * +-------------------------+------------+----------------+-------------+--------------+--------------+---------------+----------------+------------+------------+------------+-------------+---------------+-------------+
 * | event_name              | count_star | total_latency  | avg_latency | max_latency  | read_latency | write_latency | misc_latency   | count_read | total_read | avg_read   | count_write | total_written | avg_written |
 * +-------------------------+------------+----------------+-------------+--------------+--------------+---------------+----------------+------------+------------+------------+-------------+---------------+-------------+
 * | sql/file_parser         |       5945 | 33615441247050 |  5654405471 | 203652881640 |  22093704230 |   27389668280 | 33565957874540 |         26 |       7008 |   269.5385 |         808 |       2479209 |   3068.3280 |
 * | sql/FRM                 |       6332 |  1755386796800 |   277224688 | 145624702340 | 519139578620 |    1677016640 |  1234570201540 |       2040 |     865905 |   424.4632 |         439 |        103445 |    235.6378 |
 * | innodb/innodb_data_file |       1344 |  1522989889460 |  1133176798 | 350700491310 | 203817502460 |  450959403830 |   868212983170 |        147 |    2408448 | 16384.0000 |        1001 |      56213504 |  56157.3467 |
 * | innodb/innodb_log_file  |        828 |   893475794640 |  1079076921 |  30108124800 |  16315236730 |  705886928240 |   171273629670 |          6 |      69632 | 11605.3333 |         413 |       2294272 |   5555.1380 |
 * | myisam/kfile            |       7826 |   246001992860 |    31433883 |  19265276810 |  74419162870 |   23923730090 |   147659099900 |        770 |     141058 |   183.1922 |        4516 |        249602 |     55.2706 |
 * | myisam/dfile            |      13431 |   228191713620 |    16989882 |  32500163410 |  89162969350 |   17341973610 |   121686770660 |       5819 |    4873176 |   837.4594 |        1577 |       2853444 |   1809.4128 |
 * | csv/metadata            |          8 |    28975194560 |  3621899320 |  20148109020 |    399265620 |             0 |    28575928940 |          2 |         70 |    35.0000 |           0 |             0 |      0.0000 |
 * | mysys/charset           |          3 |    24244722970 |  8081574072 |  24151547420 |  24151547420 |             0 |       93175550 |          1 |      17722 | 17722.0000 |           0 |             0 |      0.0000 |
 * | sql/ERRMSG              |          5 |    20427386850 |  4085477370 |  19312386730 |  20324183100 |             0 |      103203750 |          3 |      60390 | 20130.0000 |           0 |             0 |      0.0000 |
 * | mysys/cnf               |          5 |    11366169230 |  2273233846 |  11283602460 |  11287953040 |             0 |       78216190 |          3 |         56 |    18.6667 |           0 |             0 |      0.0000 |
 * | sql/dbopt               |         57 |     4042348570 |    70918224 |    843703380 |            0 |     186430270 |     3855918300 |          0 |          0 |     0.0000 |           7 |           431 |     61.5714 |
 * | csv/data                |          4 |      411548280 |   102887070 |    234886080 |            0 |             0 |      411548280 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |
 * | sql/misc                |         24 |      369128240 |    15380092 |     33771660 |            0 |             0 |      369128240 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |
 * | archive/data            |         39 |      277856540 |     7124169 |     16180840 |            0 |             0 |      277856540 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |
 * | sql/pid                 |          3 |      218026640 |    72675421 |    154841440 |            0 |      21639800 |      196386840 |          0 |          0 |     0.0000 |           1 |             6 |      6.0000 |
 * | sql/casetest            |          5 |      197152150 |    39430430 |    126310080 |            0 |             0 |      197152150 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |
 * | sql/global_ddl_log      |          2 |       14604980 |     7302490 |     12120550 |            0 |             0 |       14604980 |          0 |          0 |     0.0000 |           0 |             0 |      0.0000 |
 * +-------------------------+------------+----------------+-------------+--------------+--------------+---------------+----------------+------------+------------+------------+-------------+---------------+-------------+
 * 17 rows in set (0.00 sec)
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.6+
 */

DROP VIEW IF EXISTS io_global_by_wait_by_latency_raw;

CREATE SQL SECURITY INVOKER VIEW io_global_by_wait_by_latency_raw AS
SELECT SUBSTRING_INDEX(event_name, '/', -2) event_name,
       count_star,
       sum_timer_wait total_latency,
       avg_timer_wait avg_latency,
       max_timer_wait max_latency,
       sum_timer_read read_latency,
       sum_timer_write write_latency,
       sum_timer_misc misc_latency,
       count_read,
       sum_number_of_bytes_read total_read,
       IFNULL(sum_number_of_bytes_read / count_read, 0) avg_read,
       count_write,
       sum_number_of_bytes_write total_written,
       IFNULL(sum_number_of_bytes_write / count_write, 0) avg_written
  FROM performance_schema.file_summary_by_event_name 
 WHERE event_name LIKE 'wait/io/file/%'
   AND count_star > 0
 ORDER BY sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/memory_by_user.sql
   ===================================================================
*/
  
  
/*
 * View: memory_by_user_by_current_bytes
 *
 * Summarizes memory use by user
 *
 * mysql> select * from memory_by_user_by_current_bytes;
 * +------+--------------------+-------------------+-------------------+-------------------+-----------------+
 * | user | current_count_used | current_allocated | current_avg_alloc | current_max_alloc | total_allocated |
 * +------+--------------------+-------------------+-------------------+-------------------+-----------------+
 * | NULL |               2085 | 10.64 MiB         | 5.23 KiB          | 8.00 MiB          | 30.16 MiB       |
 * | root |               1401 | 1.09 MiB          | 815 bytes         | 334.97 KiB        | 42.73 MiB       |
 * | mark |                201 | 496.08 KiB        | 2.47 KiB          | 334.97 KiB        | 5.50 MiB        |
 * +------+--------------------+-------------------+-------------------+-------------------+-----------------+
 * 
 * Versions: 5.7.2+
 */

DROP VIEW IF EXISTS memory_by_user_by_current_bytes;

CREATE SQL SECURITY INVOKER VIEW memory_by_user_by_current_bytes AS
SELECT user,
       SUM(current_count_used) AS current_count_used,
       wbsys.format_bytes(SUM(current_number_of_bytes_used)) AS current_allocated,
       wbsys.format_bytes(SUM(current_number_of_bytes_used) / SUM(current_count_used)) AS current_avg_alloc,
       wbsys.format_bytes(MAX(current_number_of_bytes_used)) AS current_max_alloc,
       wbsys.format_bytes(SUM(sum_number_of_bytes_alloc)) AS total_allocated
  FROM performance_schema.memory_summary_by_user_by_event_name
 GROUP BY user
 ORDER BY SUM(current_number_of_bytes_used) DESC;

/*
 * View: memory_by_user_by_current_bytes_raw
 *
 * Summarizes memory use by user
 *
 * mysql> select * from memory_by_user_by_current_bytes_raw;
 * +------+--------------------+-------------------+-------------------+-------------------+-----------------+
 * | user | current_count_used | current_allocated | current_avg_alloc | current_max_alloc | total_allocated |
 * +------+--------------------+-------------------+-------------------+-------------------+-----------------+
 * | NULL |               2085 |          11159333 |         5352.1981 |           8390792 |        31627304 |
 * | root |               1399 |           1124553 |          803.8263 |            343008 |        45426133 |
 * | mark |                201 |            507990 |         2527.3134 |            343008 |         5769804 |
 * +------+--------------------+-------------------+-------------------+-------------------+-----------------+
 * 3 rows in set (0.01 sec)
 * 
 * Versions: 5.7.2+
 */

DROP VIEW IF EXISTS memory_by_user_by_current_bytes_raw;

CREATE SQL SECURITY INVOKER VIEW memory_by_user_by_current_bytes_raw AS
SELECT user,
       SUM(current_count_used) AS current_count_used,
       SUM(current_number_of_bytes_used) AS current_allocated,
       SUM(current_number_of_bytes_used) / SUM(current_count_used) AS current_avg_alloc,
       MAX(current_number_of_bytes_used) AS current_max_alloc,
       SUM(sum_number_of_bytes_alloc) AS total_allocated
  FROM performance_schema.memory_summary_by_user_by_event_name
 GROUP BY user
 ORDER BY SUM(current_number_of_bytes_used) DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/memory_global_by_current_allocated.sql
   ===================================================================
*/
  
  
/* 
 * View: memory_global_by_current_allocated
 * 
 * Shows the current memory usage within the server globally broken down by allocation type
 *
 * mysql> select * from memory_global_by_current_allocated;
 * +----------------------------------------+---------------+---------------+-------------------+------------+------------+----------------+
 * | event_name                             | current_count | current_alloc | current_avg_alloc | high_count | high_alloc | high_avg_alloc |
 * +----------------------------------------+---------------+---------------+-------------------+------------+------------+----------------+
 * | memory/sql/TABLE_SHARE::mem_root       |           269 | 568.21 KiB    | 2.11 KiB          |        339 | 706.04 KiB | 2.08 KiB       |
 * | memory/sql/TABLE                       |           214 | 366.56 KiB    | 1.71 KiB          |        245 | 481.13 KiB | 1.96 KiB       |
 * | memory/sql/sp_head::main_mem_root      |            32 | 334.97 KiB    | 10.47 KiB         |        421 | 9.73 MiB   | 23.66 KiB      |
 * | memory/sql/Filesort_buffer::sort_keys  |             1 | 255.89 KiB    | 255.89 KiB        |          1 | 256.00 KiB | 256.00 KiB     |
 * | memory/mysys/array_buffer              |            82 | 121.66 KiB    | 1.48 KiB          |       1124 | 852.55 KiB | 777 bytes      |
 * ...
 * +----------------------------------------+---------------+---------------+-------------------+------------+------------+----------------+
 * 26 rows in set (0.30 sec)
 *
 * Versions: 5.7.2+
 */

DROP VIEW IF EXISTS memory_global_by_current_allocated;

CREATE SQL SECURITY INVOKER VIEW memory_global_by_current_allocated AS
SELECT event_name,
       current_count_used AS current_count,
       format_bytes(current_number_of_bytes_used) AS current_alloc,
       format_bytes(current_number_of_bytes_used / current_count_used) AS current_avg_alloc,
       high_count_used AS high_count,
       format_bytes(high_number_of_bytes_used) AS high_alloc,
       format_bytes(high_number_of_bytes_used / high_count_used) AS high_avg_alloc
  FROM performance_schema.memory_summary_global_by_event_name
 WHERE current_number_of_bytes_used > 0
 ORDER BY current_number_of_bytes_used DESC;

/* 
 * View: memory_global_by_current_allocated_raw
 * 
 * Shows the current memory usage within the server globally broken down by allocation type
 *
 * mysql> select * from memory_global_by_current_allocated_raw;
 * +----------------------------------------+---------------+---------------+-------------------+------------+------------+----------------+
 * | event_name                             | current_count | current_alloc | current_avg_alloc | high_count | high_alloc | high_avg_alloc |
 * +----------------------------------------+---------------+---------------+-------------------+------------+------------+----------------+
 * | memory/sql/TABLE_SHARE::mem_root       |           270 |        582656 |         2157.9852 |        339 |     722984 |      2132.6962 |
 * | memory/sql/TABLE                       |           214 |        375353 |         1753.9860 |        245 |     492672 |      2010.9061 |
 * | memory/sql/sp_head::main_mem_root      |            32 |        343008 |        10719.0000 |        421 |   10200008 |     24228.0475 |
 * | memory/sql/Filesort_buffer::sort_keys  |             1 |        262036 |       262036.0000 |          1 |     262140 |    262140.0000 |
 * | memory/mysys/array_buffer              |            82 |        124576 |         1519.2195 |       1124 |     873008 |       776.6975 |
 * ...
 * +----------------------------------------+---------------+---------------+-------------------+------------+------------+----------------+
 * 26 rows in set (0.00 sec)
 *
 * Versions: 5.7.2+
 */

DROP VIEW IF EXISTS memory_global_by_current_allocated_raw;

CREATE SQL SECURITY INVOKER VIEW memory_global_by_current_allocated_raw AS
SELECT event_name,
       current_count_used AS current_count,
       current_number_of_bytes_used AS current_alloc,
       current_number_of_bytes_used / current_count_used AS current_avg_alloc,
       high_count_used AS high_count,
       high_number_of_bytes_used AS high_alloc,
       high_number_of_bytes_used / high_count_used AS high_avg_alloc
  FROM performance_schema.memory_summary_global_by_event_name
 WHERE current_number_of_bytes_used > 0
 ORDER BY current_number_of_bytes_used DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/memory_global_total.sql
   ===================================================================
*/
  
  
/* 
 * View: memory_global_total
 * 
 * Shows the total memory usage within the server globally
 *
 * mysql> select * from memory_global_total;
 * +-----------------+
 * | total_allocated |
 * +-----------------+
 * | 1.35 MiB        |
 * +-----------------+
 * 1 row in set (0.18 sec)
 *
 * Versions: 5.7.2+
 */

DROP VIEW IF EXISTS memory_global_total;

CREATE SQL SECURITY INVOKER VIEW memory_global_total AS
SELECT wbsys.format_bytes(SUM(CURRENT_NUMBER_OF_BYTES_USED)) total_allocated
  FROM performance_schema.memory_summary_global_by_event_name;

/* 
 * View: memory_global_total_raw
 * 
 * Shows the total memory usage within the server globally
 *
 * mysql> select * from memory_global_total_raw;
 * +-----------------+
 * | total_allocated |
 * +-----------------+
 * |         1420023 |
 * +-----------------+
 * 1 row in set (0.01 sec)
 *
 * Versions: 5.7.2+
 */

DROP VIEW IF EXISTS memory_global_total_raw;

CREATE SQL SECURITY INVOKER VIEW memory_global_total_raw AS
SELECT SUM(CURRENT_NUMBER_OF_BYTES_USED) total_allocated
  FROM performance_schema.memory_summary_global_by_event_name;

/* 
   ===================================================================
   Contents from file: views/p_s/schema_index_statistics.sql
   ===================================================================
*/
  
  
/*
 * View: schema_index_statistics
 *
 * Mimic INDEX_STATISTICS from Google et al
 * However, order by the total wait time descending - top indexes are most contended
 *
 * Versions: 5.6.2+
 *
 * mysql> select * from schema_index_statistics limit 5;
 * +------------------+-------------+------------+---------------+----------------+---------------+----------------+--------------+----------------+--------------+----------------+
 * | table_schema     | table_name  | index_name | rows_selected | select_latency | rows_inserted | insert_latency | rows_updated | update_latency | rows_deleted | delete_latency |
 * +------------------+-------------+------------+---------------+----------------+---------------+----------------+--------------+----------------+--------------+----------------+
 * | mem              | mysqlserver | PRIMARY    |          6208 | 108.27 ms      |             0 | 0 ps           |         5470 | 1.47 s         |            0 | 0 ps           |
 * | mem              | innodb      | PRIMARY    |          4666 | 76.27 ms       |             0 | 0 ps           |         4454 | 571.47 ms      |            0 | 0 ps           |
 * | mem              | connection  | PRIMARY    |          1064 | 20.98 ms       |             0 | 0 ps           |         1064 | 457.30 ms      |            0 | 0 ps           |
 * | mem              | environment | PRIMARY    |          5566 | 151.17 ms      |             0 | 0 ps           |          694 | 252.57 ms      |            0 | 0 ps           |
 * | mem              | querycache  | PRIMARY    |          1698 | 27.99 ms       |             0 | 0 ps           |         1698 | 371.72 ms      |            0 | 0 ps           |
 * +------------------+-------------+------------+---------------+----------------+---------------+----------------+--------------+----------------+--------------+----------------+
 *
 * (Example from 5.6.6)
 */

DROP VIEW IF EXISTS schema_index_statistics;

CREATE SQL SECURITY INVOKER VIEW schema_index_statistics AS
SELECT OBJECT_SCHEMA AS table_schema,
       OBJECT_NAME AS table_name,
       INDEX_NAME as index_name,
       COUNT_FETCH AS rows_selected,
       wbsys.format_time(SUM_TIMER_FETCH) AS select_latency,
       COUNT_INSERT AS rows_inserted,
       wbsys.format_time(SUM_TIMER_INSERT) AS insert_latency,
       COUNT_UPDATE AS rows_updated,
       wbsys.format_time(SUM_TIMER_UPDATE) AS update_latency,
       COUNT_DELETE AS rows_deleted,
       wbsys.format_time(SUM_TIMER_INSERT) AS delete_latency
  FROM performance_schema.table_io_waits_summary_by_index_usage
 WHERE index_name IS NOT NULL
 ORDER BY sum_timer_wait DESC;

/*
 * View: schema_index_statistics_raw
 *
 * Mimic INDEX_STATISTICS from Google et al
 * However, order by the total wait time descending - top indexes are most contended
 *
 * Versions: 5.6.2+
 *
 * mysql> SELECT * FROM schema_index_statistics_raw LIMIT 5;
 * +---------------+----------------------+-------------------+---------------+----------------+---------------+----------------+--------------+----------------+--------------+----------------+
 * | table_schema  | table_name           | index_name        | rows_selected | select_latency | rows_inserted | insert_latency | rows_updated | update_latency | rows_deleted | delete_latency |
 * +---------------+----------------------+-------------------+---------------+----------------+---------------+----------------+--------------+----------------+--------------+----------------+
 * | common_schema | _global_sql_tokens   | PRIMARY           |          1886 |     1129676730 |             0 |              0 |            0 |              0 |         1878 |              0 |
 * | common_schema | _script_statements   | PRIMARY           |          4606 |     4212160680 |             0 |              0 |            0 |              0 |            0 |              0 |
 * | common_schema | _global_qs_variables | declaration_depth |           256 |     1650193090 |             0 |              0 |           32 |     1372148050 |            0 |              0 |
 * | common_schema | _global_qs_variables | PRIMARY           |             0 |              0 |             0 |              0 |            0 |              0 |           16 |              0 |
 * | common_schema | metadata             | PRIMARY           |             5 |       76730810 |             0 |              0 |            4 |      114310170 |            0 |              0 |
 * +---------------+----------------------+-------------------+---------------+----------------+---------------+----------------+--------------+----------------+--------------+----------------+
 * 5 rows in set (0.01 sec) *
 *
 * (Example from 5.6.6)
 */

DROP VIEW IF EXISTS schema_index_statistics_raw;

CREATE SQL SECURITY INVOKER VIEW schema_index_statistics_raw AS
SELECT OBJECT_SCHEMA AS table_schema,
       OBJECT_NAME AS table_name,
       INDEX_NAME as index_name,
       COUNT_FETCH AS rows_selected,
       SUM_TIMER_FETCH AS select_latency,
       COUNT_INSERT AS rows_inserted,
       SUM_TIMER_INSERT AS insert_latency,
       COUNT_UPDATE AS rows_updated,
       SUM_TIMER_UPDATE AS update_latency,
       COUNT_DELETE AS rows_deleted,
       SUM_TIMER_INSERT AS delete_latency
  FROM performance_schema.table_io_waits_summary_by_index_usage
 WHERE index_name IS NOT NULL
 ORDER BY sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/schema_table_statistics.sql
   ===================================================================
*/
  
  
/* 
 * View: schema_table_statistics
 *
 * Mimic TABLE_STATISTICS from Google et al ordered by the total wait time descending
 * 
 * mysql> select * from schema_table_statistics limit 1\G
 * *************************** 1. row ***************************
 *                  table_schema: mem
 *                    table_name: mysqlserver
 *                  rows_fetched: 27087
 *                 fetch_latency: 442.72 ms
 *                 rows_inserted: 2
 *                insert_latency: 185.04 µs 
 *                  rows_updated: 5096
 *                update_latency: 1.39 s
 *                  rows_deleted: 0
 *                delete_latency: 0 ps
 *              io_read_requests: 2565
 *                 io_read_bytes: 1121627
 *               io_read_latency: 10.07 ms
 *             io_write_requests: 1691
 *                io_write_bytes: 128383
 *              io_write_latency: 14.17 ms
 *              io_misc_requests: 2698
 *               io_misc_latency: 433.66 ms
 * 
 * (Example from 5.6.6)
 *
 * Versions: 5.6.2+
 *
 */ 
 
DROP VIEW IF EXISTS schema_table_statistics;

CREATE SQL SECURITY INVOKER VIEW schema_table_statistics AS 
SELECT pst.object_schema AS table_schema,
       pst.object_name AS table_name,
       pst.count_fetch AS rows_fetched,
       wbsys.format_time(pst.sum_timer_fetch) AS fetch_latency,
       pst.count_insert AS rows_inserted,
       wbsys.format_time(pst.sum_timer_insert) AS insert_latency,
       pst.count_update AS rows_updated,
       wbsys.format_time(pst.sum_timer_update) AS update_latency,
       pst.count_delete AS rows_deleted,
       wbsys.format_time(pst.sum_timer_delete) AS delete_latency,
       SUM(fsbi.count_read) AS io_read_requests,
       wbsys.format_bytes(SUM(fsbi.sum_number_of_bytes_read)) AS io_read,
       wbsys.format_time(SUM(fsbi.sum_timer_read)) AS io_read_latency,
       SUM(fsbi.count_write) AS io_write_requests,
       wbsys.format_bytes(SUM(fsbi.sum_number_of_bytes_write)) AS io_write,
       wbsys.format_time(SUM(fsbi.sum_timer_write)) AS io_write_latency,
       SUM(fsbi.count_misc) AS io_misc_requests,
       wbsys.format_time(SUM(fsbi.sum_timer_misc)) AS io_misc_latency
  FROM performance_schema.table_io_waits_summary_by_table AS pst
  LEFT JOIN performance_schema.file_summary_by_instance AS fsbi
    ON pst.object_schema = extract_schema_from_file_name(fsbi.file_name)
   AND pst.object_name = extract_table_from_file_name(fsbi.file_name)
 GROUP BY pst.object_schema, pst.object_name
 ORDER BY pst.sum_timer_wait DESC;

/* 
 * View: schema_table_statistics_raw
 *
 * Mimic TABLE_STATISTICS from Google et al ordered by the total wait time descending
 * 
 * mysql> SELECT * FROM schema_table_statistics_raw LIMIT 1\G
 * *************************** 1. row ***************************
 *      table_schema: common_schema
 *        table_name: help_content
 *      rows_fetched: 0
 *     fetch_latency: 0
 *     rows_inserted: 169
 *    insert_latency: 409815527680
 *      rows_updated: 0
 *    update_latency: 0
 *      rows_deleted: 0
 *    delete_latency: 0
 *  io_read_requests: 14
 *           io_read: 1180
 *   io_read_latency: 52406770
 * io_write_requests: 131
 *          io_write: 11719246
 *  io_write_latency: 133726902790
 *  io_misc_requests: 61
 *   io_misc_latency: 209081089750
 * 1 row in set (1.24 sec)
 * 
 * (Example from 5.6.6)
 *
 * Versions: 5.6.2+
 *
 */ 
 
DROP VIEW IF EXISTS schema_table_statistics_raw;

CREATE SQL SECURITY INVOKER VIEW schema_table_statistics_raw AS 
SELECT pst.object_schema AS table_schema,
       pst.object_name AS table_name,
       pst.count_fetch AS rows_fetched,
       pst.sum_timer_fetch AS fetch_latency,
       pst.count_insert AS rows_inserted,
       pst.sum_timer_insert AS insert_latency,
       pst.count_update AS rows_updated,
       pst.sum_timer_update AS update_latency,
       pst.count_delete AS rows_deleted,
       pst.sum_timer_delete AS delete_latency,
       SUM(fsbi.count_read) AS io_read_requests,
       SUM(fsbi.sum_number_of_bytes_read) AS io_read,
       SUM(fsbi.sum_timer_read) AS io_read_latency,
       SUM(fsbi.count_write) AS io_write_requests,
       SUM(fsbi.sum_number_of_bytes_write) AS io_write,
       SUM(fsbi.sum_timer_write) AS io_write_latency,
       SUM(fsbi.count_misc) AS io_misc_requests,
       SUM(fsbi.sum_timer_misc) AS io_misc_latency
  FROM performance_schema.table_io_waits_summary_by_table AS pst
  LEFT JOIN performance_schema.file_summary_by_instance AS fsbi
    ON pst.object_schema = extract_schema_from_file_name(fsbi.file_name)
   AND pst.object_name = extract_table_from_file_name(fsbi.file_name)
 GROUP BY pst.object_schema, pst.object_name
 ORDER BY pst.sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/schema_table_statistics_with_buffer.sql
   ===================================================================
*/
  
  
/* 
 * View: schema_table_statistics_with_buffer
 *
 * Mimic TABLE_STATISTICS from Google et al
 * However, order by the total wait time descending, and add in more statistics
 * such as caching stats for the InnoDB buffer pool with InnoDB tables
 *
 * mysql> select * from schema_table_statistics_with_buffer limit 1\G
 * *************************** 1. row ***************************
 *                  table_schema: mem
 *                    table_name: mysqlserver
 *                  rows_fetched: 27087
 *                 fetch_latency: 442.72 ms
 *                 rows_inserted: 2
 *                insert_latency: 185.04 µs 
 *                  rows_updated: 5096
 *                update_latency: 1.39 s
 *                  rows_deleted: 0
 *                delete_latency: 0 ps
 *              io_read_requests: 2565
 *                 io_read_bytes: 1121627
 *               io_read_latency: 10.07 ms
 *             io_write_requests: 1691
 *                io_write_bytes: 128383
 *              io_write_latency: 14.17 ms
 *              io_misc_requests: 2698
 *               io_misc_latency: 433.66 ms
 *           innodb_buffer_pages: 19
 *    innodb_buffer_pages_hashed: 19
 *       innodb_buffer_pages_old: 19
 * innodb_buffer_bytes_allocated: 311296
 *      innodb_buffer_bytes_data: 1924
 *     innodb_buffer_rows_cached: 2
 * 
 * (Example from 5.6.6)
 *
 * Versions: 5.6.2+
 *
 */ 
 
DROP VIEW IF EXISTS schema_table_statistics_with_buffer;

CREATE SQL SECURITY INVOKER VIEW schema_table_statistics_with_buffer AS 
SELECT pst.object_schema AS table_schema,
       pst.object_name AS table_name,
       pst.count_fetch AS rows_fetched,
       format_time(pst.sum_timer_fetch) AS fetch_latency,
       pst.count_insert AS rows_inserted,
       format_time(pst.sum_timer_insert) AS insert_latency,
       pst.count_update AS rows_updated,
       format_time(pst.sum_timer_update) AS update_latency,
       pst.count_delete AS rows_deleted,
       format_time(pst.sum_timer_delete) AS delete_latency,
       SUM(fsbi.count_read) AS io_read_requests,
       format_bytes(SUM(fsbi.sum_number_of_bytes_read)) AS io_read,
       format_time(SUM(fsbi.sum_timer_read)) AS io_read_latency,
       SUM(fsbi.count_write) AS io_write_requests,
       format_bytes(SUM(fsbi.sum_number_of_bytes_write)) AS io_write,
       format_time(SUM(fsbi.sum_timer_write)) AS io_write_latency,
       SUM(fsbi.count_misc) AS io_misc_requests,
       format_time(SUM(fsbi.sum_timer_misc)) AS io_misc_latency,
       ibp.allocated AS innodb_buffer_allocated,
       ibp.data AS innodb_buffer_data,
       ibp.pages AS innodb_buffer_pages,
       ibp.pages_hashed AS innodb_buffer_pages_hashed,
       ibp.pages_old AS innodb_buffer_pages_old,
       ibp.rows_cached AS innodb_buffer_rows_cached
  FROM performance_schema.table_io_waits_summary_by_table AS pst
  LEFT JOIN performance_schema.file_summary_by_instance AS fsbi
    ON pst.object_schema = extract_schema_from_file_name(fsbi.file_name)
   AND pst.object_name = extract_table_from_file_name(fsbi.file_name)
  LEFT JOIN wbsys.innodb_buffer_stats_by_table AS ibp
    ON pst.object_schema = ibp.object_schema
   AND pst.object_name = ibp.object_name
 GROUP BY pst.object_schema, pst.object_name
 ORDER BY pst.sum_timer_wait DESC;

/* 
 * View: schema_table_statistics_with_buffer_raw
 *
 * Mimic TABLE_STATISTICS from Google et al
 * However, order by the total wait time descending, and add in more statistics
 * such as caching stats for the InnoDB buffer pool with InnoDB tables
 *
 * mysql> SELECT * FROM schema_table_statistics_with_buffer_raw LIMIT 1\G
 * *************************** 1. row ***************************
 *               table_schema: common_schema
 *                 table_name: help_content
 *               rows_fetched: 0
 *              fetch_latency: 0
 *              rows_inserted: 169
 *             insert_latency: 409815527680
 *               rows_updated: 0
 *             update_latency: 0
 *               rows_deleted: 0
 *             delete_latency: 0
 *           io_read_requests: 14
 *                    io_read: 1180
 *            io_read_latency: 52406770
 *          io_write_requests: 131
 *                   io_write: 11719246
 *           io_write_latency: 133726902790
 *           io_misc_requests: 61
 *            io_misc_latency: 209081089750
 *    innodb_buffer_allocated: 688128
 *         innodb_buffer_data: 423667
 *        innodb_buffer_pages: 42
 * innodb_buffer_pages_hashed: 42
 *    innodb_buffer_pages_old: 42
 *  innodb_buffer_rows_cached: 210
 * 1 row in set (1.18 sec)
 * 
 * (Example from 5.6.6)
 *
 * Versions: 5.6.2+
 *
 */ 
 
DROP VIEW IF EXISTS schema_table_statistics_with_buffer_raw;

CREATE SQL SECURITY INVOKER VIEW schema_table_statistics_with_buffer_raw AS 
SELECT pst.object_schema AS table_schema,
       pst.object_name AS table_name,
       pst.count_fetch AS rows_fetched,
       pst.sum_timer_fetch AS fetch_latency,
       pst.count_insert AS rows_inserted,
       pst.sum_timer_insert AS insert_latency,
       pst.count_update AS rows_updated,
       pst.sum_timer_update AS update_latency,
       pst.count_delete AS rows_deleted,
       pst.sum_timer_delete AS delete_latency,
       SUM(fsbi.count_read) AS io_read_requests,
       SUM(fsbi.sum_number_of_bytes_read) AS io_read,
       SUM(fsbi.sum_timer_read) AS io_read_latency,
       SUM(fsbi.count_write) AS io_write_requests,
       SUM(fsbi.sum_number_of_bytes_write) AS io_write,
       SUM(fsbi.sum_timer_write) AS io_write_latency,
       SUM(fsbi.count_misc) AS io_misc_requests,
       SUM(fsbi.sum_timer_misc) AS io_misc_latency,
       ibp.allocated AS innodb_buffer_allocated,
       ibp.data AS innodb_buffer_data,
       ibp.pages AS innodb_buffer_pages,
       ibp.pages_hashed AS innodb_buffer_pages_hashed,
       ibp.pages_old AS innodb_buffer_pages_old,
       ibp.rows_cached AS innodb_buffer_rows_cached
  FROM performance_schema.table_io_waits_summary_by_table AS pst
  LEFT JOIN performance_schema.file_summary_by_instance AS fsbi
    ON pst.object_schema = extract_schema_from_file_name(fsbi.file_name)
   AND pst.object_name = extract_table_from_file_name(fsbi.file_name)
  LEFT JOIN wbsys.innodb_buffer_stats_by_table_raw AS ibp
    ON pst.object_schema = ibp.object_schema
   AND pst.object_name = ibp.object_name
 GROUP BY pst.object_schema, pst.object_name
 ORDER BY pst.sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/schema_tables_with_full_table_scans.sql
   ===================================================================
*/
  
  
/* 
 * View: schema_tables_with_full_table_scans
 *
 * Find tables that are being accessed by full table scans
 * ordering by the number of rows scanned descending
 *
 * mysql> select * from schema_tables_with_full_table_scans limit 5;
 * +------------------+-------------------+-------------------+
 * | object_schema    | object_name       | rows_full_scanned |
 * +------------------+-------------------+-------------------+
 * | mem              | rule_alarms       |              1210 |
 * | mem30__advisors  | advisor_schedules |              1021 |
 * | mem30__inventory | agent             |               498 |
 * | mem              | dc_p_string       |               449 |
 * | mem30__inventory | mysqlserver       |               294 |
 * +------------------+-------------------+-------------------+
 *
 * Versions: 5.6.2+
 *
 */

DROP VIEW IF EXISTS schema_tables_with_full_table_scans;

CREATE SQL SECURITY INVOKER VIEW schema_tables_with_full_table_scans AS
SELECT object_schema, 
       object_name,
       count_read AS rows_full_scanned
  FROM performance_schema.table_io_waits_summary_by_index_usage 
 WHERE index_name IS NULL
   AND count_read > 0
 ORDER BY count_read DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/schema_unused_indexes.sql
   ===================================================================
*/
  
  
/* 
 * View: schema_unused_indexes
 * 
 * Find indexes that have had no events against them (and hence, no usage)
 *
 * mysql> select * from schema_unused_indexes;
 * +---------------------------+-------------------------------+--------------------------------------------------------+
 * | object_schema             | object_name                   | index_name                                             |
 * +---------------------------+-------------------------------+--------------------------------------------------------+
 * | mem                       | dc_p_double                   | PRIMARY                                                |
 * | mem                       | dc_p_double                   | end_time                                               |
 * | mem                       | dc_p_long                     | PRIMARY                                                |
 * | mem                       | dc_p_long                     | end_time                                               |
 * | mem                       | dc_p_string                   | begin_time                                             |
 * | mem                       | dc_p_string                   | end_time                                               |
 * ...
 *
 * Versions: 5.6.2+
 *
 */

DROP VIEW IF EXISTS schema_unused_indexes;
 
CREATE SQL SECURITY INVOKER VIEW schema_unused_indexes AS
SELECT object_schema,
       object_name,
       index_name
  FROM performance_schema.table_io_waits_summary_by_index_usage 
 WHERE index_name IS NOT NULL
   AND count_star = 0
   AND object_schema != 'mysql'
 ORDER BY object_schema, object_name;
/* 
   ===================================================================
   Contents from file: views/p_s/statement_analysis.sql
   ===================================================================
*/
  
  
/*
 * View: statement_analysis
 *
 * Lists a normalized statement view with aggregated statistics,
 * mimics the MySQL Enterprise Monitor Query Analysis view,
 * ordered by the total execution time per normalized statement
 * 
 * mysql> select * from statement_analysis limit 5;
 * +-------------------------------------------------------------------+------+-----------+------------+-----------+------------+---------------+-------------+-------------+--------------+-----------+---------------+---------------+-------------------+------------+-----------------+-------------+-------------------+----------------------------------+---------------------+---------------------+
 * | query                                                             | db   | full_scan | exec_count | err_count | warn_count | total_latency | max_latency | avg_latency | lock_latency | rows_sent | rows_sent_avg | rows_examined | rows_examined_avg | tmp_tables | tmp_disk_tables | rows_sorted | sort_merge_passes | digest                           | first_seen          | last_seen           |
 * +-------------------------------------------------------------------+------+-----------+------------+-----------+------------+---------------+-------------+-------------+--------------+-----------+---------------+---------------+-------------------+------------+-----------------+-------------+-------------------+----------------------------------+---------------------+---------------------+
 * | INSERT INTO `mem__quan` . `nor ... nDuration` = IF ( VALUES ( ... | mem  |           |    7604979 |         0 |          0 | 27.36h        | 831.62 ms   | 12.95 ms    | 2.24h        |         0 |             0 |             0 |                 0 |          0 |               0 |           0 |                 0 | 361bbfa1983c4ec4901cb2237f256138 | 2013-12-04 20:05:34 | 2013-12-18 17:50:34 |
 * | COMMIT                                                            | mem  |           |   23437411 |         0 |          0 | 7.60h         | 3.69 s      | 1.17 ms     | 0 ps         |         0 |             0 |             0 |                 0 |          0 |               0 |           0 |                 0 | e51be358a1cbf99c1acab35cc1c6b683 | 2013-12-04 20:04:08 | 2013-12-18 17:50:55 |
 * | INSERT INTO `mem__quan` . `nor ... `lastSeen` ) ) , `lastSeen` )  | mem  |           |    7603328 |       644 |          0 | 5.34h         | 840.00 ms   | 2.53 ms     | 00:39:13.55  |         0 |             0 |             0 |                 0 |          0 |               0 |           0 |                 0 | 6134e9d6f25eb8e6cddf11f6938f202a | 2013-12-04 20:05:34 | 2013-12-18 17:50:34 |
 * | SELECT `fsstatisti0_` . `fs` A ...  `fsstatisti0_` . `timestamp`  | mem  |           |     160159 |         0 |          0 | 4.90h         | 1.89 s      | 110.24 ms   | 00:01:36.52  | 229526539 |          1433 |     229532297 |              1433 |          0 |               0 |           0 |                 0 | 8e87c94dc6567954202c0dbe56e626e8 | 2013-12-04 20:04:54 | 2013-12-18 17:50:44 |
 * | SELECT `blockdevic0_` . `hid`  ... vic0_` . `hid` = ? FOR UPDATE  | mem  |           |    1400390 |         0 |          0 | 1.89h         | 2.01 s      | 4.86 ms     | 00:03:31.29  |   1400381 |             1 |       1400381 |                 1 |          0 |               0 |           0 |                 0 | ac9fdbb96f5ebdebdaa963251162e6d4 | 2013-12-04 20:04:53 | 2013-12-18 17:50:44 |
 * +-------------------------------------------------------------------+------+-----------+------------+-----------+------------+---------------+-------------+-------------+--------------+-----------+---------------+---------------+-------------------+------------+-----------------+-------------+-------------------+----------------------------------+---------------------+---------------------+
 * 5 rows in set (0.02 sec)
 *
 * (Example from 5.6.14)
 *
 * Versions: 5.6.9+
 */

DROP VIEW IF EXISTS statement_analysis;

CREATE SQL SECURITY INVOKER VIEW statement_analysis AS
SELECT wbsys.format_statement(DIGEST_TEXT) AS query,
       SCHEMA_NAME AS db,
       IF(SUM_NO_GOOD_INDEX_USED > 0 OR SUM_NO_INDEX_USED > 0, '*', '') AS full_scan,
       COUNT_STAR AS exec_count,
       SUM_ERRORS AS err_count,
       SUM_WARNINGS AS warn_count,
       wbsys.format_time(SUM_TIMER_WAIT) AS total_latency,
       wbsys.format_time(MAX_TIMER_WAIT) AS max_latency,
       wbsys.format_time(AVG_TIMER_WAIT) AS avg_latency,
       wbsys.format_time(SUM_LOCK_TIME) AS lock_latency,
       SUM_ROWS_SENT AS rows_sent,
       ROUND(SUM_ROWS_SENT / COUNT_STAR) AS rows_sent_avg,
       SUM_ROWS_EXAMINED AS rows_examined,
       ROUND(SUM_ROWS_EXAMINED / COUNT_STAR)  AS rows_examined_avg,
       SUM_CREATED_TMP_TABLES AS tmp_tables,
       SUM_CREATED_TMP_DISK_TABLES AS tmp_disk_tables,
       SUM_SORT_ROWS AS rows_sorted,
       SUM_SORT_MERGE_PASSES AS sort_merge_passes,
       DIGEST AS digest,
       FIRST_SEEN AS first_seen,
       LAST_SEEN as last_seen
  FROM performance_schema.events_statements_summary_by_digest
ORDER BY SUM_TIMER_WAIT DESC;

/*
 * View: statement_analysis_raw
 *
 * Lists a normalized statement view with aggregated statistics,
 * mimics the MySQL Enterprise Monitor Query Analysis view,
 * ordered by the total execution time per normalized statement
 * 
 * mysql> select * from statement_analysis_raw limit 5;
 * +----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+------+-----------+------------+-----------+------------+-------------------+---------------+--------------+-----------+---------------+---------------+-------------------+------------+-----------------+-------------+-------------------+----------------------------------+---------------------+---------------------+
 * | query                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | db   | full_scan | exec_count | err_count | warn_count | total_latency     | max_latency   | avg_latency  | rows_sent | rows_sent_avg | rows_examined | rows_examined_avg | tmp_tables | tmp_disk_tables | rows_sorted | sort_merge_passes | digest                           | first_seen          | last_seen           |
 * +----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+------+-----------+------------+-----------+------------+-------------------+---------------+--------------+-----------+---------------+---------------+-------------------+------------+-----------------+-------------+-------------------+----------------------------------+---------------------+---------------------+
 * | INSERT INTO `mem__quan` . `normalized_statements_by_server_by_schema_data` ( `bytesMax` , `bytesMin` , `bytesTotal` , `collectionDuration` , `createdTmpDiskTables` , `createdTmpTables` , `errorCount` , `execCount` , `execTimeMax` , `execTimeMin` , `execTimeTotal` , `lockTimeTotal` , `noGoodIndexUsedCount` , `noIndexUsedCount` , `rowsExaminedTotal` , `rowsMax` , `rowsMin` , `rowsTotal` , `selectFullJoin` , `selectFullRangeJoin` , `selectRange` , `selectRangeCheck` , `selectScan` , `sortMergePasses` , `sortRange` , `sortRows` , `sortScan` , TIMESTAMP , `warningCount` , `round_robin_bin` , `normalized_statement_by_server_by_schema_id` ) VALUES (...) ON DUPLICATE KEY UPDATE `bytesMax` = IF ( VALUES ( `timestamp` ) >= `timestamp` , VALUES ( `bytesMax` ) , `bytesMax` ) , `bytesMin` = IF ( VALUES ( `timestamp` ) >= `timestamp` , VALUES ( `bytesMin` ) , `bytesMin` ) , `bytesTotal` = IF ( VALUES ( `timestamp` ) >= `timestamp` , VALUES ( `bytesTotal` ) , `bytesTotal` ) , `collectionDuration` = IF ( VALUES ( ... | mem  |           |    7603726 |         0 |          0 | 98469133153813000 |  831617448000 |  12950115000 |         0 |             0 |             0 |                 0 |          0 |               0 |           0 |                 0 | 361bbfa1983c4ec4901cb2237f256138 | 2013-12-04 20:05:34 | 2013-12-18 17:46:35 |
 * | COMMIT                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   | mem  |           |   23433832 |         0 |          0 | 27348748128106000 | 3692522134000 |   1167062000 |         0 |             0 |             0 |                 0 |          0 |               0 |           0 |                 0 | e51be358a1cbf99c1acab35cc1c6b683 | 2013-12-04 20:04:08 | 2013-12-18 17:47:18 |
 * | INSERT INTO `mem__quan` . `normalized_statements_by_server_by_schema` ( `firstSeen` , `lastSeen` , `normalized_statement_id` , `schema` , SERVER , `id` ) VALUES (...) ON DUPLICATE KEY UPDATE `firstSeen` = COALESCE ( `LEAST` ( `firstSeen` , VALUES ( `firstSeen` ) ) , `firstSeen` ) , `lastSeen` = COALESCE ( `GREATEST` ( `lastSeen` , VALUES ( `lastSeen` ) ) , `lastSeen` )                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      | mem  |           |    7602075 |       640 |          0 | 19210969261301000 |  840000846000 |   2527069000 |         0 |             0 |             0 |                 0 |          0 |               0 |           0 |                 0 | 6134e9d6f25eb8e6cddf11f6938f202a | 2013-12-04 20:05:34 | 2013-12-18 17:46:35 |
 * | SELECT `fsstatisti0_` . `fs` AS `col_0_0_` , `fsstatisti0_` . `timestamp` AS `col_0_1_` , `fsstatisti0_` . `free` AS `col_0_2_` , `fsstatisti0_` . `total` AS `col_0_3_` , `fsstatisti0_` . `perSec` AS `col_0_4_` , `fsstatisti0_` . `perSecIntercept` AS `col_0_5_` , `fsstatisti0_` . `perMinute` AS `col_0_6_` , `fsstatisti0_` . `perMinuteIntercept` AS `col_0_7_` , `fsstatisti0_` . `perHour` AS `col_0_8_` , `fsstatisti0_` . `perHourIntercept` AS `col_0_9_` , `fsstatisti0_` . `perDay` AS `col_0_10_` , `fsstatisti0_` . `perDayIntercept` AS `col_0_11_` FROM `mem__instruments` . `FsStatistics` `fsstatisti0_` WHERE ( `fsstatisti0_` . `fs` IN (?) ) AND `fsstatisti0_` . `timestamp` >= ? AND `fsstatisti0_` . `timestamp` <= ? ORDER BY `fsstatisti0_` . `fs` , `fsstatisti0_` . `timestamp`                                                                                                                                                                                                                                          | mem  |           |     160131 |         0 |          0 | 17654951709541000 | 1889823768000 | 110253178000 | 229486238 |          1433 |     229491996 |              1433 |          0 |               0 |           0 |                 0 | 8e87c94dc6567954202c0dbe56e626e8 | 2013-12-04 20:04:54 | 2013-12-18 17:47:05 |
 * | SELECT `blockdevic0_` . `hid` AS `hid1829_0_` , `blockdevic0_` . `id` AS `id2_1829_0_` , `blockdevic0_` . `name` AS `name3_1829_0_` , `blockdevic0_` . `hasName` AS `hasName4_1829_0_` , `blockdevic0_` . `timestamp` AS `timestamp5_1829_0_` , `blockdevic0_` . `os` AS `os6_1829_0_` , `blockdevic0_` . `hasOs` AS `hasOs7_1829_0_` FROM `mem__inventory` . `BlockDevice` `blockdevic0_` WHERE `blockdevic0_` . `hid` = ? FOR UPDATE                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   | mem  |           |    1400146 |         0 |          0 |  6800141572899000 | 2007817708000 |   4856737000 |   1400137 |             1 |       1400137 |                 1 |          0 |               0 |           0 |                 0 | ac9fdbb96f5ebdebdaa963251162e6d4 | 2013-12-04 20:04:53 | 2013-12-18 17:47:05 |
 * +----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+------+-----------+------------+-----------+------------+-------------------+---------------+--------------+-----------+---------------+---------------+-------------------+------------+-----------------+-------------+-------------------+----------------------------------+---------------------+---------------------+
 * 5 rows in set (0.02 sec)
 *
 * (Example from 5.6.14)
 *
 * Versions: 5.6.9+
 */

DROP VIEW IF EXISTS statement_analysis_raw;

CREATE SQL SECURITY INVOKER VIEW statement_analysis_raw AS
SELECT DIGEST_TEXT AS query,
       SCHEMA_NAME AS db,
       IF(SUM_NO_GOOD_INDEX_USED > 0 OR SUM_NO_INDEX_USED > 0, '*', '') AS full_scan,
       COUNT_STAR AS exec_count,
       SUM_ERRORS AS err_count,
       SUM_WARNINGS AS warn_count,
       SUM_TIMER_WAIT AS total_latency,
       MAX_TIMER_WAIT AS max_latency,
       AVG_TIMER_WAIT AS avg_latency,
       SUM_ROWS_SENT AS rows_sent,
       ROUND(SUM_ROWS_SENT / COUNT_STAR) AS rows_sent_avg,
       SUM_ROWS_EXAMINED AS rows_examined,
       ROUND(SUM_ROWS_EXAMINED / COUNT_STAR)  AS rows_examined_avg,
       SUM_CREATED_TMP_TABLES AS tmp_tables,
       SUM_CREATED_TMP_DISK_TABLES AS tmp_disk_tables,
       SUM_SORT_ROWS AS rows_sorted,
       SUM_SORT_MERGE_PASSES AS sort_merge_passes,
       DIGEST AS digest,
       FIRST_SEEN AS first_seen,
       LAST_SEEN as last_seen
  FROM performance_schema.events_statements_summary_by_digest
ORDER BY SUM_TIMER_WAIT DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/statements_with_errors_or_warnings.sql
   ===================================================================
*/
  
  
/*
 * View: statements_with_errors_or_warnings
 *
 * List all normalized statements that have raised errors or warnings.
 *
 * mysql> select * from statements_with_errors_or_warnings limit 5;
 * +-------------------------------------------------------------------+-----------+------------+--------+-----------+----------+-------------+---------------------+---------------------+----------------------------------+
 * | query                                                             | db        | exec_count | errors | error_pct | warnings | warning_pct | first_seen          | last_seen           | digest                           |
 * +-------------------------------------------------------------------+-----------+------------+--------+-----------+----------+-------------+---------------------+---------------------+----------------------------------+
 * | INSERT INTO `mem__quan` . `nor ... `lastSeen` ) ) , `lastSeen` )  | mem       |    7619897 |    645 |    0.0085 |        0 |      0.0000 | 2013-12-04 20:05:34 | 2013-12-18 18:48:34 | 6134e9d6f25eb8e6cddf11f6938f202a |
 * | INSERT INTO `mem__inventory` . ... , `Filesystem` ) VALUES (...)  | mem       |         54 |      3 |    5.5556 |        0 |      0.0000 | 2013-12-04 20:10:22 | 2013-12-18 18:48:44 | e225410633e09d7c6288cc390654b361 |
 * | INSERT INTO `mem__events` . `e ...  ?, ... , (_charset) ?, ... )  | mem       |        521 |      3 |    0.5758 |        0 |      0.0000 | 2013-12-04 20:04:53 | 2013-12-18 18:05:34 | 9b9714753319e42ffb3483377c1e596c |
 * | CREATE SQL SECURITY INVOKER VI ... CREATED_T` `SUM_SORT_ROWS` AS  | wbsys |          2 |      2 |  100.0000 |        0 |      0.0000 | 2013-12-18 17:46:28 | 2013-12-18 17:46:32 | 7131bbf490c4b7bdc93dd76599084996 |
 * | SELECT `mysqlconne0_` . `hid`  ... ` AS `hasProc18_1821_0_` , ... | mem       |     982158 |      1 |    0.0001 |        0 |      0.0000 | 2013-12-04 20:04:48 | 2013-12-18 18:49:04 | 24259bc30a8f8b0b53ba0283d940f938 |
 * +-------------------------------------------------------------------+-----------+------------+--------+-----------+----------+-------------+---------------------+---------------------+----------------------------------+
 * 5 rows in set (0.02 sec)
 *
 * (Example from 5.6.14)
 *
 * Versions 5.6.9+
 *
 */

DROP VIEW IF EXISTS statements_with_errors_or_warnings;

CREATE SQL SECURITY INVOKER VIEW statements_with_errors_or_warnings AS
SELECT format_statement(DIGEST_TEXT) AS query,
       SCHEMA_NAME as db,
       COUNT_STAR AS exec_count,
       SUM_ERRORS AS errors,
       (SUM_ERRORS / COUNT_STAR) * 100 as error_pct,
       SUM_WARNINGS AS warnings,
       (SUM_WARNINGS / COUNT_STAR) * 100 as warning_pct,
       FIRST_SEEN as first_seen,
       LAST_SEEN as last_seen,
       DIGEST AS digest
  FROM performance_schema.events_statements_summary_by_digest
 WHERE SUM_ERRORS > 0
    OR SUM_WARNINGS > 0
ORDER BY SUM_ERRORS DESC, SUM_WARNINGS DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/statements_with_full_table_scans.sql
   ===================================================================
*/
  
  
/*
 * View: statements_with_full_table_scans
 *
 * Lists all normalized statements that use have done a full table scan
 * ordered by number the percentage of times a full scan was done,
 * then by the number of times the statement executed
 *
 * mysql> select * from statements_with_full_table_scans limit 5;
 * +-------------------------------------------------------------------+-------+------------+---------------------+--------------------------+-------------------+-----------+---------------+---------------+-------------------+---------------------+---------------------+----------------------------------+
 * | query                                                             | db    | exec_count | no_index_used_count | no_good_index_used_count | no_index_used_pct | rows_sent | rows_examined | rows_sent_avg | rows_examined_avg | first_seen          | last_seen           | digest                           |
 * +-------------------------------------------------------------------+-------+------------+---------------------+--------------------------+-------------------+-----------+---------------+---------------+-------------------+---------------------+---------------------+----------------------------------+
 * | SELECT `InterfaceAddress4` . ` ... . `hid` WHERE `Os` . `id` = ?  | mem   |     355189 |              355277 |                        0 |               100 |    404862 |       4987778 |             1 |                14 | 2013-12-04 20:04:54 | 2013-12-18 18:54:34 | 42d79211cc5e82e32e00340bc6a50b14 |
 * | SELECT COUNT ( * ) FROM `INFOR ... NE = ? AND `SUPPORT` IN (...)  | mysql |     172765 |              172776 |                        0 |               100 |    172773 |       1554966 |             1 |                 9 | 2013-12-04 20:04:52 | 2013-12-18 18:54:34 | 491ee7143ca1d98f36c24d7eb6d25272 |
 * | SELECT CAST ( `SUM_NUMBER_OF_B ... HERE `EVENT_NAME` = ? LIMIT ?  | mysql |     100455 |              100455 |                        0 |               100 |    100455 |       3094014 |             1 |                31 | 2013-12-04 20:04:52 | 2013-12-18 18:54:34 | b5a370d80095c69a2085547e3a24f552 |
 * | SELECT COUNT ( * ) FROM `INFOR ... CHEMA` = ? AND TABLE_NAME = ?  | mysql |      80360 |               80360 |                        0 |               100 |     80360 |             0 |             1 |                 0 | 2013-12-04 20:05:37 | 2013-12-18 18:54:34 | 6567aa2ac8ad7fe7831b4114dee7c849 |
 * | SELECT DISTINCTROW `mysqlconne ... conne0_` . `socketPath` AS ... | mem   |      67832 |               67821 |                        0 |               100 |        92 |        406943 |             0 |                 6 | 2013-12-04 20:04:54 | 2013-12-18 18:54:34 | fc358ad6384cc77adff425602a0a8fc1 |
 * +-------------------------------------------------------------------+-------+------------+---------------------+--------------------------+-------------------+-----------+---------------+---------------+-------------------+---------------------+---------------------+----------------------------------+
 * 5 rows in set (0.02 sec)
 * 
 * (Example from 5.6.14)
 *
 * Versions: 5.6.9+
 *
 */

DROP VIEW IF EXISTS statements_with_full_table_scans;

CREATE SQL SECURITY INVOKER VIEW statements_with_full_table_scans AS
SELECT wbsys.format_statement(DIGEST_TEXT) AS query,
       SCHEMA_NAME as db,
       COUNT_STAR AS exec_count,
       SUM_NO_INDEX_USED AS no_index_used_count,
       SUM_NO_GOOD_INDEX_USED AS no_good_index_used_count,
       ROUND((SUM_NO_INDEX_USED / COUNT_STAR) * 100) AS no_index_used_pct,
       SUM_ROWS_SENT AS rows_sent,
       SUM_ROWS_EXAMINED AS rows_examined,
       ROUND(SUM_ROWS_SENT/COUNT_STAR) AS rows_sent_avg,
       ROUND(SUM_ROWS_EXAMINED/COUNT_STAR) AS rows_examined_avg,
       FIRST_SEEN as first_seen,
       LAST_SEEN as last_seen,
       DIGEST AS digest
  FROM performance_schema.events_statements_summary_by_digest
 WHERE SUM_NO_INDEX_USED > 0
    OR SUM_NO_GOOD_INDEX_USED > 0
ORDER BY no_index_used_pct DESC, exec_count DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/statements_with_runtimes_in_95th_percentile.sql
   ===================================================================
*/
  
  
/*
 * View: _digest_avg_latency_by_avg_us
 *
 * Helper view for _digest_95th_percentile_by_avg_us
 *
 * Versions: 5.6.5+
 */
DROP VIEW IF EXISTS _digest_avg_latency_by_avg_us;

CREATE SQL SECURITY INVOKER VIEW _digest_avg_latency_by_avg_us AS
SELECT COUNT(*) cnt, 
       ROUND(avg_timer_wait/1000000) AS avg_us
  FROM performance_schema.events_statements_summary_by_digest
 GROUP BY avg_us;

/*
 * View: _digest_95th_percentile_by_avg_us
 *
 * Helper view for statements_with_runtimes_in_95th_percentile.
 * Lists the 95th percentile runtime, for all statements
 *
 * mysql> select * from _digest_95th_percentile_by_avg_us;
 * +--------+------------+
 * | avg_us | percentile |
 * +--------+------------+
 * |    964 |     0.9525 |
 * +--------+------------+
 *
 * Versions: 5.6.5+
 */
DROP VIEW IF EXISTS _digest_95th_percentile_by_avg_us;

CREATE SQL SECURITY INVOKER VIEW _digest_95th_percentile_by_avg_us AS
SELECT s2.avg_us avg_us,
       SUM(s1.cnt)/(SELECT COUNT(*) FROM performance_schema.events_statements_summary_by_digest) percentile
  FROM _digest_avg_latency_by_avg_us AS s1
  JOIN _digest_avg_latency_by_avg_us AS s2
    ON s1.avg_us <= s2.avg_us
 GROUP BY s2.avg_us
HAVING percentile > 0.95
 ORDER BY percentile
 LIMIT 1;

/*
 * View: statements_with_runtimes_in_95th_percentile
 *
 * List all statements who's average runtime, in microseconds, is in the top 95th percentile.
 * 
 * mysql> select * from statements_with_runtimes_in_95th_percentile limit 5;
 * +-------------------------------------------------------------------+------+-----------+------------+-----------+------------+---------------+-------------+-------------+-----------+---------------+---------------+-------------------+---------------------+---------------------+----------------------------------+
 * | query                                                             | db   | full_scan | exec_count | err_count | warn_count | total_latency | max_latency | avg_latency | rows_sent | rows_sent_avg | rows_examined | rows_examined_avg | FIRST_SEEN          | LAST_SEEN           | digest                           |
 * +-------------------------------------------------------------------+------+-----------+------------+-----------+------------+---------------+-------------+-------------+-----------+---------------+---------------+-------------------+---------------------+---------------------+----------------------------------+
 * | SELECT `e` . `round_robin_bin` ...  `timestamp` = `maxes` . `ts`  | mem  | *         |         14 |         0 |          0 | 43.96 s       | 6.69 s      | 3.14 s      |        11 |             1 |        253170 |             18084 | 2013-12-04 20:05:01 | 2013-12-04 20:06:34 | 29ba002bf039bb6439357a10134407de |
 * | SELECT `e` . `round_robin_bin` ...  `timestamp` = `maxes` . `ts`  | mem  | *         |          8 |         0 |          0 | 17.89 s       | 4.12 s      | 2.24 s      |         7 |             1 |        169534 |             21192 | 2013-12-04 20:04:54 | 2013-12-04 20:05:05 | 0b1c1f91e7e9e0ff91aa49d15f540793 |
 * | SELECT `e` . `round_robin_bin` ...  `timestamp` = `maxes` . `ts`  | mem  | *         |          1 |         0 |          0 | 2.22 s        | 2.22 s      | 2.22 s      |         1 |             1 |         40322 |             40322 | 2013-12-04 20:05:39 | 2013-12-04 20:05:39 | 07b27145c8f8a3779737df5032374833 |
 * | SELECT `e` . `round_robin_bin` ...  `timestamp` = `maxes` . `ts`  | mem  | *         |          1 |         0 |          0 | 1.97 s        | 1.97 s      | 1.97 s      |         1 |             1 |         40322 |             40322 | 2013-12-04 20:05:39 | 2013-12-04 20:05:39 | a07488137ea5c1bccf3e291c50bfd21f |
 * | SELECT `e` . `round_robin_bin` ...  `timestamp` = `maxes` . `ts`  | mem  | *         |          2 |         0 |          0 | 3.91 s        | 3.91 s      | 1.96 s      |         1 |             1 |         13126 |              6563 | 2013-12-04 20:05:04 | 2013-12-04 20:06:34 | b8bddc6566366dafc7e474f67096a93b |
 * +-------------------------------------------------------------------+------+-----------+------------+-----------+------------+---------------+-------------+-------------+-----------+---------------+---------------+-------------------+---------------------+---------------------+----------------------------------+
 * 5 rows in set (1.58 sec)
 *
 * (Example from 5.6.14)
 *
 * Versions: 5.6.9+
 *
 */

DROP VIEW IF EXISTS statements_with_runtimes_in_95th_percentile;

CREATE SQL SECURITY INVOKER VIEW statements_with_runtimes_in_95th_percentile AS
SELECT format_statement(DIGEST_TEXT) AS query,
       SCHEMA_NAME as db,
       IF(SUM_NO_GOOD_INDEX_USED > 0 OR SUM_NO_INDEX_USED > 0, '*', '') AS full_scan,
       COUNT_STAR AS exec_count,
       SUM_ERRORS AS err_count,
       SUM_WARNINGS AS warn_count,
       format_time(SUM_TIMER_WAIT) AS total_latency,
       format_time(MAX_TIMER_WAIT) AS max_latency,
       format_time(AVG_TIMER_WAIT) AS avg_latency,
       SUM_ROWS_SENT AS rows_sent,
       ROUND(SUM_ROWS_SENT / COUNT_STAR) AS rows_sent_avg,
       SUM_ROWS_EXAMINED AS rows_examined,
       ROUND(SUM_ROWS_EXAMINED / COUNT_STAR) AS rows_examined_avg,
       FIRST_SEEN AS first_seen,
       LAST_SEEN AS last_seen,
       DIGEST AS digest
  FROM performance_schema.events_statements_summary_by_digest stmts
  JOIN _digest_95th_percentile_by_avg_us AS top_percentile
    ON ROUND(stmts.avg_timer_wait/1000000) >= top_percentile.avg_us
 ORDER BY AVG_TIMER_WAIT DESC;

/*
 * View: statements_with_runtimes_in_95th_percentile_raw
 *
 * List all statements who's average runtime, in microseconds, is in the top 95th percentile.
 * 
 * mysql> SELECT * FROM statements_with_runtimes_in_95th_percentile_raw LIMIT 1\G
 * *************************** 1. row ***************************
 *             query: SELECT `e` . `round_robin_bin` AS `round1_1706_0_` , `e` . `id` AS `id1706_0_` , `e` . `timestamp` AS `timestamp1706_0_` , `e` . `rxBytes` AS `rxBytes1706_0_` , `e` . `rxPackets` AS `rxPackets1706_0_` , `e` . `rxErrors` AS `rxErrors1706_0_` , `e` . `txBytes` AS `txBytes1706_0_` , `e` . `txPackets` AS `txPackets1706_0_` , `e` . `txErrors` AS `txErrors1706_0_` , `e` . `txCollisions` AS `txColli10_1706_0_` FROM `mem__instruments` . `NetworkTrafficAdvisor_NetworkTraffic` AS `e` JOIN ( SELECT `id` AS `t` , MAX ( TIMESTAMP ) AS `ts` FROM `mem__instruments` . `NetworkTrafficAdvisor_NetworkTraffic` WHERE `id` IN (?) GROUP BY `id` ORDER BY NULL ) `maxes` ON `e` . `id` = `maxes` . `t` AND `e` . `timestamp` = `maxes` . `ts`
 *                db: mem
 *         full_scan: *
 *        exec_count: 14
 *         err_count: 0
 *        warn_count: 0
 *     total_latency: 43961670267000
 *       max_latency: 6686877140000
 *       avg_latency: 3140119304000
 *         rows_sent: 11
 *     rows_sent_avg: 1
 *     rows_examined: 253170
 * rows_examined_avg: 18084
 *        first_seen: 2013-12-04 20:05:01
 *         last_seen: 2013-12-04 20:06:34
 *            digest: 29ba002bf039bb6439357a10134407de
 * 1 row in set (1.50 sec)
 *
 * (Example from 5.6.14)
 *
 * Versions: 5.6.9+
 *
 */

DROP VIEW IF EXISTS statements_with_runtimes_in_95th_percentile_raw;

CREATE SQL SECURITY INVOKER VIEW statements_with_runtimes_in_95th_percentile_raw AS
SELECT DIGEST_TEXT AS query,
       SCHEMA_NAME AS db,
       IF(SUM_NO_GOOD_INDEX_USED > 0 OR SUM_NO_INDEX_USED > 0, '*', '') AS full_scan,
       COUNT_STAR AS exec_count,
       SUM_ERRORS AS err_count,
       SUM_WARNINGS AS warn_count,
       SUM_TIMER_WAIT AS total_latency,
       MAX_TIMER_WAIT AS max_latency,
       AVG_TIMER_WAIT AS avg_latency,
       SUM_ROWS_SENT AS rows_sent,
       ROUND(SUM_ROWS_SENT / COUNT_STAR) AS rows_sent_avg,
       SUM_ROWS_EXAMINED AS rows_examined,
       ROUND(SUM_ROWS_EXAMINED / COUNT_STAR) AS rows_examined_avg,
       FIRST_SEEN as first_seen,
       LAST_SEEN as last_seen,
       DIGEST AS digest
  FROM performance_schema.events_statements_summary_by_digest stmts
  JOIN _digest_95th_percentile_by_avg_us AS top_percentile
    ON ROUND(stmts.avg_timer_wait/1000000) >= top_percentile.avg_us
 ORDER BY AVG_TIMER_WAIT DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/statements_with_sorting.sql
   ===================================================================
*/
  
  
/*
 * View: statements_with_sorting
 *
 * List all normalized statements that have done sorts,
 * ordered by sort_merge_passes, sort_scans and sort_rows, all descending
 *
 * mysql> select * from wbsys.statements_with_sorting limit 5;
 * +-------------------------------------------------------------------+-------+------------+-------------------+-----------------+-------------------+------------------+-------------+-----------------+---------------------+---------------------+----------------------------------+
 * | query                                                             | db    | exec_count | sort_merge_passes | avg_sort_merges | sorts_using_scans | sort_using_range | rows_sorted | avg_rows_sorted | first_seen          | last_seen           | digest                           |
 * +-------------------------------------------------------------------+-------+------------+-------------------+-----------------+-------------------+------------------+-------------+-----------------+---------------------+---------------------+----------------------------------+
 * | SELECT `s` . `identityId` , `s ...  `subject_id` = `s` . `id` ... | mem   |        179 |               179 |               1 |               179 |              179 |        9182 |              51 | 2013-12-04 20:05:16 | 2013-12-14 13:55:55 | e53695702a907c795766dfe4d0142bab |
 * | SELECT `cpuaverage0_` . `os` A ...  `cpuaverage0_` . `timestamp`  | mem   |          2 |                 2 |               1 |                 0 |                2 |          59 |              30 | 2013-12-04 21:08:39 | 2013-12-11 11:34:34 | e44686700b83b5b8f928773a6c4ef994 |
 * | SELECT `connection0_` . `targe ...  `connection0_` . `timestamp`  | mem   |          2 |                 2 |               1 |                 0 |                2 |          58 |              29 | 2013-12-04 21:08:39 | 2013-12-11 11:34:34 | b01a0a611acd1a5d220d7f4b7ac3e709 |
 * | SELECT `mysqlconne0_` . `mysql ...  `mysqlconne0_` . `timestamp`  | mem   |          2 |                 2 |               1 |                 0 |                2 |          58 |              29 | 2013-12-04 21:08:39 | 2013-12-11 11:34:34 | a8fc99f84a78059e8453d681ea7a75fa |
 * | SELECT `plugin_name` FROM `inf ... s` = ? ORDER BY `plugin_name`  | mysql |      24129 |                 0 |               0 |             24129 |                0 |      699741 |              29 | 2013-12-04 20:04:53 | 2013-12-18 19:10:34 | d5163a2f55578fd8e2077b8a3c00b081 |
 * +-------------------------------------------------------------------+-------+------------+-------------------+-----------------+-------------------+------------------+-------------+-----------------+---------------------+---------------------+----------------------------------+
 * 5 rows in set (0.02 sec)
 * 
 * Versions 5.6.9+
 *
 */

DROP VIEW IF EXISTS statements_with_sorting;

CREATE SQL SECURITY INVOKER VIEW statements_with_sorting AS
SELECT format_statement(DIGEST_TEXT) AS query,
       SCHEMA_NAME db,
       COUNT_STAR AS exec_count,
       SUM_SORT_MERGE_PASSES AS sort_merge_passes,
       ROUND(SUM_SORT_MERGE_PASSES / COUNT_STAR) AS avg_sort_merges,
       SUM_SORT_SCAN AS sorts_using_scans,
       SUM_SORT_RANGE AS sort_using_range,
       SUM_SORT_ROWS AS rows_sorted,
       ROUND(SUM_SORT_ROWS / COUNT_STAR) AS avg_rows_sorted,
       FIRST_SEEN as first_seen,
       LAST_SEEN as last_seen,
       DIGEST AS digest
  FROM performance_schema.events_statements_summary_by_digest
 WHERE SUM_SORT_ROWS > 0
 ORDER BY SUM_SORT_MERGE_PASSES DESC, SUM_SORT_SCAN DESC, SUM_SORT_ROWS DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/statements_with_temp_tables.sql
   ===================================================================
*/
  
  
/*
 * View: statements_with_temp_tables
 *
 * Lists all normalized statements that use temporary tables
 * ordered by number of on disk temporary tables descending first, 
 * then by the number of memory tables
 *
 * mysql> select * from statements_with_temp_tables limit 5;
 * +-------------------------------------------------------------------+-------+------------+-------------------+-----------------+--------------------------+------------------------+---------------------+---------------------+----------------------------------+
 * | query                                                             | db    | exec_count | memory_tmp_tables | disk_tmp_tables | avg_tmp_tables_per_query | tmp_tables_to_disk_pct | first_seen          | last_seen           | digest                           |
 * +-------------------------------------------------------------------+-------+------------+-------------------+-----------------+--------------------------+------------------------+---------------------+---------------------+----------------------------------+
 * | SELECT DISTINCTROW `agent0_` . ... gent` `agent0_` INNER JOIN ... | mem   |    1423339 |           1242409 |         1242393 |                        1 |                    100 | 2013-12-04 20:04:48 | 2013-12-18 19:16:48 | fa4b07b4cf70914c1119ec444684dfec |
 * | SELECT DISTINCTROW `mysqlconne ... conne0_` . `socketPath` AS ... | mem   |      67908 |             67898 |           67898 |                        1 |                    100 | 2013-12-04 20:04:54 | 2013-12-18 19:16:34 | fc358ad6384cc77adff425602a0a8fc1 |
 * | SELECT * FROM ( SELECT `digest ... ` , `sum_no_index_used` AS ... | mysql |      20113 |            140791 |           60339 |                        7 |                     43 | 2013-12-04 20:04:54 | 2013-12-18 19:16:34 | 50761c6a1818824328745d8a136b9ed6 |
 * | SELECT COUNT ( * ) FROM `INFOR ... GINS` WHERE `PLUGIN_NAME` = ?  | mysql |      44248 |             44248 |           44248 |                        1 |                    100 | 2013-12-04 20:04:52 | 2013-12-18 19:16:34 | 090f173ed2c6c8c9b8dc60718703ff56 |
 * | SELECT `plugin_name` FROM `inf ... s` = ? ORDER BY `plugin_name`  | mysql |      24136 |             24136 |           24136 |                        1 |                    100 | 2013-12-04 20:04:53 | 2013-12-18 19:16:34 | d5163a2f55578fd8e2077b8a3c00b081 |
 * +-------------------------------------------------------------------+-------+------------+-------------------+-----------------+--------------------------+------------------------+---------------------+---------------------+----------------------------------+
 * 5 rows in set (0.02 sec)
 * 
 * (Example from 5.6.14)
 *
 * Versions: 5.6.9+
 *
 */

DROP VIEW IF EXISTS statements_with_temp_tables;

CREATE SQL SECURITY INVOKER VIEW statements_with_temp_tables AS
SELECT wbsys.format_statement(DIGEST_TEXT) AS query,
       SCHEMA_NAME as db,
       COUNT_STAR AS exec_count,
       SUM_CREATED_TMP_TABLES AS memory_tmp_tables,
       SUM_CREATED_TMP_DISK_TABLES AS disk_tmp_tables,
       ROUND(SUM_CREATED_TMP_TABLES / COUNT_STAR) AS avg_tmp_tables_per_query,
       ROUND((SUM_CREATED_TMP_DISK_TABLES / SUM_CREATED_TMP_TABLES) * 100) AS tmp_tables_to_disk_pct,
       FIRST_SEEN as first_seen,
       LAST_SEEN as last_seen,
       DIGEST AS digest
  FROM performance_schema.events_statements_summary_by_digest
 WHERE SUM_CREATED_TMP_TABLES > 0
ORDER BY SUM_CREATED_TMP_DISK_TABLES DESC, SUM_CREATED_TMP_TABLES DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/user_summary_57.sql
   ===================================================================
*/
  
  
/*
 * View: user_summary
 *
 * Summarizes statement activity and connections by user
 *
 * mysql> select * from user_summary;
 * +------+------------------+---------------+-------------+---------------------+-------------------+--------------+----------------+------------------------+
 * | user | total_statements | total_latency | avg_latency | current_connections | total_connections | unique_hosts | current_memory | total_memory_allocated |
 * +------+------------------+---------------+-------------+---------------------+-------------------+--------------+----------------+------------------------+
 * | root |             5663 | 00:01:47.14   | 18.92 ms    |                   1 |                 1 |            1 | 1.41 MiB       | 543.55 MiB             |
 * | mark |              225 | 14.49 s       | 64.40 ms    |                   1 |                 1 |            1 | 707.60 KiB     | 81.02 MiB              |
 * +------+------------------+---------------+-------------+---------------------+-------------------+--------------+----------------+------------------------+
 * 2 rows in set (0.03 sec)
 * 
 * Versions: 5.6.3+
 */

DROP VIEW IF EXISTS user_summary;

CREATE SQL SECURITY INVOKER VIEW user_summary AS
SELECT accounts.user,
       SUM(essbubem.count_star) AS total_statements,
       wbsys.format_time(SUM(essbubem.sum_timer_wait)) AS total_latency,
       wbsys.format_time(SUM(essbubem.sum_timer_wait) / SUM(count_star)) AS avg_latency,
       accounts.current_connections,
       accounts.total_connections,
       COUNT(DISTINCT host) AS unique_hosts,
       mem.current_allocated AS current_memory,
       mem.total_allocated AS total_memory_allocated
  FROM performance_schema.accounts
  JOIN performance_schema.events_statements_summary_by_user_by_event_name essbubem ON accounts.user = essbubem.user
  JOIN wbsys.memory_by_user_by_current_bytes mem ON accounts.user = mem.user
 WHERE accounts.user IS NOT NULL
 GROUP BY accounts.user
 ORDER BY SUM(sum_timer_wait) DESC;

/*
 * View: user_summary_raw
 *
 * Summarizes statement activity and connections by user
 *
 * mysql> select * from user_summary_raw;
 * +------+------------------+-----------------+------------------+---------------------+-------------------+--------------+----------------+------------------------+
 * | user | total_statements | total_latency   | avg_latency      | current_connections | total_connections | unique_hosts | current_memory | total_memory_allocated |
 * +------+------------------+-----------------+------------------+---------------------+-------------------+--------------+----------------+------------------------+
 * | root |             5685 | 107175100271000 | 18852260381.8821 |                   1 |                 1 |            1 |        1459022 |              572855680 |
 * | mark |              225 |  14489223428000 | 64396548568.8889 |                   1 |                 1 |            1 |         724578 |               84958286 |
 * +------+------------------+-----------------+------------------+---------------------+-------------------+--------------+----------------+------------------------+
 * 2 rows in set (0.05 sec)
 * 
 * Versions: 5.6.3+
 */

DROP VIEW IF EXISTS user_summary_raw;

CREATE SQL SECURITY INVOKER VIEW user_summary_raw AS
SELECT accounts.user,
       SUM(essbubem.count_star) AS total_statements,
       SUM(essbubem.sum_timer_wait) AS total_latency,
       SUM(essbubem.sum_timer_wait) / SUM(count_star) AS avg_latency,
       accounts.current_connections,
       accounts.total_connections,
       COUNT(DISTINCT host) AS unique_hosts,
       mem.current_allocated AS current_memory,
       mem.total_allocated AS total_memory_allocated
  FROM performance_schema.accounts
  JOIN performance_schema.events_statements_summary_by_user_by_event_name essbubem ON accounts.user = essbubem.user
  JOIN wbsys.memory_by_user_by_current_bytes_raw mem ON accounts.user = mem.user
 WHERE accounts.user IS NOT NULL
 GROUP BY accounts.user
 ORDER BY SUM(sum_timer_wait) DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/user_summary_by_statement_type.sql
   ===================================================================
*/
  
  
/*
 * View: user_summary_by_statement_type
 *
 * Summarizes the types of statements executed by each user
 *
 * mysql> select * from user_summary_by_statement_type;
 * +------+----------------------+--------+---------------+-------------+--------------+-----------+---------------+---------------+------------+
 * | user | statement            | count  | total_latency | max_latency | lock_latency | rows_sent | rows_examined | rows_affected | full_scans |
 * +------+----------------------+--------+---------------+-------------+--------------+-----------+---------------+---------------+------------+
 * | root | create_view          |   2063 | 00:05:04.20   | 463.58 ms   | 1.42 s       |         0 |             0 |             0 |          0 |
 * | root | select               |    174 | 40.87 s       | 28.83 s     | 858.13 ms    |      5212 |        157022 |             0 |         82 |
 * | root | stmt                 |   6645 | 15.31 s       | 491.78 ms   | 0 ps         |         0 |             0 |          7951 |          0 |
 * | root | call_procedure       |     17 | 4.78 s        | 1.02 s      | 37.94 ms     |         0 |             0 |            19 |          0 |
 * | root | create_table         |     19 | 3.04 s        | 431.71 ms   | 0 ps         |         0 |             0 |             0 |          0 |
 * ...
 * +------+----------------------+--------+---------------+-------------+--------------+-----------+---------------+---------------+------------+
 * 41 rows in set (0.24 sec)
 * 
 * Versions: 5.6.3+
 */

DROP VIEW IF EXISTS user_summary_by_statement_type;

CREATE SQL SECURITY INVOKER VIEW user_summary_by_statement_type AS
SELECT user,
       SUBSTRING_INDEX(event_name, '/', -1) AS statement,
       count_star AS count,
       format_time(sum_timer_wait) AS total_latency,
       format_time(max_timer_wait) AS max_latency,
       format_time(sum_lock_time) AS lock_latency,
       sum_rows_sent AS rows_sent,
       sum_rows_examined AS rows_examined,
       sum_rows_affected AS rows_affected,
       sum_no_index_used + sum_no_good_index_used AS full_scans
  FROM performance_schema.events_statements_summary_by_user_by_event_name
 WHERE user IS NOT NULL
   AND sum_timer_wait != 0
 ORDER BY user, sum_timer_wait DESC;

/*
 * View: user_summary_by_statement_type_raw
 *
 * Summarizes the types of statements executed by each user
 *
 * mysql> select * from user_summary_by_statement_type_raw;
 * +------+----------------------+--------+-----------------+----------------+----------------+-----------+---------------+---------------+------------+
 * | user | statement            | count  | total_latency   | max_latency    | lock_latency   | rows_sent | rows_examined | rows_affected | full_scans |
 * +------+----------------------+--------+-----------------+----------------+----------------+-----------+---------------+---------------+------------+
 * | root | create_view          |   2110 | 312717366332000 |   463578029000 |  1432355000000 |         0 |             0 |             0 |          0 |
 * | root | select               |    177 |  41115690428000 | 28827579292000 |   858709000000 |      5254 |        157437 |             0 |         83 |
 * | root | stmt                 |   6645 |  15305389969000 |   491780297000 |              0 |         0 |             0 |          7951 |          0 |
 * | root | call_procedure       |     17 |   4783806053000 |  1016083397000 |    37936000000 |         0 |             0 |            19 |          0 |
 * | root | create_table         |     19 |   3035120946000 |   431706815000 |              0 |         0 |             0 |             0 |          0 |
 * ...
 * +------+----------------------+--------+-----------------+----------------+----------------+-----------+---------------+---------------+------------+
 * 41 rows in set (0.01 sec)
 * 
 * Versions: 5.6.3+
 */

DROP VIEW IF EXISTS user_summary_by_statement_type_raw;

CREATE SQL SECURITY INVOKER VIEW user_summary_by_statement_type_raw AS
SELECT user,
       SUBSTRING_INDEX(event_name, '/', -1) AS statement,
       count_star AS count,
       sum_timer_wait AS total_latency,
       max_timer_wait AS max_latency,
       sum_lock_time AS lock_latency,
       sum_rows_sent AS rows_sent,
       sum_rows_examined AS rows_examined,
       sum_rows_affected AS rows_affected,
       sum_no_index_used + sum_no_good_index_used AS full_scans
  FROM performance_schema.events_statements_summary_by_user_by_event_name
 WHERE user IS NOT NULL
   AND sum_timer_wait != 0
 ORDER BY user, sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/user_summary_by_stages.sql
   ===================================================================
*/
  
  
/*
 * View: user_summary_by_stages
 *
 * Summarizes stages by user, ordered by user and total latency per stage
 * 
 * mysql> select * from user_summary_by_stages;
 * +------+-------------------------------------+--------+-------------+-----------+
 * | user | event_name                          | count  | wait_sum    | wait_avg  |
 * +------+-------------------------------------+--------+-------------+-----------+
 * | root | stage/sql/System lock               |   9230 | 00:03:11.40 | 20.74 ms  |
 * | root | stage/sql/Opening tables            | 534362 | 00:01:36.18 | 180.00 us |
 * | root | stage/sql/checking permissions      |  22119 | 31.84 s     | 1.44 ms   |
 * | root | stage/sql/Creating sort index       |    307 | 30.26 s     | 98.57 ms  |
 * | root | stage/sql/creating table            |     22 | 3.59 s      | 163.13 ms |
 * ...
 * +------+-------------------------------------+--------+-------------+-----------+
 * 27 rows in set (0.00 sec)
 * 
 * Versions: 5.6.3+
 */

DROP VIEW IF EXISTS user_summary_by_stages;

CREATE SQL SECURITY INVOKER VIEW user_summary_by_stages AS
SELECT user, event_name,
       count_star AS count,
       format_time(sum_timer_wait) AS wait_sum, 
       format_time(avg_timer_wait) AS wait_avg 
  FROM performance_schema.events_stages_summary_by_user_by_event_name
 WHERE user IS NOT NULL 
   AND sum_timer_wait != 0 
 ORDER BY user, sum_timer_wait DESC;

/*
 * View: user_summary_by_stages_raw
 *
 * Summarizes stages by user, ordered by user and total latency per stage
 * 
 * mysql> select * from user_summary_by_stages_raw;
 * +------+-------------------------------------+--------+-----------------+--------------+
 * | user | event_name                          | count  | wait_sum        | wait_avg     |
 * +------+-------------------------------------+--------+-----------------+--------------+
 * | root | stage/sql/System lock               |   9231 | 191395549684000 |  20733999000 |
 * | root | stage/sql/Opening tables            | 534704 |  96185600966000 |    179885000 |
 * | root | stage/sql/checking permissions      |  22123 |  31840645121000 |   1439255000 |
 * | root | stage/sql/Creating sort index       |    308 |  30260103983000 |  98247090000 |
 * | root | stage/sql/creating table            |     22 |   3588876152000 | 163130734000 |
 * ...
 * +------+-------------------------------------+--------+-----------------+--------------+
 * 27 rows in set (0.00 sec)
 * 
 * Versions: 5.6.3+
 */

DROP VIEW IF EXISTS user_summary_by_stages_raw;

CREATE SQL SECURITY INVOKER VIEW user_summary_by_stages_raw AS
SELECT user, event_name,
       count_star AS count,
       sum_timer_wait AS wait_sum, 
       avg_timer_wait AS wait_avg 
  FROM performance_schema.events_stages_summary_by_user_by_event_name
 WHERE user IS NOT NULL 
   AND sum_timer_wait != 0 
 ORDER BY user, sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/wait_classes_global_by_avg_latency.sql
   ===================================================================
*/
  
  
/*
 * View: wait_classes_global_by_avg_latency
 * 
 * Lists the top wait classes by average latency, ignoring idle (this may be very large)
 *
 * mysql> select * from wait_classes_global_by_avg_latency where event_class != 'idle';
 * +-------------------+--------------+---------------+-------------+-------------+-------------+
 * | event_class       | total_events | total_latency | min_latency | avg_latency | max_latency |
 * +-------------------+--------------+---------------+-------------+-------------+-------------+
 * | wait/io/file      |       543123 | 44.60 s       | 19.44 ns    | 82.11 µs    | 4.21 s      |
 * | wait/io/table     |        22002 | 766.60 ms     | 148.72 ns   | 34.84 µs    | 44.97 ms    |
 * | wait/io/socket    |        79613 | 967.17 ms     | 0 ps        | 12.15 µs    | 27.10 ms    |
 * | wait/lock/table   |        35409 | 18.68 ms      | 65.45 ns    | 527.51 ns   | 969.88 µs   |
 * | wait/synch/rwlock |        37935 | 4.61 ms       | 21.38 ns    | 121.61 ns   | 34.65 µs    |
 * | wait/synch/mutex  |       390622 | 18.60 ms      | 19.44 ns    | 47.61 ns    | 10.32 µs    |
 * +-------------------+--------------+---------------+-------------+-------------+-------------+
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS wait_classes_global_by_avg_latency;

CREATE SQL SECURITY INVOKER VIEW wait_classes_global_by_avg_latency AS
SELECT SUBSTRING_INDEX(event_name,'/', 3) event_class,
       SUM(COUNT_STAR) total_events,
       format_time(CAST(SUM(sum_timer_wait) AS UNSIGNED)) total_latency,
       format_time(MIN(min_timer_wait)) min_latency,
       format_time(SUM(sum_timer_wait) / SUM(COUNT_STAR)) avg_latency,
       format_time(CAST(MAX(max_timer_wait) AS UNSIGNED)) max_latency
  FROM performance_schema.events_waits_summary_global_by_event_name
 WHERE sum_timer_wait > 0
   AND event_name != 'idle'
 GROUP BY event_class
 ORDER BY SUM(sum_timer_wait) / SUM(COUNT_STAR) DESC;

/*
 * View: wait_classes_global_by_avg_latency_raw
 * 
 * Lists the top wait classes by average latency, ignoring idle (this may be very large)
 *
 * mysql> select * from wait_classes_global_by_avg_latency_raw;
 * +-------------------+--------------+-------------------+-------------+--------------------+------------------+
 * | event_class       | total_events | total_latency     | min_latency | avg_latency        | max_latency      |
 * +-------------------+--------------+-------------------+-------------+--------------------+------------------+
 * | idle              |         4331 | 16044682716000000 |     2000000 | 3704613880397.1369 | 1593550454000000 |
 * | wait/io/file      |        23037 |    20856702551880 |           0 |     905356711.0249 |     350700491310 |
 * | wait/io/table     |       224924 |      719670285750 |      116870 |       3199615.3623 |     208579012460 |
 * | wait/lock/table   |         6972 |        3674766030 |      109330 |        527074.8752 |          8855730 |
 * | wait/synch/rwlock |        11916 |        1273279800 |       37700 |        106854.6324 |          6838780 |
 * | wait/synch/mutex  |      1031881 |       80464286240 |       56550 |         77978.2613 |       2590408470 |
 * +-------------------+--------------+-------------------+-------------+--------------------+------------------+
 * 6 rows in set (0.01 sec)
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS wait_classes_global_by_avg_latency_raw;

CREATE SQL SECURITY INVOKER VIEW wait_classes_global_by_avg_latency_raw AS
SELECT SUBSTRING_INDEX(event_name,'/', 3) event_class,
       SUM(COUNT_STAR) total_events,
       SUM(sum_timer_wait) total_latency,
       MIN(min_timer_wait) min_latency,
       SUM(sum_timer_wait) / SUM(COUNT_STAR) avg_latency,
       MAX(max_timer_wait)max_latency
  FROM performance_schema.events_waits_summary_global_by_event_name
 WHERE sum_timer_wait > 0
   AND event_name != 'idle'
 GROUP BY event_class
 ORDER BY SUM(sum_timer_wait) / SUM(COUNT_STAR) DESC;
 
/* 
   ===================================================================
   Contents from file: views/p_s/wait_classes_global_by_latency.sql
   ===================================================================
*/
  
  
/*
 * View: wait_classes_global_by_latency
 * 
 * Lists the top wait classes by total latency, ignoring idle (this may be very large)
 *
 * mysql> select * from wait_classes_global_by_latency;
 * +-------------------+--------------+---------------+-------------+-------------+-------------+
 * | event_class       | total_events | total_latency | min_latency | avg_latency | max_latency |
 * +-------------------+--------------+---------------+-------------+-------------+-------------+
 * | wait/io/file      |       550470 | 46.01 s       | 19.44 ns    | 83.58 µs    | 4.21 s      |
 * | wait/io/socket    |       228833 | 2.71 s        | 0 ps        | 11.86 µs    | 29.93 ms    |
 * | wait/io/table     |        64063 | 1.89 s        | 99.79 ns    | 29.43 µs    | 68.07 ms    |
 * | wait/lock/table   |        76029 | 47.19 ms      | 65.45 ns    | 620.74 ns   | 969.88 µs   |
 * | wait/synch/mutex  |       635925 | 34.93 ms      | 19.44 ns    | 54.93 ns    | 107.70 µs   |
 * | wait/synch/rwlock |        61287 | 7.62 ms       | 21.38 ns    | 124.37 ns   | 34.65 µs    |
 * +-------------------+--------------+---------------+-------------+-------------+-------------+
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS wait_classes_global_by_latency;

CREATE SQL SECURITY INVOKER VIEW wait_classes_global_by_latency AS
SELECT SUBSTRING_INDEX(event_name,'/', 3) event_class, 
       SUM(COUNT_STAR) total_events,
       format_time(SUM(sum_timer_wait)) total_latency,
       format_time(MIN(min_timer_wait)) min_latency,
       format_time(SUM(sum_timer_wait) / SUM(COUNT_STAR)) avg_latency,
       format_time(MAX(max_timer_wait)) max_latency
  FROM performance_schema.events_waits_summary_global_by_event_name
 WHERE sum_timer_wait > 0
   AND event_name != 'idle'
 GROUP BY SUBSTRING_INDEX(event_name,'/', 3) 
 ORDER BY SUM(sum_timer_wait) DESC;

/*
 * View: wait_classes_global_by_latency_raw
 * 
 * Lists the top wait classes by total latency, ignoring idle (this may be very large)
 *
 * mysql> SELECT * FROM wait_classes_global_by_latency_raw;
 * +-------------------+--------------+----------------+-------------+----------------+--------------+
 * | event_class       | total_events | total_latency  | min_latency | avg_latency    | max_latency  |
 * +-------------------+--------------+----------------+-------------+----------------+--------------+
 * | wait/io/file      |        29468 | 27100905420290 |           0 | 919672370.7170 | 350700491310 |
 * | wait/io/table     |       224924 |   719670285750 |      116870 |   3199615.3623 | 208579012460 |
 * | wait/synch/mutex  |      1532036 |   118515948070 |       56550 |     77358.4616 |   2590408470 |
 * | wait/io/socket    |         1193 |    10677541030 |           0 |   8950160.1257 |    287760330 |
 * | wait/lock/table   |         6972 |     3674766030 |      109330 |    527074.8752 |      8855730 |
 * | wait/synch/rwlock |        13646 |     1579833580 |       37700 |    115772.6499 |     28293850 |
 * +-------------------+--------------+----------------+-------------+----------------+--------------+
 * 6 rows in set (0.01 sec)
 *
 * (Example from 5.6.6)
 *
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS wait_classes_global_by_latency_raw;

CREATE SQL SECURITY INVOKER VIEW wait_classes_global_by_latency_raw AS
SELECT SUBSTRING_INDEX(event_name,'/', 3) event_class, 
       SUM(COUNT_STAR) total_events,
       SUM(sum_timer_wait) total_latency,
       MIN(min_timer_wait) min_latency,
       SUM(sum_timer_wait) / SUM(COUNT_STAR) avg_latency,
       MAX(max_timer_wait) max_latency
  FROM performance_schema.events_waits_summary_global_by_event_name
 WHERE sum_timer_wait > 0
   AND event_name != 'idle'
 GROUP BY SUBSTRING_INDEX(event_name,'/', 3) 
 ORDER BY SUM(sum_timer_wait) DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/waits_by_user_by_latency.sql
   ===================================================================
*/
  
  
/*
 * View: waits_by_user_by_latency
 *
 * Lists the top wait events by their total latency, ignoring idle (this may be very large)
 *
 * mysql> select * from waits_global_by_latency limit 5;
 * +--------------------------------------+--------------+---------------+-------------+-------------+
 * | event                                | total_events | total_latency | avg_latency | max_latency |
 * +--------------------------------------+--------------+---------------+-------------+-------------+
 * | wait/io/file/myisam/dfile            |   3623719744 | 00:47:49.09   | 791.70 ns   | 312.96 ms   |
 * | wait/io/table/sql/handler            |     69114944 | 00:44:30.74   | 38.64 us    | 879.49 ms   |
 * | wait/io/file/innodb/innodb_log_file  |     28100261 | 00:37:42.12   | 80.50 us    | 476.00 ms   |
 * | wait/io/socket/sql/client_connection |    200704863 | 00:18:37.81   | 5.57 us     | 1.27 s      |
 * | wait/io/file/innodb/innodb_data_file |      2829403 | 00:08:12.89   | 174.20 us   | 455.22 ms   |
 * +--------------------------------------+--------------+---------------+-------------+-------------+
 * 
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS waits_by_user_by_latency;

CREATE SQL SECURITY INVOKER VIEW waits_by_user_by_latency AS
SELECT user,
       event_name AS event,
       count_star AS total_events,
       wbsys.format_time(sum_timer_wait) AS total_latency,
       wbsys.format_time(avg_timer_wait) AS avg_latency,
       wbsys.format_time(max_timer_wait) AS max_latency
  FROM performance_schema.events_waits_summary_by_user_by_event_name
 WHERE event_name != 'idle'
   AND user IS NOT NULL
   AND sum_timer_wait > 0
 ORDER BY user, sum_timer_wait DESC;

/*
 * View: waits_by_user_by_latency_raw
 *
 * Lists the top wait events by their total latency, ignoring idle (this may be very large)
 *
 * mysql> select * from waits_global_by_latency_raw limit 5;
 * +--------------------------------------+--------------+---------------+-------------+--------------+
 * | event                                | total_events | total_latency | avg_latency | max_latency  |
 * +--------------------------------------+--------------+---------------+-------------+--------------+
 * | wait/io/file/sql/file_parser         |          679 | 3536136351540 |  5207858773 | 129860439800 |
 * | wait/io/file/innodb/innodb_data_file |          195 |  848170566100 |  4349592637 | 350700491310 |
 * | wait/io/file/sql/FRM                 |         1355 |  400428476500 |   295518990 |  44823120940 |
 * | wait/io/file/innodb/innodb_log_file  |           20 |   54298899070 |  2714944765 |  30108124800 |
 * | wait/io/file/mysys/charset           |            3 |   24244722970 |  8081574072 |  24151547420 |
 * +--------------------------------------+--------------+---------------+-------------+--------------+
 * 5 rows in set (0.01 sec)
 * 
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS waits_by_user_by_latency_raw;

CREATE SQL SECURITY INVOKER VIEW waits_by_user_by_latency_raw AS
SELECT user,
       event_name AS event,
       count_star AS total_events,
       sum_timer_wait AS total_latency,
       avg_timer_wait AS avg_latency,
       max_timer_wait AS max_latency
  FROM performance_schema.events_waits_summary_by_user_by_event_name
 WHERE event_name != 'idle'
   AND user IS NOT NULL
   AND sum_timer_wait > 0
 ORDER BY user, sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: views/p_s/waits_global_by_latency.sql
   ===================================================================
*/
  
  
/*
 * View: waits_global_by_latency
 *
 * Lists the top wait events by their total latency, ignoring idle (this may be very large)
 *
 * mysql> select * from waits_global_by_latency limit 5;
 * +--------------------------------------+--------------+---------------+-------------+-------------+
 * | event                                | total_events | total_latency | avg_latency | max_latency |
 * +--------------------------------------+--------------+---------------+-------------+-------------+
 * | wait/io/file/myisam/dfile            |   3623719744 | 00:47:49.09   | 791.70 ns   | 312.96 ms   |
 * | wait/io/table/sql/handler            |     69114944 | 00:44:30.74   | 38.64 us    | 879.49 ms   |
 * | wait/io/file/innodb/innodb_log_file  |     28100261 | 00:37:42.12   | 80.50 us    | 476.00 ms   |
 * | wait/io/socket/sql/client_connection |    200704863 | 00:18:37.81   | 5.57 us     | 1.27 s      |
 * | wait/io/file/innodb/innodb_data_file |      2829403 | 00:08:12.89   | 174.20 us   | 455.22 ms   |
 * +--------------------------------------+--------------+---------------+-------------+-------------+
 * 
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS waits_global_by_latency;

CREATE SQL SECURITY INVOKER VIEW waits_global_by_latency AS
SELECT event_name AS event,
       count_star AS total_events,
       wbsys.format_time(sum_timer_wait) AS total_latency,
       wbsys.format_time(avg_timer_wait) AS avg_latency,
       wbsys.format_time(max_timer_wait) AS max_latency
  FROM performance_schema.events_waits_summary_global_by_event_name
 WHERE event_name != 'idle'
   AND sum_timer_wait > 0
 ORDER BY sum_timer_wait DESC;

/*
 * View: waits_global_by_latency_raw
 *
 * Lists the top wait events by their total latency, ignoring idle (this may be very large)
 *
 * mysql> select * from waits_global_by_latency_raw limit 5;
 * +--------------------------------------+--------------+---------------+-------------+--------------+
 * | event                                | total_events | total_latency | avg_latency | max_latency  |
 * +--------------------------------------+--------------+---------------+-------------+--------------+
 * | wait/io/file/sql/file_parser         |          679 | 3536136351540 |  5207858773 | 129860439800 |
 * | wait/io/file/innodb/innodb_data_file |          195 |  848170566100 |  4349592637 | 350700491310 |
 * | wait/io/file/sql/FRM                 |         1355 |  400428476500 |   295518990 |  44823120940 |
 * | wait/io/file/innodb/innodb_log_file  |           20 |   54298899070 |  2714944765 |  30108124800 |
 * | wait/io/file/mysys/charset           |            3 |   24244722970 |  8081574072 |  24151547420 |
 * +--------------------------------------+--------------+---------------+-------------+--------------+
 * 5 rows in set (0.01 sec)
 * 
 * Versions: 5.5+
 */

DROP VIEW IF EXISTS waits_global_by_latency_raw;

CREATE SQL SECURITY INVOKER VIEW waits_global_by_latency_raw AS
SELECT event_name AS event,
       count_star AS total_events,
       sum_timer_wait AS total_latency,
       avg_timer_wait AS avg_latency,
       max_timer_wait AS max_latency
  FROM performance_schema.events_waits_summary_global_by_event_name
 WHERE event_name != 'idle'
   AND sum_timer_wait > 0
 ORDER BY sum_timer_wait DESC;
/* 
   ===================================================================
   Contents from file: after_setup.sql
   ===================================================================
*/
  
  
SET @@sql_log_bin = @sql_log_bin;
